"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var GameObject_1 = require("./GameObject");
var Component_1 = require("./Component");
var GLMath_1 = require("./math/GLMath");
var AmbientType;
(function (AmbientType) {
    AmbientType[AmbientType["Background"] = 0] = "Background";
    AmbientType[AmbientType["AmbientColor"] = 1] = "AmbientColor";
})(AmbientType = exports.AmbientType || (exports.AmbientType = {}));
var ClearType;
(function (ClearType) {
    ClearType[ClearType["Background"] = 0] = "Background";
    ClearType[ClearType["Skybox"] = 1] = "Skybox";
})(ClearType = exports.ClearType || (exports.ClearType = {}));
var ProjectionType;
(function (ProjectionType) {
    ProjectionType[ProjectionType["perspective"] = 0] = "perspective";
    ProjectionType[ProjectionType["orthographic"] = 1] = "orthographic";
})(ProjectionType = exports.ProjectionType || (exports.ProjectionType = {}));
var Camera = /** @class */ (function (_super) {
    __extends(Camera, _super);
    function Camera() {
        var _this = _super.call(this) || this;
        _this.enabled = true;
        _this.order = 0;
        _this.m_fov = 60;
        _this.m_projDirty = false;
        _this.orthosize = 10.0;
        _this.m_worldMtxCalculated = false;
        _this.m_background = GLMath_1.vec4.zero;
        _this.m_ambientColor = GLMath_1.glmath.vec4(0.1, 0.1, 0.1, 1.0);
        _this.m_ambientType = AmbientType.AmbientColor;
        _this.m_clearType = ClearType.Background;
        _this.m_dataTrsDirty = true;
        _this.m_dataProjDirty = true;
        _this.ambientDataDirty = true;
        _this.worldRHS = true;
        _this.m_projMtx = GLMath_1.mat4.perspectiveFoV(60, 1, 0.01, 100);
        _this.m_projectionType = ProjectionType.perspective;
        return _this;
    }
    Object.defineProperty(Camera.prototype, "isDataTrsDirty", {
        get: function () {
            return this.m_dataTrsDirty || this.transform.isDirty;
        },
        set: function (v) {
            this.m_dataTrsDirty = v;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Camera.prototype, "isDataProjDirty", {
        get: function () {
            return this.m_dataProjDirty;
        },
        set: function (v) {
            this.m_dataProjDirty = v;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Camera.prototype, "far", {
        get: function () {
            return this.m_far;
        },
        set: function (v) {
            if (this.m_far == v)
                return;
            this.m_far = v;
            this.m_projDirty = true;
            this.m_dataProjDirty = true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Camera.prototype, "near", {
        get: function () {
            return this.m_near;
        },
        set: function (v) {
            if (this.m_near == v)
                return;
            this.m_near = v;
            this.m_projDirty = true;
            this.m_dataProjDirty = true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Camera.prototype, "fov", {
        get: function () {
            return this.m_fov;
        },
        set: function (v) {
            if (this.m_fov == v)
                return;
            this.m_fov = v;
            this.m_projDirty = true;
            this.m_dataProjDirty = true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Camera.prototype, "aspect", {
        get: function () {
            return this.m_aspectratio;
        },
        set: function (v) {
            if (v == this.m_aspectratio)
                return;
            this.m_aspectratio = v;
            this.m_projDirty = true;
            this.m_dataProjDirty = true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Camera.prototype, "ambientColor", {
        get: function () {
            return this.m_ambientColor;
        },
        set: function (v) {
            this.m_ambientColor = v;
            this.ambientDataDirty = true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Camera.prototype, "background", {
        get: function () {
            return this.m_background;
        },
        set: function (v) {
            this.m_background = v;
            if (this.m_ambientType == AmbientType.Background) {
                this.ambientDataDirty = true;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Camera.prototype, "skybox", {
        get: function () {
            return this.m_skybox;
        },
        set: function (skybox) {
            this.m_skybox = skybox;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Camera.prototype, "clearType", {
        get: function () {
            return this.m_clearType;
        },
        set: function (t) {
            this.m_clearType = t;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Camera.prototype, "ambientType", {
        get: function () {
            return this.m_ambientType;
        },
        set: function (t) {
            if (this.m_ambientType == t)
                return;
            this.m_ambientType = t;
            this.ambientDataDirty = true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Camera.prototype, "WorldMatrix", {
        /** View matrix of camera */
        get: function () {
            var trs = this.transform;
            if (!this.m_worldMtxCalculated && trs.isDirty) {
                this.m_worldMtx = GLMath_1.mat4.coordCvt(trs.position, trs.worldForward, trs.worldUp);
                this.m_dataTrsDirty = true;
                this.m_worldMtxCalculated = true;
            }
            return this.m_worldMtx;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Camera.prototype, "ProjMatrix", {
        /** Projection matrix of camera */
        get: function () {
            if (this.m_projDirty) {
                this.m_projMtx = GLMath_1.mat4.perspectiveFoV(this.m_fov, this.m_aspectratio, this.m_near, this.m_far);
                this.m_projDirty = false;
                this.m_dataProjDirty = true;
            }
            return this.m_projMtx;
        },
        enumerable: true,
        configurable: true
    });
    Camera.prototype.onUpdate = function (scene) {
        scene.mainCamera = this;
        this.m_worldMtxCalculated = false;
    };
    Camera.persepctive = function (gobj, fov, aspectratio, near, far) {
        var camera = new Camera();
        camera.m_fov = fov;
        camera.m_aspectratio = aspectratio;
        camera.m_near = near;
        camera.m_far = far;
        if (gobj == null) {
            gobj = new GameObject_1.GameObject();
        }
        gobj.addComponent(camera);
        camera.m_projMtx = GLMath_1.mat4.perspectiveFoV(fov, aspectratio, near, far);
        camera.m_projectionType = ProjectionType.perspective;
        var trs = gobj.transform;
        camera.m_worldMtx = GLMath_1.mat4.coordCvt(trs.localPosition, trs.worldForward, trs.worldUp);
        return camera;
    };
    Camera.orthographic = function (gobj, size, aspectratio, near, far) {
        var camera = new Camera();
        camera.m_aspectratio = aspectratio;
        camera.m_near = near;
        camera.m_far = far;
        if (gobj == null) {
            gobj = new GameObject_1.GameObject();
        }
        gobj.addComponent(camera);
        camera.orthosize = size;
        var w = size * aspectratio;
        camera.m_projMtx = GLMath_1.mat4.orthographic(w, size, near, far);
        camera.m_projectionType = ProjectionType.orthographic;
        var trs = gobj.transform;
        camera.m_worldMtx = GLMath_1.mat4.coordCvt(trs.localPosition, trs.worldForward, trs.worldUp);
        return camera;
    };
    return Camera;
}(Component_1.Component));
exports.Camera = Camera;
