"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Mesh_1 = require("./Mesh");
var DynamicMesh = /** @class */ (function (_super) {
    __extends(DynamicMesh, _super);
    function DynamicMesh(name) {
        return _super.call(this, name, true) || this;
    }
    DynamicMesh.prototype.refreshMeshBuffer = function (glctx) {
        if (this.m_bufferInited)
            return;
        var gl = glctx.gl;
        if (this.m_seperatedBuffer) {
            var vertexdesc = this.vertexDesc;
            var posdesc = vertexdesc.position;
            if (posdesc != null) {
                var buffer = gl.createBuffer();
                gl.bindBuffer(gl.ARRAY_BUFFER, buffer);
                var datapos = this.m_dataPosition;
                if (datapos)
                    gl.bufferData(gl.ARRAY_BUFFER, datapos, gl.DYNAMIC_DRAW);
                this.bufferVertices = buffer;
                posdesc.offset = 0;
            }
            var uvdesc = vertexdesc.uv;
            if (uvdesc != null) {
                var buffer = gl.createBuffer();
                gl.bindBuffer(gl.ARRAY_BUFFER, buffer);
                var datauv = this.m_dataUV;
                if (datauv)
                    gl.bufferData(gl.ARRAY_BUFFER, datauv, gl.DYNAMIC_DRAW);
                this.bufferUV = buffer;
                posdesc.offset = 0;
            }
            var normaldesc = vertexdesc.normal;
            if (normaldesc != null) {
                var buffer = gl.createBuffer();
                gl.bindBuffer(gl.ARRAY_BUFFER, buffer);
                var datanormal = this.m_dataNormal;
                if (datanormal)
                    gl.bufferData(gl.ARRAY_BUFFER, datanormal, gl.DYNAMIC_DRAW);
                this.bufferNormal = buffer;
                posdesc.offset = 0;
            }
            gl.bindBuffer(gl.ARRAY_BUFFER, null);
            //indices
            var dataIndices = this.m_dataIndices;
            var hasIndices = dataIndices != null && dataIndices.length != 0;
            if (hasIndices) {
                var buffer = gl.createBuffer();
                gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, buffer);
                gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, dataIndices, gl.STATIC_DRAW);
                gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, null);
                this.bufferIndices = buffer;
            }
        }
        else {
            throw new Error('dynamic buffer only support seperated buffer currently');
        }
        this.m_bufferInited = true;
    };
    DynamicMesh.prototype.uploadDataBufferPosition = function (gl, databuffer) {
        if (databuffer === void 0) { databuffer = null; }
        var buffer = this.bufferVertices;
        if (buffer == null)
            throw new Error('buffer is null');
        var data = null;
        if (databuffer != null && databuffer != this.m_dataPosition) {
            if (databuffer.byteLength > this.m_dataPosition.byteLength)
                throw new Error('buffer overflow');
            data = databuffer;
        }
        else {
            data = this.m_dataPosition;
        }
        gl.bindBuffer(gl.ARRAY_BUFFER, buffer);
        gl.bufferData(gl.ARRAY_BUFFER, data, gl.DYNAMIC_DRAW);
        gl.bindBuffer(gl.ARRAY_BUFFER, null);
    };
    DynamicMesh.prototype.uploadDataBufferUV = function (gl, databuffer) {
        if (databuffer === void 0) { databuffer = null; }
        var buffer = this.bufferUV;
        if (buffer == null)
            throw new Error('uvbuffer is null');
        var data = null;
        if (databuffer != null && databuffer != this.m_dataUV) {
            if (databuffer.byteLength > this.m_dataUV.byteLength)
                throw new Error('uvbuffer overflow');
            data = databuffer;
        }
        else {
            data = this.m_dataUV;
        }
        gl.bindBuffer(gl.ARRAY_BUFFER, buffer);
        gl.bufferData(gl.ARRAY_BUFFER, data, gl.DYNAMIC_DRAW);
        gl.bindBuffer(gl.ARRAY_BUFFER, null);
    };
    DynamicMesh.prototype.uploadDataBufferNormal = function (gl, databuffer) {
        if (databuffer === void 0) { databuffer = null; }
        var buffer = this.bufferNormal;
        if (buffer == null)
            throw new Error('normal buffer is null');
        var data = null;
        if (databuffer != null && databuffer != this.m_dataNormal) {
            if (databuffer.byteLength > this.m_dataNormal.byteLength)
                throw new Error('normal buffer overflow');
            data = databuffer;
        }
        else {
            data = this.m_dataNormal;
        }
        gl.bindBuffer(gl.ARRAY_BUFFER, buffer);
        gl.bufferData(gl.ARRAY_BUFFER, data, gl.DYNAMIC_DRAW);
        gl.bindBuffer(gl.ARRAY_BUFFER, null);
    };
    DynamicMesh.prototype.uploadDataBufferIndices = function (gl, databuffer, dynamicdraw) {
        if (databuffer === void 0) { databuffer = null; }
        if (dynamicdraw === void 0) { dynamicdraw = true; }
        var buffer = this.bufferIndices;
        if (buffer == null)
            throw new Error('normal buffer is null');
        var data = null;
        if (databuffer != null && databuffer != this.m_dataIndices) {
            if (databuffer.byteLength > this.m_dataIndices.byteLength)
                throw new Error('normal buffer overflow');
            data = databuffer;
        }
        else {
            data = this.m_dataIndices;
        }
        gl.bindBuffer(gl.ARRAY_BUFFER, buffer);
        gl.bufferData(gl.ARRAY_BUFFER, data, dynamicdraw ? gl.DYNAMIC_DRAW : gl.STATIC_DRAW);
        gl.bindBuffer(gl.ARRAY_BUFFER, null);
    };
    return DynamicMesh;
}(Mesh_1.Mesh));
exports.DynamicMesh = DynamicMesh;
