"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Transform_1 = require("./Transform");
var GameObject = /** @class */ (function () {
    function GameObject(name) {
        this.active = true;
        this.name = name;
        this.transform = new Transform_1.Transform(this);
    }
    Object.defineProperty(GameObject.prototype, "render", {
        get: function () {
            return this.m_render;
        },
        set: function (v) {
            v['m_object'] = this;
            this.m_render = v;
        },
        enumerable: true,
        configurable: true
    });
    GameObject.prototype.addRender = function (t) {
        var render = new t();
        render['m_object'] = this;
        this.m_render = render;
        return render;
    };
    GameObject.prototype.update = function (scene) {
        var comp = this.components;
        if (comp != null) {
            for (var i = 0, len = comp.length; i < len; i++) {
                var c = comp[i];
                if (c.onUpdate != null) {
                    c.onUpdate(scene);
                }
            }
        }
        var trs = this.transform;
        var trsdirty = trs.isDirty;
        var children = this.transform.children;
        if (children != null) {
            for (var i = 0, len = children.length; i < len; i++) {
                var g = children[i].gameobject;
                g.transform.setObjMatrixDirty(trsdirty);
                g.update(scene);
            }
        }
    };
    GameObject.prototype.addComponent = function (c) {
        var comps = this.components;
        if (comps == null) {
            comps = [];
            this.components = comps;
        }
        var index = comps.indexOf(c);
        if (index >= 0)
            return;
        c.gameobject = this;
        if (c.onStart != null)
            c.onStart();
        comps.push(c);
    };
    GameObject.prototype.getComponent = function (t) {
        var comps = this.components;
        for (var i = 0, len = comps.length; i < len; i++) {
            if (comps[i] instanceof t)
                return comps[i];
        }
        return null;
    };
    GameObject.prototype.getChildByName = function (name) {
        var children = this.transform.children;
        if (children == null)
            return null;
        for (var i = 0, len = children.length; i < len; i++) {
            var ct = children[i].gameobject;
            if (ct.name === name)
                return ct;
            var cc = ct.getChildByName(name);
            if (cc != null)
                return cc;
        }
        return null;
    };
    return GameObject;
}());
exports.GameObject = GameObject;
