"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var GLMath_1 = require("./math/GLMath");
var InputSnapShot = /** @class */ (function () {
    function InputSnapShot() {
        this.key = {};
        this.keyDown = {};
        this.keyUp = {};
        this.hasKeyPressEvent = false;
        this.mouse = new Array(4);
        this.mouseDown = new Array(4);
        this.mouseUp = new Array(4);
        /**
         * [posx,posy,deltax,deltay]
         */
        this.mousepos = new GLMath_1.vec4();
        this.mousewheel = false;
        this.mousewheelDelta = 0;
        this.mouseMove = false;
        this.deltaTime = 0;
    }
    InputSnapShot.prototype.getKeyDown = function (key) {
        return this.keyDown[key] == true;
    };
    InputSnapShot.prototype.getKeyUp = function (key) {
        return this.keyUp[key] == true;
    };
    InputSnapShot.prototype.getKey = function (key) {
        return this.key[key];
    };
    InputSnapShot.prototype.getMouseBtn = function (btn) {
        return this.mouse[btn];
    };
    InputSnapShot.prototype.getMouseDown = function (btn) {
        return this.mouseDown[btn];
    };
    InputSnapShot.prototype.getMouseUp = function (btn) {
        return this.mouseUp[btn];
    };
    return InputSnapShot;
}());
exports.InputSnapShot = InputSnapShot;
var InputCache = /** @class */ (function () {
    function InputCache() {
        this.mousepos = new GLMath_1.vec4();
        this.mousebtn = new Array(4);
        this.mousedown = new Array(4);
        this.mouseup = new Array(4);
        this.mousewheel = false;
        this.mousewheelDelta = 0;
        this.keyDown = {};
        this.keyUp = {};
        this.m_keydirty = false;
        this.m_mousedirty = false;
        this.m_mousePosDirty = false;
        this.m_shotMouseDownFalse = false;
        this.m_shotMouseUpFalse = false;
    }
    InputCache.prototype.setKeyDown = function (e) {
        var kp = this.keyPress;
        if (kp == null) {
            kp = {};
            this.keyPress = kp;
        }
        kp[e.key] = true;
        this.keyDown[e.key] = true;
        this.m_keydirty = true;
    };
    InputCache.prototype.setKeyUp = function (e) {
        var kp = this.keyPress;
        if (kp == null) {
            kp = {};
            this.keyPress = kp;
        }
        kp[e.key] = false;
        this.keyUp[e.key] = true;
        this.m_keydirty = true;
    };
    InputCache.prototype.setMousePos = function (e) {
        var mp = this.mousepos;
        var x = e.offsetX;
        var y = e.offsetY;
        mp.z = x - mp.x;
        mp.w = y - mp.y;
        mp.x = x;
        mp.y = y;
        this.m_mousePosDirty = true;
    };
    InputCache.prototype.setButtonDown = function (e) {
        var btn = e.button;
        this.mousebtn[btn] = true;
        this.mousedown[btn] = true;
        this.m_mousedirty = true;
    };
    InputCache.prototype.setButtonUp = function (e) {
        var btn = e.button;
        this.mousebtn[btn] = false;
        this.mouseup[btn] = true;
        this.m_mousedirty = true;
    };
    InputCache.prototype.setMouseWheel = function (delta) {
        this.mousewheel = true;
        this.mousewheelDelta += delta;
    };
    InputCache.prototype.reset = function () {
        this.mousewheel = false;
        this.mousewheelDelta = 0;
        if (this.m_mousedirty) {
            var mouseup = this.mouseup;
            var mousedown = this.mousedown;
            mouseup.fill(false);
            mousedown.fill(false);
            this.m_mousedirty = false;
        }
        if (this.m_keydirty) {
            this.keyDown = [];
            this.keyUp = [];
            this.m_keydirty = false;
            this.keyPress = {};
        }
    };
    InputCache.prototype.applytoSnapShot = function (shot) {
        var mousePosDirty = this.m_mousePosDirty;
        shot.mouseMove = mousePosDirty;
        if (mousePosDirty) {
            this.m_mousePosDirty = false;
            var shotmousepos = shot.mousepos;
            shotmousepos.set(this.mousepos);
        }
        shot.mousewheelDelta = this.mousewheelDelta;
        shot.mousewheel = this.mousewheelDelta != 0 && this.mousewheel;
        var smdown = shot.mouseDown;
        var smup = shot.mouseUp;
        var smouse = shot.mouse;
        if (this.m_mousedirty) {
            var cmdown = this.mousedown;
            for (var i = 0; i < 4; i++) {
                smdown[i] = cmdown[i];
            }
            var cmup = this.mouseup;
            for (var i = 0; i < 4; i++) {
                smup[i] = cmup[i];
            }
            this.m_shotMouseDownFalse = false;
            this.m_shotMouseUpFalse = false;
            var cmouse = this.mousebtn;
            for (var i = 0; i < 4; i++) {
                smouse[i] = cmouse[i];
            }
        }
        else {
            if (!this.m_shotMouseDownFalse) {
                smdown.fill(false);
                this.m_shotMouseDownFalse = true;
            }
            if (!this.m_shotMouseUpFalse) {
                smup.fill(false);
                this.m_shotMouseUpFalse = true;
            }
        }
        var keydirty = this.m_keydirty;
        shot.hasKeyPressEvent = keydirty;
        if (keydirty) {
            //key press
            var skey = shot.key;
            var ckey = this.keyPress;
            for (var k in ckey) {
                skey[k] = ckey[k];
            }
            shot.keyDown = this.keyDown;
            shot.keyUp = this.keyUp;
        }
        else {
            shot.keyDown = null;
            shot.keyUp = null;
        }
    };
    return InputCache;
}());
var Input = /** @class */ (function () {
    function Input() {
    }
    Input.init = function (canvas) {
        if (Input.s_inited)
            return;
        Input.s_canvas = canvas;
        Input.regEventListener();
        Input.s_inited = true;
    };
    Input.prototype.release = function () {
        Input.removeEventListener();
    };
    Input.regEventListener = function () {
        window.addEventListener('keydown', Input.onEvtKeyDown);
        window.addEventListener('keyup', Input.onEvtKeyUp);
        var canvas = Input.s_canvas;
        canvas.addEventListener('mousemove', Input.onEvtMouseMove);
        canvas.addEventListener('mousedown', Input.onEvtMouseDown);
        canvas.addEventListener('mouseup', Input.onEvtMouseUp);
        canvas.addEventListener('mousewheel', Input.onEvtMouseWheel);
    };
    Input.removeEventListener = function () {
        window.removeEventListener('keydown', Input.onEvtKeyDown);
        window.removeEventListener('keyup', Input.onEvtKeyUp);
        var canvas = Input.s_canvas;
        canvas.removeEventListener('mousemove', Input.onEvtMouseMove);
        canvas.removeEventListener('mousedown', Input.onEvtMouseDown);
        canvas.removeEventListener('mouseup', Input.onEvtMouseUp);
    };
    Input.onEvtKeyDown = function (e) {
        var c = Input.s_cache;
        c.setKeyDown(e);
    };
    Input.onEvtKeyUp = function (e) {
        var c = Input.s_cache;
        c.setKeyUp(e);
    };
    Input.onEvtMouseMove = function (e) {
        var c = Input.s_cache;
        c.setMousePos(e);
    };
    Input.onEvtMouseDown = function (e) {
        var c = Input.s_cache;
        c.setButtonDown(e);
    };
    Input.onEvtMouseUp = function (e) {
        var c = Input.s_cache;
        c.setButtonUp(e);
    };
    Input.onEvtMouseWheel = function (e) {
        var c = Input.s_cache;
        c.setMouseWheel(e.deltaY);
    };
    Input.onFrame = function (dt) {
        var c = Input.s_cache;
        var snpashot = Input.snapshot;
        snpashot.deltaTime = dt;
        c.applytoSnapShot(Input.snapshot);
        c.reset();
    };
    Input.s_inited = false;
    Input.s_cache = new InputCache();
    Input.snapshot = new InputSnapShot();
    return Input;
}());
exports.Input = Input;
