/// <reference types="webgl2" />
import { vec4, vec3 } from "./math/GLMath";
import { Shader, ShaderTags } from "./shaderfx/Shader";
import { ShaderOptions } from "./shaderfx/ShaderVariant";
import { Texture } from "./Texture";
import { GLProgram } from "./gl/GLProgram";
export declare type MaterialProperty = {
    type: number;
    value: any;
};
export declare class MaterialPorpertyBlock {
    uniforms: {
        [key: string]: MaterialProperty;
    };
    /** default is null, perperty might be set with @function <setUniformBlock> or @function <setUniformBlockwitName> */
    uniformsBlock: {
        [slot: number]: number;
    };
    private m_program;
    constructor(program?: GLProgram);
    setProgram(program: GLProgram): void;
    clone(): MaterialPorpertyBlock;
    getUniform(name: string): MaterialProperty;
    release(): void;
}
export declare class Material {
    private m_program;
    private m_shader;
    private m_propertyBlock;
    private m_optConfig;
    private m_useVariants;
    name: string;
    private m_shadertags;
    static DEF_TEXID_NUM: number;
    readonly program: GLProgram;
    shaderTags: ShaderTags;
    readonly propertyBlock: MaterialPorpertyBlock;
    setShader(shader: Shader): void;
    constructor(shader?: Shader);
    clone(): Material;
    setColor(name: string, color: vec4): void;
    setTexture(name: string, tex: WebGLTexture | Texture): void;
    setFloat(name: string, val: number): void;
    setVec2(name: string, valx: number, valy: number): void;
    setVec3(name: string, val: vec3): void;
    setVec4(name: string, val: vec4): void;
    /**
     * Set uniform block binding
     * @param name
     * @param index
     */
    setUniformBlockwitName(name: string, index: number): void;
    /**
     * Set uniform block binding
     * @param bufferindex shader uniform index, this value can be queried by @function <getUniformBlockIndex>
     * @param index uniform buffer binded index
     */
    setUniformBlock(bufferindex: number, index: number): void;
    /**
     * Query the uniform blocksindex in current shader program
     * @param name
     */
    getUniformBlockIndex(name: string): number;
    /**
     *
     * @param slotindex
     */
    getUniformBlockName(slotindex: number): string;
    /**
     * Query the uniform block name with uniform block slot index.
     * @todo merge this function to class ShaderProgram
     * @param program
     * @param slotindex
     */
    static programGetUniformBlockName(program: GLProgram, slotindex: number): string;
    /**
     * SetFlag will not switch to new program immediately,
     * setUniform parameters must be called after @property {program} refreshed.
     * @param key
     * @param value
     * @param refresh
     */
    setFlag(key: string, value: string, refresh?: boolean): void;
    /**
     * refresh program after @function <setFlag> called.
     */
    refreshProgram(): GLProgram;
    setFlagNoVerify(options: ShaderOptions): void;
    getFlag(key: string): string;
    private m_applyTexCount;
    apply(gl: WebGL2RenderingContext): void;
    /**
     * TODO clean apply process
     * especially for binded texture
     * @param gl
     */
    clean(gl: WebGL2RenderingContext): void;
    private setUniform;
}
