"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Utility_1 = require("./Utility");
var Texture_1 = require("./Texture");
var ShaderFX_1 = require("./shaderfx/ShaderFX");
var MaterialPorpertyBlock = /** @class */ (function () {
    function MaterialPorpertyBlock(program) {
        if (program == null)
            return;
        if (this.uniforms == null) {
            this.uniforms = {};
        }
        this.setProgram(program);
    }
    MaterialPorpertyBlock.prototype.setProgram = function (program) {
        if (program == this.m_program)
            return;
        if (this.m_program == null) {
            var uinfo_1 = program.UniformsInfo;
            var selfu_1 = this.uniforms;
            for (var uname in uinfo_1) {
                var info = uinfo_1[uname];
                selfu_1[uname] = { type: info.type, value: null };
            }
            this.m_program = program;
            return;
        }
        var uinfo = program.UniformsInfo;
        var selfu = this.uniforms;
        for (var key in selfu) {
            if (uinfo[key] == null) {
                delete selfu[key];
            }
        }
        for (var key in uinfo) {
            var u = selfu[key];
            var up = uinfo[key];
            if (u == null) {
                selfu[key] = { type: up.type, value: null };
            }
            else {
                if (u.type != up.type) {
                    u.type = up.type;
                    u.value = null;
                    //TODO this can be optimized by checking if old value is compatible with new uniform type.
                    //(e.g., Texture object wrap with TEXTURE_CUBE and TEXTURE_2D)
                }
            }
        }
        //sync uniformblocks
        var selfprogrm = this.m_program;
        var selfBlock = this.uniformsBlock;
        var uniformBlock = program.UniformBlock;
        var newblocks = null;
        if (selfBlock != null) {
            for (var slot in selfBlock) {
                var slotindex = selfBlock[slot];
                var blockname = Material.programGetUniformBlockName(selfprogrm, slotindex);
                if (blockname == null)
                    continue;
                var newindex = uniformBlock[blockname];
                if (newindex == null)
                    continue;
                delete selfBlock[slot];
                selfBlock[newindex] = slotindex;
            }
        }
        for (var bname in uniformBlock) {
            if (ShaderFX_1.ShaderFX.isInternalUniformBlockName(bname))
                continue;
            var info = selfprogrm.UniformBlock[bname];
            if (info != null) {
                newblocks[uniformBlock[bname]] = selfBlock[info];
            }
            else {
                newblocks[uniformBlock[bname]] = null;
            }
        }
        this.uniformsBlock = newblocks;
        //sync program
        this.m_program = program;
    };
    MaterialPorpertyBlock.prototype.clone = function () {
        var block = new MaterialPorpertyBlock(null);
        block.m_program = this.m_program;
        block.uniforms = Utility_1.Utility.cloneMap(this.uniforms, function (p) {
            return { type: p.type, value: p.value };
        });
        return block;
    };
    MaterialPorpertyBlock.prototype.getUniform = function (name) {
        return this.uniforms[name];
    };
    MaterialPorpertyBlock.prototype.release = function () {
        this.m_program = null;
        this.uniforms = null;
    };
    return MaterialPorpertyBlock;
}());
exports.MaterialPorpertyBlock = MaterialPorpertyBlock;
var Material = /** @class */ (function () {
    function Material(shader) {
        this.m_useVariants = false;
        this.m_shadertags = null;
        this.m_applyTexCount = 0;
        if (shader == null) {
            return;
        }
        this.m_shader = shader;
        this.m_program = shader.defaultProgram;
        this.m_optConfig = shader.m_defaultOptionsConfig;
        this.m_propertyBlock = new MaterialPorpertyBlock(this.m_program);
    }
    Object.defineProperty(Material.prototype, "program", {
        get: function () {
            if (this.m_program == null) {
                var newprogram = this.m_shader.getVariantProgram(this.m_optConfig);
                this.m_propertyBlock.setProgram(newprogram);
                this.m_program = newprogram;
            }
            return this.m_program;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Material.prototype, "shaderTags", {
        get: function () {
            if (this.m_shadertags == null)
                return this.m_shader.tags;
            return this.m_shadertags;
        },
        set: function (tags) {
            this.m_shadertags = tags;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Material.prototype, "propertyBlock", {
        get: function () {
            return this.m_propertyBlock;
        },
        enumerable: true,
        configurable: true
    });
    Material.prototype.setShader = function (shader) {
        this.m_shader = shader;
        this.m_program = null;
        this.m_useVariants = false;
    };
    Material.prototype.clone = function () {
        var mat = new Material();
        mat.m_shader = this.m_shader;
        mat.m_program = this.program;
        mat.m_propertyBlock = this.m_propertyBlock.clone();
        mat.m_useVariants = this.m_useVariants;
        mat.m_optConfig = this.m_optConfig.clone();
        return mat;
    };
    Material.prototype.setColor = function (name, color) {
        var p = this.m_propertyBlock.getUniform(name);
        if (p == null)
            return;
        p.value = color;
    };
    Material.prototype.setTexture = function (name, tex) {
        var p = this.m_propertyBlock.getUniform(name);
        if (p == null)
            return;
        p.value = tex;
    };
    Material.prototype.setFloat = function (name, val) {
        var p = this.m_propertyBlock.getUniform(name);
        if (p == null)
            return;
        p.value = val;
    };
    Material.prototype.setVec2 = function (name, valx, valy) {
        var p = this.m_propertyBlock.getUniform(name);
        if (p == null)
            return;
        p.value = [valx, valy];
    };
    Material.prototype.setVec3 = function (name, val) {
        var p = this.m_propertyBlock.getUniform(name);
        if (p == null)
            return;
        p.value = val.clone();
    };
    Material.prototype.setVec4 = function (name, val) {
        var p = this.m_propertyBlock.getUniform(name);
        if (p == null)
            return;
        p.value = val;
    };
    /**
     * Set uniform block binding
     * @param name
     * @param index
     */
    Material.prototype.setUniformBlockwitName = function (name, index) {
        var u = this.m_program.UniformBlock[name];
        if (u == null)
            return;
        var propertyblock = this.m_propertyBlock;
        var uniformblock = propertyblock.uniformsBlock;
        if (uniformblock == null) {
            uniformblock = {};
            propertyblock.uniformsBlock = uniformblock;
        }
        uniformblock[u] = index;
    };
    /**
     * Set uniform block binding
     * @param bufferindex shader uniform index, this value can be queried by @function <getUniformBlockIndex>
     * @param index uniform buffer binded index
     */
    Material.prototype.setUniformBlock = function (bufferindex, index) {
        var propertyblock = this.propertyBlock;
        var uniformblock = propertyblock.uniformsBlock;
        if (uniformblock == null) {
            uniformblock = {};
            propertyblock.uniformsBlock = uniformblock;
        }
        uniformblock[bufferindex] = index;
    };
    /**
     * Query the uniform blocksindex in current shader program
     * @param name
     */
    Material.prototype.getUniformBlockIndex = function (name) {
        return this.m_program.UniformBlock[name];
    };
    /**
     *
     * @param slotindex
     */
    Material.prototype.getUniformBlockName = function (slotindex) {
        return Material.programGetUniformBlockName(this.program, slotindex);
    };
    /**
     * Query the uniform block name with uniform block slot index.
     * @todo merge this function to class ShaderProgram
     * @param program
     * @param slotindex
     */
    Material.programGetUniformBlockName = function (program, slotindex) {
        var blocks = program.UniformBlock;
        for (var key in blocks) {
            if (blocks[key] == slotindex)
                return key;
        }
    };
    /**
     * SetFlag will not switch to new program immediately,
     * setUniform parameters must be called after @property {program} refreshed.
     * @param key
     * @param value
     * @param refresh
     */
    Material.prototype.setFlag = function (key, value, refresh) {
        if (refresh === void 0) { refresh = false; }
        var defOptCfg = this.m_shader.m_defaultOptionsConfig;
        var verified = this.m_shader.m_defaultOptionsConfig.verifyFlag(key, value);
        if (!verified) {
            console.warn("set shader flag verify failed: " + key + ":" + value);
            return;
        }
        if (!this.m_useVariants) {
            this.m_optConfig = defOptCfg.clone();
        }
        if (this.m_optConfig.setFlag(key, value)) {
            this.m_program = null;
            this.m_useVariants = true;
        }
        else {
            console.warn("set shader flag: value not changed");
        }
        if (refresh)
            this.refreshProgram();
    };
    /**
     * refresh program after @function <setFlag> called.
     */
    Material.prototype.refreshProgram = function () {
        return this.program;
    };
    Material.prototype.setFlagNoVerify = function (options) {
        var useVariants = this.m_useVariants;
        var cfg = useVariants ? this.m_optConfig : this.m_shader.m_defaultOptionsConfig;
        var val = cfg.getFlag(options.flag);
        if (val == null)
            return;
        if (val == options.default)
            return;
        if (!useVariants) {
            this.m_optConfig = cfg.clone();
            this.m_useVariants = true;
        }
        this.m_optConfig.setFlag(options.flag, options.default);
        this.m_program = null;
    };
    Material.prototype.getFlag = function (key) {
        var optCfg = this.m_useVariants ? this.m_shader.m_defaultOptionsConfig : this.m_optConfig;
        return optCfg.getFlag(key);
    };
    Material.prototype.apply = function (gl) {
        this.m_applyTexCount = 0;
        var program = this.program;
        var propertyblock = this.m_propertyBlock;
        var pu = propertyblock.uniforms;
        for (var key in pu) {
            var u = pu[key];
            if (key === "uShadowMap")
                continue;
            this.setUniform(gl, program.Uniforms[key], u.type, u.value);
        }
        var puniformblocks = propertyblock.uniformsBlock;
        if (puniformblocks != null) {
            var glp = program.Program;
            for (var key in puniformblocks) {
                var ind = Number(key);
                gl.uniformBlockBinding(glp, ind, puniformblocks[ind]);
            }
        }
    };
    /**
     * TODO clean apply process
     * especially for binded texture
     * @param gl
     */
    Material.prototype.clean = function (gl) {
        // let pu = this.m_propertyBlock.uniforms;
        // for(let i=0,len = pu.length;i<len;i++){
        //     let u = pu[i];
        //     let loc = this.program.Uniforms[u.key];
        //     let type = u.type;
        //     if(type == gl.SAMPLER_2D){
        //         //gl.bindTexture(gl.TEXTURE,null);
        //     }
        // }
    };
    Material.prototype.setUniform = function (gl, loc, type, val) {
        if (val == null && type != gl.SAMPLER_2D)
            return;
        switch (type) {
            case gl.FLOAT:
                gl.uniform1f(loc, val);
                break;
            case gl.FLOAT_VEC2:
                gl.uniform2fv(loc, val);
                break;
            case gl.FLOAT_VEC3:
                gl.uniform3fv(loc, val.raw);
                break;
            case gl.FLOAT_VEC4:
                gl.uniform4fv(loc, val.raw);
                break;
            case gl.SAMPLER_CUBE:
                if (val != null) {
                    var texCount = this.m_applyTexCount;
                    var tex = null;
                    if (val instanceof Texture_1.Texture) {
                        tex = val.rawtexture;
                    }
                    else if (val instanceof WebGLTexture) {
                        tex = val;
                    }
                    if (tex == null) {
                        //raw texture is null or onloading...
                        //TODO no internal texture_cube_map_support
                        //gl.uniform1i(loc,Material.DEF_TEXID_NUM);
                        return;
                    }
                    gl.activeTexture(gl.TEXTURE4 + texCount);
                    gl.bindTexture(gl.TEXTURE_CUBE_MAP, tex);
                    gl.uniform1i(loc, 4 + texCount);
                }
                else {
                    //texture is null
                    //use default white texture
                    //TODO no internal texture_cube_map_support
                    //gl.uniform1i(loc,Material.DEF_TEXID_NUM);
                }
                break;
            case gl.SAMPLER_2D:
                if (val != null) {
                    var texCount = this.m_applyTexCount;
                    var tex = null;
                    if (val instanceof Texture_1.Texture) {
                        tex = val.rawtexture;
                    }
                    else if (val instanceof WebGLTexture) {
                        tex = val;
                    }
                    if (tex == null) {
                        //raw texture is null or onloading...
                        gl.uniform1i(loc, Material.DEF_TEXID_NUM);
                        return;
                    }
                    gl.activeTexture(gl.TEXTURE4 + texCount);
                    gl.bindTexture(gl.TEXTURE_2D, tex);
                    gl.uniform1i(loc, 4 + texCount);
                    this.m_applyTexCount = texCount + 1;
                }
                else {
                    //texture is null
                    //use default white texture
                    gl.uniform1i(loc, Material.DEF_TEXID_NUM);
                }
                break;
        }
    };
    Material.DEF_TEXID_NUM = 3;
    return Material;
}());
exports.Material = Material;
