import { GLDataType } from "./gl/GL";
import { GLContext } from "./gl/GLContext";
export declare enum MeshTopology {
    Triangles = 4,
    TriangleFan = 6,
    TriangleStrip = 5,
    Points = 0,
    Lines = 1,
    LineStrip = 3,
    LineLoop = 2
}
export declare class MeshVertexAttrDesc {
    type: GLDataType;
    /** Component size [1,2,3,4] */
    size: number;
    totalbytes: number;
    offset: number;
    /**
     * constructor of MeshVertexAttrDesc
     * @param type data type
     * @param size componsnet length [1,2,3,4]
     * @param bytes total size in bytes
     * @param offset offset in bytes
     */
    constructor(type: GLDataType, size: number, bytes: number, offset?: number);
    readonly length: number;
}
export declare class MeshVertexDesc {
    position: MeshVertexAttrDesc;
    uv: MeshVertexAttrDesc;
    normal: MeshVertexAttrDesc;
    readonly totalByteSize: number;
}
export declare class MeshIndicesDesc {
    topology: MeshTopology;
    indiceCount: number;
    type: GLDataType;
    totalbytes: number;
    offset: number;
    constructor(topology?: MeshTopology, indices?: number, type?: GLDataType, offset?: number);
    set(topology?: MeshTopology, indices?: number, type?: GLDataType, offset?: number): void;
}
export declare type MeshDataBufferIndices = Uint16Array | Uint32Array | Uint8Array;
export declare type MeshDataBuffer = ArrayBuffer | DataView | Float32Array | Float64Array | Int32Array | Int16Array | Int8Array | Uint16Array | Uint32Array | Uint8Array;
export declare class Mesh {
    bufferVertices: WebGLBuffer;
    bufferIndices: WebGLBuffer;
    bufferUV: WebGLBuffer;
    bufferNormal: WebGLBuffer;
    name: string;
    readonly vertexDesc: MeshVertexDesc;
    readonly indiceDesc: MeshIndicesDesc;
    private static s_quad;
    private static s_cube;
    private static s_sphere;
    protected m_dataPosition: MeshDataBuffer;
    protected m_dataUV: MeshDataBuffer;
    protected m_dataNormal: MeshDataBuffer;
    protected m_dataIndices: MeshDataBufferIndices;
    readonly dataPosition: MeshDataBuffer;
    readonly dataUV: MeshDataBuffer;
    readonly dataNormal: MeshDataBuffer;
    readonly dataIndices: MeshDataBufferIndices;
    protected m_bufferInited: boolean;
    protected m_seperatedBuffer: boolean;
    readonly seperatedBuffer: boolean;
    constructor(name?: string, serperatedBuffer?: boolean);
    readonly bufferInited: boolean;
    setNormal(data: MeshDataBuffer, type: GLDataType, size: number): void;
    setUV(data: MeshDataBuffer, type: GLDataType, size: number): void;
    /**
     *
     * @param data
     * @param type data type
     * @param size component size
     */
    setPosition(data: MeshDataBuffer, type: GLDataType, size: number): void;
    setIndices(data: MeshDataBufferIndices, type: GLDataType, mode: MeshTopology): void;
    static readonly Quad: Mesh;
    static readonly Sphere: Mesh;
    static readonly Cube: Mesh;
    calculateNormal(): void;
    refreshMeshBuffer(glctx: GLContext): void;
}
export declare class MeshBufferUtility {
    /**
     *
     * @param target
     * @param buffer
     * @param offsetInByte
     * @returns endpos in byte
     */
    static copyBuffer(target: DataView, buffer: MeshDataBuffer, offsetInByte: number): number;
    static TypeSize(type: GLDataType): number;
    static IndicesBufferFillQuad(databuffer: MeshDataBufferIndices, quadsize: number): void;
}
