"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var GL_1 = require("./gl/GL");
var GLMath_1 = require("./math/GLMath");
var MeshTopology;
(function (MeshTopology) {
    MeshTopology[MeshTopology["Triangles"] = 4] = "Triangles";
    MeshTopology[MeshTopology["TriangleFan"] = 6] = "TriangleFan";
    MeshTopology[MeshTopology["TriangleStrip"] = 5] = "TriangleStrip";
    MeshTopology[MeshTopology["Points"] = 0] = "Points";
    MeshTopology[MeshTopology["Lines"] = 1] = "Lines";
    MeshTopology[MeshTopology["LineStrip"] = 3] = "LineStrip";
    MeshTopology[MeshTopology["LineLoop"] = 2] = "LineLoop";
})(MeshTopology = exports.MeshTopology || (exports.MeshTopology = {}));
var MeshVertexAttrDesc = /** @class */ (function () {
    /**
     * constructor of MeshVertexAttrDesc
     * @param type data type
     * @param size componsnet length [1,2,3,4]
     * @param bytes total size in bytes
     * @param offset offset in bytes
     */
    function MeshVertexAttrDesc(type, size, bytes, offset) {
        if (offset === void 0) { offset = 0; }
        this.offset = 0;
        this.type = type;
        this.size = size;
        this.totalbytes = bytes;
        this.offset = 0;
    }
    Object.defineProperty(MeshVertexAttrDesc.prototype, "length", {
        get: function () {
            return this.totalbytes / this.size / MeshBufferUtility.TypeSize(this.type);
        },
        enumerable: true,
        configurable: true
    });
    return MeshVertexAttrDesc;
}());
exports.MeshVertexAttrDesc = MeshVertexAttrDesc;
var MeshVertexDesc = /** @class */ (function () {
    function MeshVertexDesc() {
    }
    Object.defineProperty(MeshVertexDesc.prototype, "totalByteSize", {
        get: function () {
            var bytes = this.position.totalbytes;
            if (this.uv != null) {
                bytes += this.uv.totalbytes;
            }
            if (this.normal != null) {
                bytes += this.normal.totalbytes;
            }
            return bytes;
        },
        enumerable: true,
        configurable: true
    });
    return MeshVertexDesc;
}());
exports.MeshVertexDesc = MeshVertexDesc;
var MeshIndicesDesc = /** @class */ (function () {
    function MeshIndicesDesc(topology, indices, type, offset) {
        if (topology === void 0) { topology = MeshTopology.Triangles; }
        if (indices === void 0) { indices = 0; }
        if (type === void 0) { type = GL_1.GL.UNSIGNED_SHORT; }
        if (offset === void 0) { offset = 0; }
        this.indiceCount = 0;
        this.topology = topology;
        this.indiceCount = indices;
        this.type = type;
        this.offset = offset;
        this.totalbytes = indices * MeshBufferUtility.TypeSize(type);
    }
    MeshIndicesDesc.prototype.set = function (topology, indices, type, offset) {
        if (topology === void 0) { topology = MeshTopology.Triangles; }
        if (indices === void 0) { indices = 0; }
        if (type === void 0) { type = GL_1.GL.UNSIGNED_SHORT; }
        if (offset === void 0) { offset = 0; }
        this.topology = topology;
        this.indiceCount = indices;
        this.type = type;
        this.offset = offset;
        this.totalbytes = indices * MeshBufferUtility.TypeSize(type);
    };
    return MeshIndicesDesc;
}());
exports.MeshIndicesDesc = MeshIndicesDesc;
var Mesh = /** @class */ (function () {
    function Mesh(name, serperatedBuffer) {
        if (serperatedBuffer === void 0) { serperatedBuffer = false; }
        this.vertexDesc = new MeshVertexDesc();
        this.indiceDesc = new MeshIndicesDesc();
        this.m_bufferInited = false;
        this.m_seperatedBuffer = false;
        this.name = name;
        this.m_seperatedBuffer = serperatedBuffer;
    }
    Object.defineProperty(Mesh.prototype, "dataPosition", {
        get: function () { return this.m_dataPosition; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Mesh.prototype, "dataUV", {
        get: function () { return this.m_dataUV; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Mesh.prototype, "dataNormal", {
        get: function () { return this.m_dataNormal; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Mesh.prototype, "dataIndices", {
        get: function () { return this.m_dataIndices; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Mesh.prototype, "seperatedBuffer", {
        get: function () { return this.m_seperatedBuffer; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Mesh.prototype, "bufferInited", {
        get: function () {
            return this.m_bufferInited;
        },
        enumerable: true,
        configurable: true
    });
    Mesh.prototype.setNormal = function (data, type, size) {
        this.m_dataNormal = data;
        this.vertexDesc.normal = new MeshVertexAttrDesc(type, size, data.byteLength);
    };
    Mesh.prototype.setUV = function (data, type, size) {
        this.m_dataUV = data;
        this.vertexDesc.uv = new MeshVertexAttrDesc(type, size, data.byteLength);
    };
    /**
     *
     * @param data
     * @param type data type
     * @param size component size
     */
    Mesh.prototype.setPosition = function (data, type, size) {
        this.m_dataPosition = data;
        this.vertexDesc.position = new MeshVertexAttrDesc(type, size, data.byteLength);
    };
    Mesh.prototype.setIndices = function (data, type, mode) {
        this.m_dataIndices = data;
        var inddesc = this.indiceDesc;
        inddesc.indiceCount = data.length;
        inddesc.topology = mode;
        inddesc.offset = 0;
        inddesc.type = type;
        inddesc.totalbytes = data.byteLength;
    };
    Object.defineProperty(Mesh, "Quad", {
        get: function () {
            if (Mesh.s_quad != null)
                return Mesh.s_quad;
            var quad = new Mesh();
            Mesh.s_quad = quad;
            var dataPosition = new Float32Array([
                -0.5, -0.5, 0, 1,
                0.5, -0.5, 0, 1,
                -0.5, 0.5, 0, 1,
                0.5, 0.5, 0, 1
            ]);
            var dataIndices = new Uint16Array([
                0, 1, 2,
                1, 3, 2
            ]);
            var dataUV = new Float32Array([
                0.0, 1.0,
                1.0, 1.0,
                0.0, 0.0,
                1.0, 0.0
            ]);
            quad.m_dataPosition = dataPosition;
            quad.m_dataUV = dataUV;
            quad.m_dataIndices = dataIndices;
            quad.name = "quad";
            var vertexdesc = quad.vertexDesc;
            vertexdesc.position = new MeshVertexAttrDesc(GL_1.GL.FLOAT, 4, dataPosition.length * 4);
            vertexdesc.uv = new MeshVertexAttrDesc(GL_1.GL.FLOAT, 2, dataUV.length * 4);
            quad.indiceDesc.set(MeshTopology.Triangles, dataIndices.length, GL_1.GL.UNSIGNED_SHORT, 0);
            quad.calculateNormal();
            return quad;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Mesh, "Sphere", {
        get: function () {
            if (Mesh.s_sphere != null)
                return Mesh.s_sphere;
            var sphere = new Mesh('sphere');
            Mesh.s_sphere = sphere;
            var slicey = 16;
            var slicer = slicey * 2;
            var radstep = Math.PI / slicey;
            var rad = -Math.PI / 2.0 + radstep;
            var positions = [];
            var uvs = [];
            var pcount = (slicey - 1) * (slicer + 1) + 2;
            positions.push(.0, -1.0, .0, 1.0);
            uvs.push(0.5, 0.0);
            for (var t = 1; t < slicey; t++) {
                var y = Math.sin(rad);
                var d = Math.cos(rad);
                var yaw = 0;
                var v = t * 1.0 / slicey;
                for (var s = 0; s <= slicer; s++) {
                    var x = d * Math.cos(yaw);
                    var z = d * Math.sin(yaw);
                    positions.push(x, y, z, 1.0);
                    uvs.push(s * 1.0 / slicer, v);
                    yaw += radstep;
                }
                rad += radstep;
            }
            positions.push(.0, 1.0, .0, 1.0);
            uvs.push(0.5, 1.0);
            var indices = [];
            {
                //bottom
                for (var t = 1, tbmax = slicer; t <= tbmax; t++) {
                    indices.push(0, t, t + 1);
                }
                //center
                var slicerlayer = slicer + 1;
                for (var t = 0; t < slicey - 2; t++) {
                    var ib = 1 + t * slicerlayer;
                    var it_1 = ib + slicerlayer;
                    for (var s = 0; s < slicerlayer; s++) {
                        var ibs = ib + s;
                        var its = it_1 + s;
                        indices.push(ibs, ibs + 1, its + 1, ibs, its + 1, its);
                    }
                }
                //top
                var imax = pcount - 1;
                var istart = imax - slicer - 1;
                for (var t = istart, ttmax = imax; t < ttmax; t++) {
                    indices.push(imax, t, t + 1);
                }
            }
            var dataposition = new Float32Array(positions);
            sphere.m_dataPosition = dataposition;
            var dataindices = new Uint16Array(indices);
            sphere.m_dataIndices = dataindices;
            var datauv = new Float32Array(uvs);
            sphere.m_dataUV = datauv;
            sphere.m_dataNormal = dataposition;
            var vertexdesc = sphere.vertexDesc;
            vertexdesc.position = new MeshVertexAttrDesc(GL_1.GL.FLOAT, 4, dataposition.byteLength);
            vertexdesc.normal = new MeshVertexAttrDesc(GL_1.GL.FLOAT, 4, dataposition.byteLength);
            vertexdesc.uv = new MeshVertexAttrDesc(GL_1.GL.FLOAT, 2, datauv.byteLength);
            sphere.indiceDesc.set(MeshTopology.Triangles, indices.length, GL_1.GL.UNSIGNED_SHORT, 0);
            return sphere;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Mesh, "Cube", {
        get: function () {
            if (Mesh.s_cube != null)
                return Mesh.s_cube;
            var cube = new Mesh();
            Mesh.s_cube = cube;
            var dataPosition = new Float32Array([
                -1, 1, 1, 1,
                1, 1, 1, 1,
                -1, -1, 1, 1,
                1, -1, 1, 1,
                1, 1, 1, 1,
                1, 1, -1, 1,
                1, -1, 1, 1,
                1, -1, -1, 1,
                1, 1, -1, 1,
                -1, 1, -1, 1,
                1, -1, -1, 1,
                -1, -1, -1, 1,
                -1, 1, -1, 1,
                -1, 1, 1, 1,
                -1, -1, -1, 1,
                -1, -1, 1, 1,
                -1, 1, -1, 1,
                1, 1, -1, 1,
                -1, 1, 1, 1,
                1, 1, 1, 1,
                -1, -1, 1, 1,
                1, -1, 1, 1,
                -1, -1, -1, 1,
                1, -1, -1, 1,
            ]);
            var dataUV = new Float32Array(48);
            for (var i = 0; i < 6; i++) {
                dataUV.set([0, 1, 1, 1, 0, 0, 1, 0], i * 8);
            }
            var dataIndices = [];
            for (var i_1 = 0; i_1 < 6; i_1++) {
                var k = i_1 * 4;
                dataIndices.push(k, k + 1, k + 2, k + 1, k + 3, k + 2);
            }
            cube.m_dataIndices = new Uint16Array(dataIndices);
            cube.m_dataPosition = dataPosition;
            cube.m_dataUV = dataUV;
            cube.name = "cube";
            var vertexdesc = cube.vertexDesc;
            vertexdesc.position = new MeshVertexAttrDesc(GL_1.GL.FLOAT, 4, dataPosition.length * 4);
            vertexdesc.uv = new MeshVertexAttrDesc(GL_1.GL.FLOAT, 2, dataUV.length * 4);
            cube.indiceDesc.set(MeshTopology.Triangles, dataIndices.length, GL_1.GL.UNSIGNED_SHORT, 0);
            cube.calculateNormal();
            return cube;
        },
        enumerable: true,
        configurable: true
    });
    Mesh.prototype.calculateNormal = function () {
        if (this.indiceDesc.topology != MeshTopology.Triangles) {
            console.warn(MeshTopology[this.indiceDesc.topology] + " is not supported.");
            return;
        }
        var normal = this.m_dataNormal;
        var position = this.m_dataPosition;
        if (position == null) {
            console.warn('vertices position is needed for normal calculation.');
            return;
        }
        var indics = this.m_dataIndices;
        if (indics == null) {
            console.warn('indices data is needed for normal calculation.');
            return;
        }
        var positionattr = this.vertexDesc.position;
        var floatLength = positionattr.totalbytes / MeshBufferUtility.TypeSize(positionattr.type);
        var normaldata = new Float32Array(floatLength);
        var verticesLen = floatLength / positionattr.size;
        var normalVec = new Array(verticesLen);
        var normalAcc = new Uint16Array(verticesLen);
        var tricount = indics.length / 3;
        for (var i = 0; i < tricount; i++) {
            var off = i * 3;
            var i1 = indics[off];
            var i2 = indics[off + 1];
            var i3 = indics[off + 2];
            var i1v = i1 * 4;
            var i2v = i2 * 4;
            var i3v = i3 * 4;
            var v1 = new GLMath_1.vec3([position[i1v], position[i1v + 1], position[i1v + 2]]);
            var v2 = new GLMath_1.vec3([position[i2v], position[i2v + 1], position[i2v + 2]]);
            var v3 = new GLMath_1.vec3([position[i3v], position[i3v + 1], position[i3v + 2]]);
            var n = GLMath_1.vec3.Cross(v1.sub(v2), v3.sub(v2)).normalize;
            normalAcc[i1]++;
            if (normalVec[i1] == null)
                normalVec[i1] = GLMath_1.vec3.zero;
            normalVec[i1].add(n);
            normalAcc[i2]++;
            if (normalVec[i2] == null)
                normalVec[i2] = GLMath_1.vec3.zero;
            normalVec[i2].add(n);
            normalAcc[i3]++;
            if (normalVec[i3] == null)
                normalVec[i3] = GLMath_1.vec3.zero;
            normalVec[i3].add(n);
        }
        for (var i = 0; i < verticesLen; i++) {
            var v = normalVec[i].div(normalAcc[i]).raw;
            normaldata.set([v[0], v[1], v[2], 0], i * 4);
        }
        this.m_dataNormal = normaldata;
        this.vertexDesc.normal = new MeshVertexAttrDesc(GL_1.GL.FLOAT, 4, normaldata.length * 4);
    };
    Mesh.prototype.refreshMeshBuffer = function (glctx) {
        if (this.m_bufferInited)
            return;
        var gl = glctx.gl;
        var buffervert = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, buffervert);
        this.bufferVertices = buffervert;
        var dataTotalSize = this.vertexDesc.totalByteSize;
        var totalData = new ArrayBuffer(dataTotalSize);
        var totalDataView = new DataView(totalData, 0, dataTotalSize);
        var offset = 0;
        var vertexDesc = this.vertexDesc;
        if (this.m_dataPosition != null) {
            vertexDesc.position.offset = offset;
            offset = MeshBufferUtility.copyBuffer(totalDataView, this.m_dataPosition, offset);
            offset = Math.ceil(offset / 4.0) * 4;
        }
        if (this.m_dataUV != null) {
            vertexDesc.uv.offset = offset;
            offset = MeshBufferUtility.copyBuffer(totalDataView, this.m_dataUV, offset);
            offset = Math.ceil(offset / 4.0) * 4;
        }
        if (this.m_dataNormal != null) {
            vertexDesc.normal.offset = offset;
            offset = MeshBufferUtility.copyBuffer(totalDataView, this.m_dataNormal, offset);
            offset = Math.ceil(offset / 4.0) * 4;
        }
        gl.bufferData(gl.ARRAY_BUFFER, totalData, gl.STATIC_DRAW);
        gl.bindBuffer(gl.ARRAY_BUFFER, null);
        //Indices
        var dataIndices = this.m_dataIndices;
        var hasIndices = dataIndices != null && dataIndices.length != 0;
        if (hasIndices) {
            var bufferIndices = gl.createBuffer();
            gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, bufferIndices);
            gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, dataIndices, gl.STATIC_DRAW);
            gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, null);
            this.bufferIndices = bufferIndices;
        }
        this.m_bufferInited = true;
    };
    return Mesh;
}());
exports.Mesh = Mesh;
var MeshBufferUtility = /** @class */ (function () {
    function MeshBufferUtility() {
    }
    /**
     *
     * @param target
     * @param buffer
     * @param offsetInByte
     * @returns endpos in byte
     */
    MeshBufferUtility.copyBuffer = function (target, buffer, offsetInByte) {
        var offset = offsetInByte;
        if (buffer instanceof DataView) {
            var sourceView = new Uint8Array(buffer.buffer, 0, buffer.byteLength);
            for (var i = 0, len = sourceView.byteLength; i < len; i++) {
                target.setUint8(offset, sourceView[i]);
                offset++;
            }
            return offset;
        }
        else if (buffer instanceof ArrayBuffer) {
            var sourceView = new Uint8Array(buffer, 0, buffer.byteLength);
            for (var i = 0, len = sourceView.byteLength; i < len; i++) {
                target.setUint8(offset, sourceView[i]);
                offset++;
            }
            return offset;
        }
        else if (buffer instanceof Float32Array) {
            for (var i = 0, len = buffer.length; i < len; i++) {
                target.setFloat32(offset, buffer[i], true);
                offset += 4;
            }
            return offset;
        }
        else if (buffer instanceof Uint16Array) {
            for (var i = 0, len = buffer.length; i < len; i++) {
                target.setUint16(offset, buffer[i]);
                offset += 2;
            }
            return offset;
        }
        else {
            throw new Error('not implemented');
        }
    };
    MeshBufferUtility.TypeSize = function (type) {
        if (type == GL_1.GLConst.FLOAT || type == GL_1.GLConst.UNSIGNED_INT) {
            return 4;
        }
        if (type == GL_1.GLConst.SHORT || type == GL_1.GLConst.UNSIGNED_SHORT) {
            return 2;
        }
        if (type == GL_1.GLConst.BYTE || type == GL_1.GLConst.UNSIGNED_BYTE) {
            return 1;
        }
        return 0;
    };
    MeshBufferUtility.IndicesBufferFillQuad = function (databuffer, quadsize) {
        var itemlen = quadsize * 6;
        if (databuffer.length < itemlen)
            throw new Error('buffer size exceeded.');
        var index = 0;
        var vindex = 0;
        for (var i = 0; i < quadsize; i++) {
            databuffer[index] = vindex;
            databuffer[index + 1] = vindex + 1;
            databuffer[index + 2] = vindex + 2;
            databuffer[index + 3] = vindex;
            databuffer[index + 4] = vindex + 2;
            databuffer[index + 5] = vindex + 3;
            index += 6;
            vindex += 4;
        }
    };
    return MeshBufferUtility;
}());
exports.MeshBufferUtility = MeshBufferUtility;
