"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var ShaderFX_1 = require("./shaderfx/ShaderFX");
var BaseRender_1 = require("./BaseRender");
var MeshRender = /** @class */ (function (_super) {
    __extends(MeshRender, _super);
    /**
     * @param mesh
     * @param mat
     * @param dynamic dynamic meshRender will not generate VertexArrayObject
     */
    function MeshRender(mesh, mat, dynamic) {
        if (dynamic === void 0) { dynamic = false; }
        var _this = _super.call(this) || this;
        _this.m_dynamic = false;
        _this.m_vaoProgamId = -1;
        _this.mesh = mesh;
        _this.material = mat;
        _this.castShadow = true;
        _this.m_dynamic = dynamic;
        return _this;
    }
    Object.defineProperty(MeshRender.prototype, "dynamic", {
        get: function () {
            return this.m_dynamic;
        },
        enumerable: true,
        configurable: true
    });
    MeshRender.prototype.release = function (glctx) {
        if (this.m_vao != null) {
            glctx.gl.deleteVertexArray(this.m_vao);
            this.m_vao = null;
        }
        this.mesh = null;
    };
    MeshRender.prototype.draw = function (gl) {
        var mesh = this.mesh;
        var desc = mesh.indiceDesc;
        gl.drawElements(desc.topology, desc.indiceCount, desc.type, desc.offset);
    };
    MeshRender.prototype.refreshData = function (glctx) {
        var mesh = this.mesh;
        var mat = this.material;
        if (mat == null || mat.program == null) {
            throw new Error("material or program is null");
        }
        if (this.m_dynamic) {
            if (!mesh.bufferInited) {
                mesh.refreshMeshBuffer(glctx);
            }
        }
        else {
            var vao = this.m_vao;
            if (vao != null) {
                var curid = mat.program.id;
                if (curid == this.m_vaoProgamId)
                    return;
                console.log("program changes");
                glctx.gl.deleteVertexArray(this.m_vao);
                this.m_vaoProgamId = -1;
            }
            this.m_vao = MeshRender.CreateVertexArrayObj(glctx, mesh, mat.program);
            this.m_vaoProgamId = mat.program.id;
        }
    };
    /**
     * Bind meshbuffers
     * dynamic meshrender: bindBuffer
     * static meshrender: bindVertexArray
     * @param gl
     */
    MeshRender.prototype.bindVertexArray = function (gl) {
        if (this.m_dynamic) {
            MeshRender.bindBuffers(gl, this.mesh, this.material.program);
        }
        else {
            gl.bindVertexArray(this.m_vao);
        }
    };
    /**
     * Unbind meshbuffers
     * dynamic meshrender: unbindBuffer
     * static meshrender: unbindVertexArray
     * @param gl
     */
    MeshRender.prototype.unbindVertexArray = function (gl, unbindBuffer) {
        if (unbindBuffer === void 0) { unbindBuffer = true; }
        if (this.m_dynamic) {
            if (unbindBuffer) {
                gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, null);
                gl.bindBuffer(gl.ARRAY_BUFFER, null);
            }
        }
        else {
            gl.bindVertexArray(null);
        }
    };
    MeshRender.bindBuffers = function (gl, mesh, program) {
        var vertdesc = mesh.vertexDesc;
        var attrs = program.Attributes;
        if (mesh.seperatedBuffer) {
            if (vertdesc.position != null) {
                gl.bindBuffer(gl.ARRAY_BUFFER, mesh.bufferVertices);
                var aPos = attrs[ShaderFX_1.ShaderFX.ATTR_aPosition];
                if (aPos != null) {
                    var posdesc = vertdesc.position;
                    gl.vertexAttribPointer(aPos, posdesc.size, gl.FLOAT, false, posdesc.size * 4, posdesc.offset);
                    gl.enableVertexAttribArray(aPos);
                }
            }
            if (vertdesc.uv != null) {
                gl.bindBuffer(gl.ARRAY_BUFFER, mesh.bufferUV);
                var aUV = attrs[ShaderFX_1.ShaderFX.ATTR_aUV];
                if (aUV != null) {
                    var uvdesc = vertdesc.uv;
                    gl.vertexAttribPointer(aUV, uvdesc.size, gl.FLOAT, false, uvdesc.size * 4, uvdesc.offset);
                    gl.enableVertexAttribArray(aUV);
                }
            }
            if (vertdesc.normal) {
                gl.bindBuffer(gl.ARRAY_BUFFER, mesh.bufferNormal);
                var aNorm = attrs[ShaderFX_1.ShaderFX.ATTR_aNormal];
                if (aNorm != null) {
                    var normdesc = vertdesc.normal;
                    gl.vertexAttribPointer(aNorm, normdesc.size, gl.FLOAT, false, normdesc.size * 4, normdesc.offset);
                    gl.enableVertexAttribArray(aNorm);
                }
            }
            gl.bindBuffer(gl.ARRAY_BUFFER, null);
        }
        else {
            gl.bindBuffer(gl.ARRAY_BUFFER, mesh.bufferVertices);
            if (vertdesc.position != null) {
                var aPos = attrs[ShaderFX_1.ShaderFX.ATTR_aPosition];
                if (aPos != null) {
                    var posdesc = vertdesc.position;
                    gl.vertexAttribPointer(aPos, posdesc.size, gl.FLOAT, false, posdesc.size * 4, posdesc.offset);
                    gl.enableVertexAttribArray(aPos);
                }
            }
            if (vertdesc.uv != null) {
                var aUV = attrs[ShaderFX_1.ShaderFX.ATTR_aUV];
                if (aUV != null) {
                    var uvdesc = vertdesc.uv;
                    gl.vertexAttribPointer(aUV, uvdesc.size, gl.FLOAT, false, uvdesc.size * 4, uvdesc.offset);
                    gl.enableVertexAttribArray(aUV);
                }
            }
            if (vertdesc.normal) {
                var aNorm = attrs[ShaderFX_1.ShaderFX.ATTR_aNormal];
                if (aNorm != null) {
                    var normdesc = vertdesc.normal;
                    gl.vertexAttribPointer(aNorm, normdesc.size, gl.FLOAT, false, normdesc.size * 4, normdesc.offset);
                    gl.enableVertexAttribArray(aNorm);
                }
            }
        }
        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, mesh.bufferIndices);
    };
    MeshRender.CreateVertexArrayObj = function (glctx, mesh, program) {
        if (!mesh.bufferInited) {
            mesh.refreshMeshBuffer(glctx);
        }
        if (program == null)
            throw new Error("program is null");
        var gl = glctx.gl;
        var vao = gl.createVertexArray();
        gl.bindVertexArray(vao);
        MeshRender.bindBuffers(gl, mesh, program);
        gl.bindVertexArray(null);
        return vao;
    };
    return MeshRender;
}(BaseRender_1.BaseRender));
exports.MeshRender = MeshRender;
