"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Shader_1 = require("./shaderfx/Shader");
var PipelineStateCache = /** @class */ (function () {
    function PipelineStateCache(glctx) {
        this.gl = glctx.gl;
        this.m_curtags = new Shader_1.ShaderTags();
    }
    /**
     * Reset the pipeline to the given state.
     * @param tags
     */
    PipelineStateCache.prototype.reset = function (tags) {
        this.m_lastTags = null;
        var curtags = this.m_curtags;
        var gl = this.gl;
        if (tags.ztest != null) {
            var ztest = tags.ztest;
            if (curtags.ztest != ztest) {
                curtags.ztest = ztest;
                gl.depthFunc(ztest);
            }
        }
        if (tags.culling != null) {
            var culling = tags.culling;
            if (curtags.culling != culling) {
                var curculling = curtags.culling;
                curtags.culling = culling;
                if (culling == Shader_1.CullingMode.None) {
                    gl.disable(gl.CULL_FACE);
                }
                else {
                    if (curculling == Shader_1.CullingMode.None) {
                        gl.enable(gl.CULL_FACE);
                    }
                    gl.cullFace(culling);
                }
            }
        }
        if (tags.zwrite != null) {
            var zwrite = tags.zwrite;
            if (curtags.zwrite != zwrite) {
                curtags.zwrite = zwrite;
                gl.depthMask(zwrite);
            }
        }
        var blend = tags.blend;
        if (curtags.blend != blend) {
            if (!blend) {
                gl.disable(gl.BLEND);
            }
            else {
                gl.enable(gl.BLEND);
            }
        }
        // if(tags.blendOp != null){
        //     let blendop = tags.blendOp;
        //     if(curtags.blendOp != blendop){
        //         curtags.blendOp = blendop;
        //         if(blendop == null){
        //             gl.disable(gl.BLEND);
        //         }
        //         else{
        //             gl.enable(gl.BLEND);
        //             let op = tags.blendOp;
        //             let srcf = tags.blendFactorSrc;
        //             let dstf = tags.blendFactorDst;
        //             if(op == null) op = BlendOperator.ADD;
        //             if(srcf == null) srcf = BlendFactor.SRC_ALPHA;
        //             if(dstf == null) dstf = BlendFactor.ONE_MINUS_SRC_ALPHA;
        //             if(curtags.blendOp != op){
        //                 curtags.blendOp = op;
        //                 gl.blendEquation(op);
        //             }
        //             let factorDirty = false;
        //             if(curtags.blendFactorSrc != srcf){
        //                 curtags.blendFactorSrc = srcf;
        //                 factorDirty = true;
        //             }
        //             if(curtags.blendFactorDst != dstf){
        //                 curtags.blendFactorDst = dstf;
        //                 factorDirty = true;
        //             }
        //             if(factorDirty) gl.blendFunc(srcf,dstf);
        //         }
        //     }
        // }
        this.m_deftags = tags;
    };
    PipelineStateCache.prototype.setZTest = function (comp) {
        var curtag = this.m_curtags;
        if (curtag.ztest == comp)
            return;
        curtag.ztest = comp;
        var gl = this.gl;
        gl.depthFunc(comp);
    };
    PipelineStateCache.prototype.setZWrite = function (enable) {
        var curtag = this.m_curtags;
        if (curtag.zwrite == enable)
            return;
        curtag.zwrite = enable;
        var gl = this.gl;
        gl.depthMask(enable);
    };
    PipelineStateCache.prototype.setBlend = function (enable) {
        var curtag = this.m_curtags;
        if (curtag.blend == enable)
            return;
        curtag.blend = enable;
        var gl = this.gl;
        if (enable) {
            gl.enable(gl.BLEND);
        }
        else {
            gl.disable(gl.BLEND);
        }
    };
    PipelineStateCache.prototype.apply = function (tags) {
        if (this.m_lastTags == tags)
            return;
        var deftags = this.m_deftags;
        var curtags = this.m_curtags;
        var gl = this.gl;
        var ztest = tags.ztest;
        if (ztest == null)
            ztest = deftags.ztest;
        if (ztest != curtags.ztest) {
            curtags.ztest = ztest;
            gl.depthFunc(ztest);
        }
        var culling = tags.culling;
        if (culling == null)
            culling = deftags.culling;
        if (culling != curtags.culling) {
            var curculling = curtags.culling;
            curtags.culling = culling;
            if (culling == Shader_1.CullingMode.None) {
                gl.disable(gl.CULL_FACE);
            }
            else {
                if (curculling == Shader_1.CullingMode.None) {
                    gl.enable(gl.CULL_FACE);
                }
                gl.cullFace(culling);
            }
        }
        var zwrite = tags.zwrite;
        if (zwrite == null)
            zwrite = deftags.zwrite;
        if (zwrite != curtags.zwrite) {
            curtags.zwrite = zwrite;
            gl.depthMask(zwrite);
        }
        var blend = tags.blend;
        if (blend != curtags.blend) {
            if (blend) {
                gl.enable(gl.BLEND);
                var tagBlendOp = tags.blendOp;
                if (curtags.blendOp != tagBlendOp) {
                    curtags.blendOp = tagBlendOp;
                    gl.blendEquation(tagBlendOp);
                }
                var tagBlendSrc = tags.blendFactorSrc;
                var tagBlendDst = tags.blendFactorDst;
                if (curtags.blendFactorSrc != tagBlendSrc || curtags.blendFactorDst != tagBlendDst) {
                    curtags.blendFactorDst = tagBlendDst;
                    curtags.blendFactorSrc = tagBlendSrc;
                    gl.blendFunc(tagBlendSrc, tagBlendDst);
                }
            }
            else {
                gl.disable(gl.BLEND);
            }
        }
        this.m_lastTags = tags;
    };
    PipelineStateCache.prototype.release = function () {
    };
    return PipelineStateCache;
}());
exports.PipelineStateCache = PipelineStateCache;
