"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var GLMath_1 = require("./math/GLMath");
/**
 * WIP efactoring
 * substitution of class Transform.
 */
var Trans = /** @class */ (function () {
    function Trans() {
        //local space
        this.m_localPosition = GLMath_1.vec3.zero;
        this.m_localRotation = GLMath_1.quat.Identity;
        this.m_localScale = GLMath_1.vec3.one;
        this.m_localMtx = GLMath_1.mat4.Identity;
        this.m_localMtxDirty = true;
        this.m_locaLTRSDirty = false;
        //world psace
        this.m_worldpos = GLMath_1.vec3.zero;
        this.m_worldRotation = GLMath_1.quat.Identity;
        this.m_worldScale = GLMath_1.vec3.one;
        this.m_worldMtx = GLMath_1.mat4.Identity;
        /**
         * mark for any values changed of world.TRS
         */
        this.m_worldMtxDirty = false;
        /**
         * This mark set to true when worldPosition changed.
         * For setting local/world position, accessing worldPosition should not invoke @function <updateWorldMatrix> .
         */
        this.m_worldPosDirty = false;
        this.m_worldScaleDirty = false;
        this.m_worldRotaDirty = false;
    }
    Object.defineProperty(Trans.prototype, "up", {
        get: function () {
            return this.worldRotation.rota(GLMath_1.vec3.up);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Trans.prototype, "right", {
        get: function () {
            return this.worldRotation.rota(GLMath_1.vec3.right);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Trans.prototype, "forward", {
        get: function () {
            return this.worldRotation.rota(GLMath_1.vec3.forward);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Trans.prototype, "localMtx", {
        //getter /setter
        get: function () {
            this.updateLocalMtx();
            return this.m_localMtx;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Trans.prototype, "localPosition", {
        get: function () { return this.m_localPosition; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Trans.prototype, "localRoataion", {
        get: function () { return this.m_localRotation; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Trans.prototype, "localScale", {
        get: function () { return this.m_localScale; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Trans.prototype, "worldPosition", {
        get: function () {
            if (this.m_worldPosDirty)
                this.updateWorldMatrix(false);
            return this.m_worldpos;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Trans.prototype, "worldRotation", {
        get: function () {
            if (this.m_worldRotaDirty) {
                this.updateWorldMatrix(false);
            }
            return this.m_worldRotation;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Trans.prototype, "worldScale", {
        get: function () {
            if (this.m_worldScaleDirty)
                this.updateWorldMatrix(false);
            return this.m_worldScale;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Trans.prototype, "worldMtx", {
        get: function () {
            this.updateWorldMtx();
            return this.m_worldMtx;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Trans.prototype, "objectToWorldMatrix", {
        get: function () {
            if (this.m_worldMtxInv == null) {
                this.m_worldMtxInv = this.worldMtx.inverse();
            }
            return this.m_worldMtxInv;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Trans.prototype, "parentWorldMtx", {
        get: function () {
            var identity = GLMath_1.mat4.Identity;
            if (this.m_parent == null)
                return identity;
            return this.m_parent.worldMtx;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Trans.prototype, "parent", {
        get: function () { return this.m_parent; },
        set: function (p) {
            if (p == null) {
                var curp = this.m_parent;
                if (curp != null)
                    curp.removeChild(this);
            }
            else {
                p.addChild(this);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Trans.prototype, "children", {
        get: function () { return this.m_children; },
        enumerable: true,
        configurable: true
    });
    /**
     * set position default is world-space
     * @param pos
     * @param worldspace @default true
     */
    Trans.prototype.setPosition = function (pos, worldspace) {
        if (worldspace === void 0) { worldspace = true; }
        if (worldspace) {
            this.m_worldpos.set(pos);
            var p = this.m_parent;
            if (p == null) {
                this.m_localPosition.set(pos);
            }
            else {
                this.m_localPosition = p.objectToWorldMatrix.mulvec(pos.vec4(1.0)).vec3();
            }
            this.m_localMtxDirty = true;
            this.m_locaLTRSDirty = true;
            this.m_worldMtxDirty = true;
        }
        else {
            this.m_localPosition.set(pos);
            this.m_localMtxDirty = true;
            this.m_locaLTRSDirty = true;
            this.m_worldMtxDirty = true;
            this.m_worldPosDirty = true;
        }
    };
    /** set local scale */
    Trans.prototype.setScale = function (scale) {
        this.m_localScale.set(scale);
        this.m_localMtxDirty = true;
        this.m_locaLTRSDirty = true;
        this.m_worldMtxDirty = true;
        this.m_worldScaleDirty = true;
    };
    /**
     * set rotation, world-space is default
     * @param q
     * @param worldspace @default true
     */
    Trans.prototype.setRotation = function (q, worldspace) {
        if (worldspace === void 0) { worldspace = true; }
        if (worldspace) {
            this.m_worldRotation.set(q);
            var p = this.parent;
            if (p == null) {
                this.m_localRotation.set(q);
            }
            else {
                this.m_localRotation = GLMath_1.quat.Div(q, this.m_parent.worldRotation);
            }
            this.m_localMtxDirty = true;
            this.m_locaLTRSDirty = true;
            this.m_worldMtxDirty = true;
        }
        else {
            this.m_localRotation.set(q);
            this.m_localMtxDirty = true;
            this.m_locaLTRSDirty = true;
            this.m_worldMtxDirty = true;
            this.m_worldRotaDirty = true;
        }
    };
    Trans.prototype.updateLocalMtx = function () {
        if (!this.m_localMtxDirty)
            return;
        this.m_localMtx.setTRS(this.m_localPosition, this.m_localRotation, this.m_localScale);
        this.m_localMtxDirty = false;
    };
    Trans.prototype.updateWorldMtx = function () {
        if (this.m_worldMtxDirty) {
            var p = this.parent;
            if (p == null) {
                this.m_worldMtx.set(this.localMtx);
            }
            else {
                this.m_worldMtx = p.worldMtx.mul(this.localMtx);
            }
            this.m_worldMtxDirty = false;
        }
    };
    Trans.prototype.updateWorldMatrix = function (pwmtxDirty) {
        var needUpdate = this.m_locaLTRSDirty || pwmtxDirty;
        if (needUpdate) {
            var p = this.m_parent;
            if (p == null) {
                this.m_worldMtx.set(this.localMtx);
                this.m_worldpos.set(this.m_localPosition);
                this.m_worldRotation.set(this.m_localRotation);
                this.m_worldScale.set(this.m_localScale);
            }
            else {
                var wmtx = p.worldMtx.mul(this.localMtx);
                this.m_worldMtx.set(wmtx);
                GLMath_1.mat4.DecomposeTRS(wmtx, this.m_worldpos, this.m_worldRotation, this.m_worldScale);
            }
            this.m_worldScaleDirty = false;
            this.m_worldPosDirty = false;
            this.m_worldRotaDirty = false;
        }
        this.m_locaLTRSDirty = false;
        this.m_worldMtxDirty = false;
    };
    Trans.prototype.removeChild = function (trs) {
        var children = this.m_children;
        if (children == null)
            return false;
        var index = children.indexOf(trs);
        if (index < 0)
            return false;
        trs.m_parent = null;
        trs.m_worldMtxDirty = true;
        trs.m_worldPosDirty = true;
        trs.m_worldRotaDirty = true;
        trs.m_worldScaleDirty = true;
        children.splice(index, 1);
        return true;
    };
    Trans.prototype.addChild = function (trs) {
        if (trs == null)
            return false;
        var children = this.m_children;
        if (children == null) {
            children = [];
            this.m_children = children;
        }
        var index = children.indexOf(trs);
        if (index >= 0)
            return true;
        trs.m_parent = this;
        trs.m_worldMtxDirty = true;
        trs.m_worldPosDirty = true;
        trs.m_worldRotaDirty = true;
        trs.m_worldScaleDirty = true;
        children.push(trs);
        return true;
    };
    return Trans;
}());
exports.Trans = Trans;
