import { vec3, quat, mat4 } from "./math/GLMath";
import { GameObject } from "./GameObject";
export declare class Transform {
    private m_localPosition;
    private m_localRotation;
    private m_localScale;
    private m_worldPositon;
    private m_worldRotation;
    private m_worldScale;
    private m_localTRSdirty;
    private m_TRSDirty;
    private m_needDecompose;
    /**
     * Snapshot of TRS dirty flag at the end of the last updated frame.
     */
    private m_frameTRSDirty;
    private m_localMtx;
    /** localToWorld mtx */
    private m_objMtx;
    /** worldToLocal mtx */
    private m_objMtxInv;
    private m_right;
    private m_forward;
    private m_up;
    private m_children;
    private m_parent;
    private m_gameobj;
    constructor(gobj: GameObject);
    readonly gameobject: GameObject;
    parent: Transform;
    localMatrix: mat4;
    /**
     * Model matrix, MATRIX_M
     */
    readonly objMatrix: mat4;
    readonly worldToLocalMatrix: mat4;
    private calObjMatrix;
    /**
     * world-space position
     */
    readonly position: vec3;
    /**
     * world-space rotation
     */
    readonly rotation: quat;
    /**
     * world-space scale
     */
    readonly lossyScale: vec3;
    readonly children: Transform[];
    /**
     * local rotation
     */
    localRotation: quat;
    localPosition: vec3;
    localScale: vec3;
    /** set local rotation */
    setRotation(q: quat): void;
    /** set local position */
    setPosition(pos: vec3): void;
    /** set local scale */
    setScale(scale: vec3): void;
    readonly worldForward: vec3;
    readonly worldUp: vec3;
    readonly worldRight: vec3;
    /**
     * local forward
     */
    forward: vec3;
    /**
     * local up
     */
    up: vec3;
    /**
     * local right
     */
    right: vec3;
    addChild(trs: Transform): boolean;
    removeChild(trs: Transform): boolean;
    setLocalDirty(dirty?: boolean): void;
    /**
     *
     * @param pmtxdirty parent's transform dirty?
     */
    setObjMatrixDirty(pmtxdirty: boolean): void;
    readonly isDirty: boolean;
    setLookAt(target: vec3, worldup?: vec3): void;
    setLookDir(forward: vec3, worldup: vec3): void;
    /**
     * Apply translate to current transform.
     * @param offset
     */
    applyTranslate(offset: vec3, local?: boolean): void;
    /**
     * Apply rotation to current transform.
     * @param q
     */
    applyRotate(q: quat): void;
    /**
     * Apply scale to current transform.
     * @param scale
     */
    applyScale(scale: vec3): void;
}
