"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var GLMath_1 = require("./math/GLMath");
var Transform = /** @class */ (function () {
    function Transform(gobj) {
        this.m_localPosition = GLMath_1.vec3.zero;
        this.m_localRotation = GLMath_1.quat.Identity;
        this.m_localScale = GLMath_1.vec3.one;
        this.m_worldPositon = GLMath_1.vec3.zero;
        this.m_worldRotation = GLMath_1.quat.Identity;
        this.m_worldScale = GLMath_1.vec3.one;
        this.m_localTRSdirty = true;
        this.m_TRSDirty = false;
        this.m_needDecompose = false;
        /**
         * Snapshot of TRS dirty flag at the end of the last updated frame.
         */
        this.m_frameTRSDirty = false;
        this.m_localMtx = GLMath_1.mat4.Identity;
        /** localToWorld mtx */
        this.m_objMtx = GLMath_1.mat4.Identity;
        this.m_right = GLMath_1.vec3.right;
        this.m_forward = GLMath_1.vec3.forward;
        this.m_up = GLMath_1.vec3.up;
        this.m_gameobj = gobj;
    }
    Object.defineProperty(Transform.prototype, "gameobject", {
        get: function () {
            return this.m_gameobj;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Transform.prototype, "parent", {
        get: function () {
            return this.m_parent;
        },
        set: function (p) {
            if (p == null) {
                var curp = this.m_parent;
                if (curp != null) {
                    curp.removeChild(this);
                }
            }
            else {
                p.addChild(this);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Transform.prototype, "localMatrix", {
        get: function () {
            if (this.m_localTRSdirty == true) {
                this.m_localMtx.setTRS(this.localPosition, this.localRotation, this.localScale);
                this.m_localTRSdirty = false;
                this.m_TRSDirty = true;
            }
            return this.m_localMtx;
        },
        set: function (mat) {
            this.m_localTRSdirty = false;
            this.m_localMtx = mat;
            GLMath_1.mat4.DecomposeTRS(mat, this.m_localPosition, this.m_localRotation, this.m_localScale);
            this.m_TRSDirty = true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Transform.prototype, "objMatrix", {
        /**
         * Model matrix, MATRIX_M
         */
        get: function () {
            this.calObjMatrix();
            return this.m_objMtx;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Transform.prototype, "worldToLocalMatrix", {
        get: function () {
            if (this.m_objMtxInv == null) {
                this.m_objMtxInv = this.objMatrix.inverse();
            }
            return this.m_objMtxInv;
        },
        enumerable: true,
        configurable: true
    });
    Transform.prototype.calObjMatrix = function (decompose) {
        if (decompose === void 0) { decompose = false; }
        if (this.m_TRSDirty) {
            if (this.parent == null) {
                this.m_objMtx.set(this.localMatrix);
                this.m_worldPositon = this.m_localPosition.clone();
                this.m_worldRotation = this.m_localRotation.clone();
                this.m_worldScale = this.m_localScale.clone();
                this.m_needDecompose = false;
            }
            else {
                var objmtx = this.parent.objMatrix.mul(this.localMatrix);
                this.m_objMtx.set(objmtx);
                this.m_needDecompose = true;
            }
            this.m_objMtxInv = null;
            this.m_TRSDirty = false;
        }
        if (decompose && this.m_needDecompose) {
            var sk = GLMath_1.vec3.zero;
            var rotamtx = new GLMath_1.mat3();
            GLMath_1.mat4.DecomposeAffine(this.m_objMtx, this.m_worldPositon, rotamtx, this.m_worldScale, sk);
            this.m_worldRotation = GLMath_1.quat.MtxToQuat(rotamtx);
            this.m_needDecompose = false;
        }
    };
    Object.defineProperty(Transform.prototype, "position", {
        /**
         * world-space position
         */
        get: function () {
            this.calObjMatrix(true);
            return this.m_worldPositon;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Transform.prototype, "rotation", {
        /**
         * world-space rotation
         */
        get: function () {
            this.calObjMatrix(true);
            return this.m_worldRotation;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Transform.prototype, "lossyScale", {
        /**
         * world-space scale
         */
        get: function () {
            this.calObjMatrix(true);
            return this.m_localScale;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Transform.prototype, "children", {
        get: function () {
            return this.m_children;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Transform.prototype, "localRotation", {
        /**
         * local rotation
         */
        get: function () {
            return this.m_localRotation;
        },
        set: function (q) {
            this.setRotation(q);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Transform.prototype, "localPosition", {
        get: function () {
            return this.m_localPosition;
        },
        set: function (pos) {
            this.setPosition(pos);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Transform.prototype, "localScale", {
        get: function () {
            return this.m_localScale;
        },
        set: function (s) {
            this.setScale(s);
        },
        enumerable: true,
        configurable: true
    });
    /** set local rotation */
    Transform.prototype.setRotation = function (q) {
        this.m_localRotation.set(q);
        this.m_forward = null;
        this.m_up = null;
        this.m_right = null;
        this.m_localTRSdirty = true;
        this.m_TRSDirty = true;
    };
    /** set local position */
    Transform.prototype.setPosition = function (pos) {
        this.m_localPosition.set(pos);
        this.m_localTRSdirty = true;
        this.m_TRSDirty = true;
    };
    /** set local scale */
    Transform.prototype.setScale = function (scale) {
        this.m_localScale.set(scale);
        this.m_localTRSdirty = true;
        this.m_TRSDirty = true;
    };
    Object.defineProperty(Transform.prototype, "worldForward", {
        get: function () {
            return this.rotation.rota(GLMath_1.vec3.forward);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Transform.prototype, "worldUp", {
        get: function () {
            return this.rotation.rota(GLMath_1.vec3.up);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Transform.prototype, "worldRight", {
        get: function () {
            return this.rotation.rota(GLMath_1.vec3.right);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Transform.prototype, "forward", {
        /**
         * local forward
         */
        get: function () {
            if (this.m_forward == null) {
                this.m_forward = this.m_localRotation.rota(GLMath_1.vec3.forward);
            }
            return this.m_forward;
        },
        set: function (dir) {
            var len = dir.length;
            if (len == 0) {
                console.warn("can not set forward to " + dir);
                return;
            }
            var forward = dir.divToRef(len);
            var up = this.up;
            var right = GLMath_1.vec3.SafeCross(forward, up).normalize;
            up = forward.cross(right).normalize;
            this.m_localRotation = GLMath_1.quat.Coordinate(forward, up);
            this.m_forward = forward;
            this.m_up.set(up);
            this.m_right = right;
            this.m_localTRSdirty = true;
            this.m_TRSDirty = true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Transform.prototype, "up", {
        /**
         * local up
         */
        get: function () {
            if (this.m_up == null) {
                this.m_up = this.m_localRotation.rota(GLMath_1.vec3.up);
            }
            return this.m_up;
        },
        set: function (dir) {
            var len = dir.length;
            if (len == 0) {
                console.warn("can not set up to " + dir);
                return;
            }
            var up = dir.divToRef(len);
            var right = GLMath_1.vec3.SafeCross(up, this.forward).normalize;
            var forward = right.cross(up).normalize;
            this.m_localRotation = GLMath_1.quat.Coordinate(forward, up);
            this.m_up = up;
            this.m_forward.set(forward);
            this.m_right = right;
            this.m_localTRSdirty = true;
            this.m_TRSDirty = true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Transform.prototype, "right", {
        /**
         * local right
         */
        get: function () {
            if (this.m_right == null) {
                this.m_right = this.m_localRotation.rota(GLMath_1.vec3.right);
            }
            return this.m_right;
        },
        set: function (dir) {
            var len = dir.length;
            if (len == 0) {
                console.warn("can not set up to " + dir);
                return;
            }
            var right = dir.divToRef(len);
            var forward = GLMath_1.vec3.SafeCross(right, this.up).normalize;
            var up = forward.cross(right);
            this.m_localRotation = GLMath_1.quat.Coordinate(forward, up);
            this.m_up.set(up);
            this.m_forward = forward;
            this.right = right;
            this.m_localTRSdirty = true;
            this.m_TRSDirty = true;
        },
        enumerable: true,
        configurable: true
    });
    Transform.prototype.addChild = function (trs) {
        if (trs == null)
            return false;
        var children = this.m_children;
        if (children == null) {
            children = [];
            this.m_children = children;
        }
        var index = children.indexOf(trs);
        if (index >= 0)
            return true;
        trs.m_parent = this;
        children.push(trs);
        return true;
    };
    Transform.prototype.removeChild = function (trs) {
        var children = this.m_children;
        if (children == null)
            return false;
        var index = children.indexOf(trs);
        if (index < 0)
            return false;
        trs.m_parent = null;
        this.m_children = children.splice(index, 1);
        return true;
    };
    Transform.prototype.setLocalDirty = function (dirty) {
        if (dirty === void 0) { dirty = true; }
        if (dirty == this.m_localTRSdirty)
            return;
        this.m_localTRSdirty = dirty;
        if (dirty) {
            this.m_TRSDirty = true;
        }
        else {
            this.localMatrix;
        }
    };
    /**
     *
     * @param pmtxdirty parent's transform dirty?
     */
    Transform.prototype.setObjMatrixDirty = function (pmtxdirty) {
        var dirty = this.m_TRSDirty || pmtxdirty;
        this.m_frameTRSDirty = dirty;
        if (dirty) {
            this.m_TRSDirty = true;
            this.calObjMatrix(false);
        }
    };
    Object.defineProperty(Transform.prototype, "isDirty", {
        get: function () { return this.m_localTRSdirty || this.m_frameTRSDirty; },
        enumerable: true,
        configurable: true
    });
    Transform.prototype.setLookAt = function (target, worldup) {
        this.m_localTRSdirty = true;
        this.setLookDir(target.subToRef(this.m_localPosition), worldup);
    };
    Transform.prototype.setLookDir = function (forward, worldup) {
        this.m_localTRSdirty = true;
        var f = forward.normalized();
        if (worldup == null)
            worldup = this.up;
        var right = GLMath_1.vec3.SafeCross(worldup, f).normalize;
        var up = f.cross(right).normalize;
        this.m_up.set(up);
        this.m_right = right;
        this.m_forward = f;
        this.m_localRotation = GLMath_1.quat.Coordinate(f, up);
    };
    /**
     * Apply translate to current transform.
     * @param offset
     */
    Transform.prototype.applyTranslate = function (offset, local) {
        if (local === void 0) { local = true; }
        if (local) {
            this.localPosition.add(offset);
            this.setLocalDirty();
        }
        else {
            var p = this.m_parent;
            if (p == null) {
                this.localPosition.add(offset);
                this.setLocalDirty();
            }
            else {
                var m = p.worldToLocalMatrix;
                var localoff = m.mulvec(offset.vec4(0));
                this.localPosition.add(localoff);
                this.setLocalDirty();
            }
        }
    };
    /**
     * Apply rotation to current transform.
     * @param q
     */
    Transform.prototype.applyRotate = function (q) {
        this.localRotation.selfRota(q);
        this.m_forward.mul(q);
        this.m_up.mul(q);
        this.m_right.mul(q);
        this.setLocalDirty();
    };
    /**
     * Apply scale to current transform.
     * @param scale
     */
    Transform.prototype.applyScale = function (scale) {
        this.m_localScale.mul(scale);
        this.setLocalDirty();
    };
    return Transform;
}());
exports.Transform = Transform;
