/// <reference types="webgl2" />
import { GLProgram } from "./GLProgram";
import { GLFrameBuffer } from "./GLFrameBuffer";
import { GLPipelineState } from "./GLPipelineState";
import { GLFenceSync } from "./GLFenceSync";
export declare class GLContext {
    private m_glFenceSynces;
    gl: WebGL2RenderingContext;
    private constructor();
    static createFromGL(wgl: WebGL2RenderingContext): GLContext;
    static createFromCanvas(canvas: HTMLCanvasElement, attrib?: WebGLContextAttributes): GLContext | null;
    createProgram(vsource: string, psource: string): GLProgram | null;
    createTextureImage(src: string, callback?: () => void): WebGLTexture | null;
    createTextureImageAsync(src: string, min_filter?: number, mag_filter?: number): Promise<WebGLTexture>;
    createTexture(internalFormat: number, width: number, height: number, linear?: boolean, mipmap?: boolean): WebGLTexture | null;
    createFrameBuffer(retain: boolean, colorInternalFormat: number, depthInternalFormat?: number, width?: number, height?: number, glfb?: GLFrameBuffer): GLFrameBuffer | null;
    private m_tempFrameBuffer;
    saveTextureToImage(texture: WebGLTexture): HTMLImageElement | null;
    saveCurrentFrameBufferToImage(x?: number, y?: number, w?: number | null, h?: number | null): HTMLImageElement;
    savePipeline(...type: number[]): GLPipelineState;
    restorePipeline(state: GLPipelineState): void;
    /**
     * Do not call this function explicitly
     * @param fs
     */
    registFenceSync(fs: GLFenceSync): void;
    /**
     * Do not call this function explicitly
     * @param fs
     */
    unregistFenceSync(fs: GLFenceSync): boolean;
    /**
     * check statu of all registed GLFenceScene
     */
    checkAllFenceSync(): void;
}
