"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var GLProgram_1 = require("./GLProgram");
var GLFrameBuffer_1 = require("./GLFrameBuffer");
var GLPipelineState_1 = require("./GLPipelineState");
var GLContext = /** @class */ (function () {
    function GLContext(wgl) {
        this.m_glFenceSynces = [];
        this.gl = wgl;
    }
    GLContext.createFromGL = function (wgl) {
        return new GLContext(wgl);
    };
    GLContext.createFromCanvas = function (canvas, attrib) {
        var g = canvas.getContext('webgl2', attrib);
        if (g == null) {
            g = canvas.getContext('webgl');
        }
        if (g == null)
            return null;
        return new GLContext(g);
    };
    GLContext.prototype.createProgram = function (vsource, psource) {
        var gl = this.gl;
        var vs = gl.createShader(gl.VERTEX_SHADER);
        gl.shaderSource(vs, vsource);
        gl.compileShader(vs);
        if (!gl.getShaderParameter(vs, gl.COMPILE_STATUS)) {
            console.error(vsource);
            console.error('compile vertex shader failed: ' + gl.getShaderInfoLog(vs));
            gl.deleteShader(vs);
            return null;
        }
        var ps = gl.createShader(gl.FRAGMENT_SHADER);
        gl.shaderSource(ps, psource);
        gl.compileShader(ps);
        if (!gl.getShaderParameter(ps, gl.COMPILE_STATUS)) {
            console.error(psource);
            console.error('compile fragment shader failed: ' + gl.getShaderInfoLog(ps));
            gl.deleteShader(ps);
            return null;
        }
        var program = gl.createProgram();
        gl.attachShader(program, vs);
        gl.attachShader(program, ps);
        gl.linkProgram(program);
        if (!gl.getProgramParameter(program, gl.LINK_STATUS)) {
            console.error('link shader program failed!:' + gl.getProgramInfoLog(program));
            gl.deleteProgram(program);
            gl.deleteShader(vs);
            gl.deleteShader(ps);
            return null;
        }
        if (program == null)
            return null;
        var p = new GLProgram_1.GLProgram(gl, program);
        return p;
    };
    GLContext.prototype.createTextureImage = function (src, callback) {
        var img = new Image();
        var gl = this.gl;
        var tex = gl.createTexture();
        img.onload = function () {
            gl.bindTexture(gl.TEXTURE_2D, tex);
            gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGB, gl.RGB, gl.UNSIGNED_BYTE, img);
            gl.generateMipmap(gl.TEXTURE_2D);
            gl.bindTexture(gl.TEXTURE_2D, null);
            console.log('init webgl texture');
            if (callback != null)
                callback();
        };
        img.src = src;
        return tex;
    };
    GLContext.prototype.createTextureImageAsync = function (src, min_filter, mag_filter) {
        return __awaiter(this, void 0, void 0, function () {
            var gl;
            return __generator(this, function (_a) {
                gl = this.gl;
                return [2 /*return*/, new Promise(function (res, rej) {
                        var img = new Image();
                        img.onload = function () {
                            var tex = gl.createTexture();
                            try {
                                gl.bindTexture(gl.TEXTURE_2D, tex);
                                gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGB, gl.RGB, gl.UNSIGNED_BYTE, img);
                                if (min_filter == null) {
                                    min_filter = gl.LINEAR_MIPMAP_LINEAR;
                                }
                                if (mag_filter == null) {
                                    mag_filter = gl.LINEAR;
                                }
                                gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, mag_filter);
                                gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, min_filter);
                                gl.generateMipmap(gl.TEXTURE_2D);
                                gl.bindTexture(gl.TEXTURE_2D, null);
                                res(tex);
                            }
                            catch (e) {
                                gl.deleteTexture(tex);
                                rej(e);
                            }
                        };
                        img.onerror = function (ev) {
                            rej(ev);
                        };
                        img.src = src;
                    })];
            });
        });
    };
    GLContext.prototype.createTexture = function (internalFormat, width, height, linear, mipmap) {
        if (linear === void 0) { linear = false; }
        if (mipmap === void 0) { mipmap = false; }
        var gl = this.gl;
        var tex = gl.createTexture();
        gl.bindTexture(gl.TEXTURE_2D, tex);
        gl.texStorage2D(gl.TEXTURE_2D, 1, internalFormat, width, height);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, linear ? gl.LINEAR : gl.NEAREST);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, linear ? (mipmap ? gl.LINEAR_MIPMAP_LINEAR : gl.LINEAR) : gl.NEAREST);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
        if (mipmap)
            gl.generateMipmap(gl.TEXTURE_2D);
        return tex;
    };
    GLContext.prototype.createFrameBuffer = function (retain, colorInternalFormat, depthInternalFormat, width, height, glfb) {
        return GLFrameBuffer_1.GLFrameBuffer.create(retain, this, colorInternalFormat, depthInternalFormat, width, height, glfb);
    };
    GLContext.prototype.saveTextureToImage = function (texture) {
        if (texture == null)
            return null;
        var gl = this.gl;
        if (this.m_tempFrameBuffer == null) {
            this.m_tempFrameBuffer = gl.createFramebuffer();
        }
        var curfb = gl.getParameter(gl.FRAMEBUFFER_BINDING);
        var curtex = gl.getParameter(gl.TEXTURE_BINDING_2D);
        gl.bindFramebuffer(gl.FRAMEBUFFER, this.m_tempFrameBuffer);
        gl.bindTexture(gl.TEXTURE_2D, texture);
        gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, texture, 0);
        var image = this.saveCurrentFrameBufferToImage();
        gl.bindTexture(gl.TEXTURE_2D, curtex);
        gl.bindFramebuffer(gl.FRAMEBUFFER, curfb);
        return image;
    };
    GLContext.prototype.saveCurrentFrameBufferToImage = function (x, y, w, h) {
        if (x === void 0) { x = 0; }
        if (y === void 0) { y = 0; }
        if (w === void 0) { w = null; }
        if (h === void 0) { h = null; }
        var gl = this.gl;
        if (w == null || h == null) {
            var canvas = gl.canvas;
            w = canvas.width;
            h = canvas.height;
        }
        var data = new Uint8Array(w * h * 4);
        gl.readPixels(x, y, w, h, gl.RGBA, gl.UNSIGNED_BYTE, data);
        var tempcanvas = document.createElement('canvas');
        tempcanvas.width = w;
        tempcanvas.height = h;
        var ctx2d = tempcanvas.getContext('2d');
        var imgdata = ctx2d.createImageData(w, h);
        imgdata.data.set(data);
        ctx2d.putImageData(imgdata, 0, 0);
        var img = new Image();
        img.src = tempcanvas.toDataURL();
        return img;
    };
    GLContext.prototype.savePipeline = function () {
        var type = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            type[_i] = arguments[_i];
        }
        return new (GLPipelineState_1.GLPipelineState.bind.apply(GLPipelineState_1.GLPipelineState, [void 0, this.gl].concat(type)))();
    };
    GLContext.prototype.restorePipeline = function (state) {
        if (state == null)
            return;
        state.restore(this.gl);
    };
    /**
     * Do not call this function explicitly
     * @param fs
     */
    GLContext.prototype.registFenceSync = function (fs) {
        this.m_glFenceSynces.push(fs);
    };
    /**
     * Do not call this function explicitly
     * @param fs
     */
    GLContext.prototype.unregistFenceSync = function (fs) {
        var syncs = this.m_glFenceSynces;
        var index = syncs.indexOf(fs);
        if (index >= 0) {
            syncs.splice(index, 1);
            return true;
        }
        return false;
    };
    /**
     * check statu of all registed GLFenceScene
     */
    GLContext.prototype.checkAllFenceSync = function () {
        var syncs = this.m_glFenceSynces;
        var len = syncs.length;
        if (len == 0)
            return;
        var remains = [];
        var changed = false;
        for (var t = 0; t < len; t++) {
            var s = syncs[t];
            if (!s.checkSignaled(true)) {
                changed = true;
                remains.push(s);
            }
        }
        if (changed) {
            this.m_glFenceSynces = remains;
        }
    };
    return GLContext;
}());
exports.GLContext = GLContext;
