"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Wrapper of WebGLSync object and gl.fenceSync.
 */
var GLFenceSync = /** @class */ (function () {
    /**
     *
     * @param glctx
     * @param autoCheckEmit GLContext will check fenceSyncStatus and emit callback when GLCtx call @function <checkAllFenceSync>
     */
    function GLFenceSync(glctx, autoCheckEmit) {
        if (autoCheckEmit === void 0) { autoCheckEmit = false; }
        this.m_signaled = false;
        this.m_released = false;
        this.glctx = glctx;
        this.m_autoCheck = autoCheckEmit;
    }
    Object.defineProperty(GLFenceSync.prototype, "pending", {
        get: function () {
            return this.m_callback != null;
        },
        enumerable: true,
        configurable: true
    });
    GLFenceSync.prototype.emit = function (onsignaled) {
        if (this.m_released)
            throw new Error('glFenceSync has beend releasd!');
        if (onsignaled == null)
            throw new Error('sync callback must not be error.');
        if (this.m_callback != null) {
            console.error("emit must wait until previous sync signaled.");
            return;
        }
        this.m_callback = onsignaled;
        var gl = this.glctx.gl;
        this.m_signaled = false;
        this.m_syncObj = gl.fenceSync(gl.SYNC_GPU_COMMANDS_COMPLETE, 0);
        if (this.m_autoCheck) {
            //register autocheck
            this.glctx.registFenceSync(this);
        }
    };
    GLFenceSync.prototype.release = function () {
        if (this.m_released)
            return;
        var glctx = this.glctx;
        if (this.m_syncObj != null) {
            glctx.unregistFenceSync(this);
            glctx.gl.deleteSync(this.m_syncObj);
            this.m_syncObj = null;
        }
        this.glctx = null;
        this.m_callback = null;
        this.m_released = true;
    };
    GLFenceSync.prototype.checkSignaled = function (autoEmitCallback) {
        if (autoEmitCallback === void 0) { autoEmitCallback = true; }
        var sync = this.m_syncObj;
        if (sync == null)
            return this.m_signaled;
        var gl = this.glctx.gl;
        var status = gl.getSyncParameter(sync, gl.SYNC_STATUS);
        if (status == gl.SIGNALED) {
            gl.deleteSync(sync);
            this.m_syncObj = null;
            this.m_signaled = true;
            if (autoEmitCallback) {
                this.m_callback();
                this.m_callback = null;
            }
            if (this.m_autoCheck) {
                this.glctx.unregistFenceSync(this);
            }
        }
        return this.m_signaled;
    };
    return GLFenceSync;
}());
exports.GLFenceSync = GLFenceSync;
