"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var GLFrameBuffer = /** @class */ (function () {
    function GLFrameBuffer() {
        this.m_valid = false;
    }
    GLFrameBuffer.create = function (retain, glctx, colorInternalFormat, depthInternalFormat, width, height, glfb) {
        var gl = glctx.gl;
        if (width == null)
            width = gl.canvas.width;
        if (height == null)
            height = gl.canvas.height;
        if (glfb == null) {
            glfb = new GLFrameBuffer();
        }
        else {
            if (glfb.isvalid) {
                glfb.release(glctx);
            }
        }
        var fb = gl.createFramebuffer();
        if (fb == null)
            return null;
        var state = retain ? glctx.savePipeline(gl.FRAMEBUFFER_BINDING, gl.TEXTURE_BINDING_2D) : null;
        gl.bindFramebuffer(gl.FRAMEBUFFER, fb);
        var colortex = gl.createTexture();
        gl.bindTexture(gl.TEXTURE_2D, colortex);
        gl.texStorage2D(gl.TEXTURE_2D, 1, colorInternalFormat, width, height);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR);
        gl.bindTexture(gl.TEXTURE_2D, null);
        gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, colortex, 0);
        var depthTex = null;
        if (depthInternalFormat != null) {
            depthTex = gl.createTexture();
            gl.bindTexture(gl.TEXTURE_2D, depthTex);
            gl.texStorage2D(gl.TEXTURE_2D, 1, depthInternalFormat, width, height);
            gl.bindTexture(gl.TEXTURE_2D, null);
            gl.framebufferTexture2D(gl.FRAMEBUFFER, gl.DEPTH_ATTACHMENT, gl.TEXTURE_2D, depthTex, 0);
        }
        gl.bindFramebuffer(gl.FRAMEBUFFER, null);
        glfb.colorTex0 = colortex;
        glfb.depthTex0 = depthTex;
        glfb.frambuffer = fb;
        glfb.width = width;
        glfb.height = height;
        glfb.colorFormat = colorInternalFormat;
        glfb.depthFormat = depthInternalFormat;
        if (state != null) {
            glctx.restorePipeline(state);
        }
        glfb.m_valid = true;
        return glfb;
    };
    GLFrameBuffer.prototype.release = function (glctx) {
        var gl = glctx.gl;
        gl.deleteFramebuffer(this.frambuffer);
        this.frambuffer = null;
        gl.deleteTexture(this.colorTex0);
        gl.deleteTexture(this.depthTex0);
        this.colorTex0 = null;
        this.depthTex0 = null;
        this.m_valid = false;
    };
    Object.defineProperty(GLFrameBuffer.prototype, "isvalid", {
        get: function () {
            return this.m_valid;
        },
        enumerable: true,
        configurable: true
    });
    GLFrameBuffer.prototype.bind = function (gl) {
        gl.bindFramebuffer(gl.FRAMEBUFFER, this.frambuffer);
    };
    return GLFrameBuffer;
}());
exports.GLFrameBuffer = GLFrameBuffer;
