"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var GLFenceSync_1 = require("./GLFenceSync");
var GLPixelPack = /** @class */ (function () {
    function GLPixelPack(glctx, buffersize, dynamic) {
        if (dynamic === void 0) { dynamic = true; }
        var gl = glctx.gl;
        var pb = gl.createBuffer();
        gl.bindBuffer(gl.PIXEL_PACK_BUFFER, pb);
        gl.bufferData(gl.PIXEL_PACK_BUFFER, buffersize, dynamic ? gl.DYNAMIC_READ : gl.STATIC_READ);
        gl.bindBuffer(gl.PIXEL_PACK_BUFFER, null);
        this.m_pb = pb;
        this.glctx = glctx;
    }
    GLPixelPack.prototype.release = function () {
        if (this.glctx == null)
            return;
        var gl = this.glctx.gl;
        if (this.m_pb != null) {
            gl.deleteBuffer(this.m_pb);
            this.m_pb = null;
        }
        this.glctx = null;
    };
    GLPixelPack.prototype.readPixelsSync = function (rect, format, type, dstbuffer, dstoffset, length) {
        if (dstoffset === void 0) { dstoffset = 0; }
        var pb = this.m_pb;
        var gl = this.glctx.gl;
        gl.bindBuffer(gl.PIXEL_PACK_BUFFER, pb);
        gl.readPixels(rect.x, rect.y, rect.z, rect.w, format, type, 0);
        gl.getBufferSubData(gl.PIXEL_PACK_BUFFER, 0, dstbuffer, dstoffset, length);
        gl.bindBuffer(gl.PIXEL_PACK_BUFFER, null);
    };
    GLPixelPack.prototype.readPixelsAsync = function (rect, format, type, dstbuffer, cb, dstoffset, length) {
        if (dstoffset === void 0) { dstoffset = 0; }
        var sync = this.m_sync;
        if (this.m_sync == null) {
            sync = new GLFenceSync_1.GLFenceSync(this.glctx, true);
            this.m_sync = sync;
        }
        else {
            if (sync.pending) {
                console.error("previous async readPixels operation is not finished, skip new operations");
                return;
            }
        }
        var pb = this.m_pb;
        var gl = this.glctx.gl;
        gl.bindBuffer(gl.PIXEL_PACK_BUFFER, pb);
        gl.readPixels(rect.x, rect.y, rect.z, rect.w, format, type, 0);
        sync.emit(function () {
            gl.bindBuffer(gl.PIXEL_PACK_BUFFER, pb);
            gl.getBufferSubData(gl.PIXEL_PACK_BUFFER, 0, dstbuffer, dstoffset, length);
            gl.bindBuffer(gl.PIXEL_PACK_BUFFER, null);
            cb(dstbuffer);
        });
    };
    return GLPixelPack;
}());
exports.GLPixelPack = GLPixelPack;
