declare type int = number;
export declare class GLTFaccessor {
    bufferView?: number;
    byteOffset: number;
    componentType: number;
    normalized: boolean;
    count: number;
    type: string;
    max?: number[];
    min?: number[];
    sparse?: GLTFsparse;
    name?: string;
    extensions?: object;
    extras?: any;
}
export declare class GLTFanimation {
    channels: GLTFchannel[];
    samplers: GLTFsampler[];
    name?: string;
    extensions?: object;
    extras?: any;
}
export declare class GLTFanimationSampler {
    input: int;
    interpolation: string;
    output: int;
    extensions?: object;
    extras?: any;
}
export declare class GLTFasset {
    copyright?: string;
    generator?: string;
    version: string;
    minVersion?: string;
    extensions?: object;
    extras?: any;
}
export declare class GLTFbuffer {
    uri?: string;
    byteLength: int;
    name?: string;
    extensions?: object;
    extras?: any;
}
export declare class GLTFbufferView {
    buffer: int;
    byteOffset: int;
    byteLength: int;
    byteStride?: int;
    target?: int;
    name?: string;
    extensions?: object;
    extras?: any;
}
export declare class GLTFcamera {
    orthographic?: GLTForthographic;
    perspective?: GLTFperspective;
    type: string;
    name?: string;
    extensions?: object;
    extras?: any;
}
export declare class GLTFchannel {
    sampler: int;
    target: GLTFtarget;
    extensions?: object;
    extras?: any;
}
export declare class GLTFimage {
    uri?: string;
    mimeType?: string;
    bufferView?: int;
    name?: string;
    extensions?: object;
    extras?: any;
}
export declare class GLTFindices {
    bufferView: int;
    byteOffset: int;
    componentType: int;
    extensions?: object;
    extras?: any;
}
export declare class GLTFmaterial {
    name?: string;
    extensions?: object;
    extras?: any;
    pbrMetallicRoughness?: GLTFpbrMetallicRoughness;
    normalTexture?: GLTFnormalTextureInfo;
    occlusionTexture?: GLTFocclusionTextureInfo;
    emissiveTexture?: GLTFtextureInfo;
    emissiveFactor?: number[];
    alphaMode: string;
    alphaCufOff?: number;
    doubleSided?: boolean;
}
export declare class GLTFmesh {
    primitives: GLTFprimitive[];
    weights?: number[];
    name?: string;
    extensions?: object;
    extras?: any;
}
export declare class GLTFnode {
    camera?: int;
    children?: int[];
    skin?: int;
    matrix: number[];
    mesh?: int;
    rotation: number[];
    scale: number[];
    translation: number[];
    weights?: number[];
    name?: string;
    extensions?: object;
    extras?: any;
}
export declare class GLTFnormalTextureInfo {
    index: int;
    texCoord?: int;
    scale?: number;
    extensions?: object;
    extras?: any;
}
export declare class GLTFocclusionTextureInfo {
    index: int;
    texCoord?: int;
    strength: number;
    extensions?: object;
    extras?: any;
}
export declare class GLTForthographic {
    xmag: number;
    ymag: number;
    zfar: number;
    znear: number;
    extensions?: object;
    extras?: any;
}
export declare class GLTFpbrMetallicRoughness {
    baseColorFactor: number[];
    baseColorTexture?: GLTFtextureInfo;
    metallicFactor: number;
    roughnessFactor: number;
    metallicRoughnessTexture?: GLTFtextureInfo;
    extensions?: object;
    extras?: any;
}
export declare class GLTFperspective {
    aspectRatio?: number;
    yfov: number;
    zfar?: number;
    znear?: number;
    extensions?: object;
    extras?: any;
}
export declare class GLTFprimitive {
    attributes: object;
    indices?: int;
    material?: int;
    mode: int;
    targets?: object[];
    extensions?: object;
    extras?: any;
}
export declare class GLTFsampler {
    magFilter?: int;
    minFilter?: int;
    wrapS: int;
    wrapT: int;
    name?: string;
    extensions?: object;
    extras?: any;
}
export declare class GLTFscene {
    nodes?: int[];
    name?: string;
    extensions?: object;
    extras?: any;
}
export declare class GLTFskin {
    inverseBindMatrices?: int;
    skeleton?: int;
    joints: int[];
    name?: string;
    extensions?: object;
    extras?: any;
}
export declare class GLTFsparse {
    count: int;
    indices: GLTFindices;
    values: GLTFvalues;
    extensions?: object;
    extras?: any;
}
export declare class GLTFtarget {
    node?: int;
    path: string;
    extensions?: object;
    extras?: any;
}
export declare class GLTFtexture {
    sampler?: int;
    sources?: int;
    name?: string;
    extensions?: object;
    extras?: any;
}
export declare class GLTFtextureInfo {
    index: int;
    texCoord: int;
    extensions?: object;
    extras?: any;
}
export declare class GLTFvalues {
    bufferView: int;
    byteOffset: int;
    extensions?: object;
    extras?: any;
}
export declare class GLTFfile {
    extensionsUsed?: string[];
    extensionsRequired?: string[];
    accessors?: GLTFaccessor[];
    animations?: GLTFanimation[];
    asset: GLTFasset;
    buffers?: GLTFbuffer[];
    bufferViews?: GLTFbufferView[];
    cameras?: GLTFcamera[];
    images?: GLTFimage[];
    materials?: GLTFmaterial[];
    meshes?: GLTFmesh[];
    nodes?: GLTFnode[];
    samplers?: GLTFsampler[];
    scene?: int;
    scenes?: GLTFscene[];
    skins?: GLTFskin[];
    textures?: GLTFtexture[];
    extensions?: object;
    extras?: any;
}
export declare class GLTFdata {
    gltf: GLTFfile;
    rawBinary: ArrayBuffer;
}
export declare class GLTFbinary {
    private constructor();
    static fromBuffer(arybuffer: ArrayBuffer): GLTFdata | undefined;
    private static parseChunk;
}
export declare class GLTFtool {
    static LoadGLTF(json: string, bin?: string, images?: string[]): Promise<GLTFdata>;
    static LoadGLTFBinary(uri: string): Promise<GLTFdata>;
}
export {};
