export declare class glmath {
    static vec3(x: number, y: number, z: number): vec3;
    static vec4(x: number, y: number, z: number, w: number): vec4;
    static vec2(x: number, y: number): vec2;
    static quat(x: number, y: number, z: number, w: number): quat;
    static clamp(v: number, min: number, max: number): number;
    static readonly Deg2Rad: number;
    static readonly Rad2Deg: number;
}
export declare class vec2 {
    raw: number[];
    x: number;
    y: number;
    readonly maginatude: number;
    constructor(v?: number[]);
    add(v: vec2): vec2;
    addToRef(v: vec2, ref?: vec2): vec2;
    sub(v: vec2): vec2;
    subToRef(v: vec2, ref?: vec2): vec2;
    mulNum(v: number): void;
    mulNumToRef(v: number, ref?: vec2): vec2;
    mul(v: vec2): void;
    mulToRef(v: vec2, ref?: vec2): vec2;
    div(v: vec2): void;
    divToRef(v: vec2, ref?: vec2): vec2;
    dot(v: vec2): number;
    clone(): vec2;
    readonly normalize: vec2;
    normalized(): vec2;
    set(v: vec2): void;
    static readonly zero: vec2;
    static readonly one: vec2;
}
export declare class vec4 {
    raw: number[];
    x: number;
    y: number;
    z: number;
    w: number;
    readonly length: number;
    constructor(v?: number[]);
    add(v: number | vec3 | vec4 | number[]): vec4;
    addToRef(v: number | vec3 | vec4 | number[]): vec4;
    sub(v: number | vec3 | vec4 | number[]): vec4;
    subToRef(v: number | vec3 | vec4 | number[]): vec4;
    mulNum(v: number): this;
    mulNumToRef(v: number): vec4;
    mul(v: number | vec4 | mat4 | number[] | quat): vec4;
    mulToRef(v: number | vec4 | mat4 | number[] | quat): vec4;
    div(v: number): this;
    divToRef(v: number): vec4;
    vec3(): vec3;
    dot(v: vec4): number;
    clone(): vec4;
    readonly normalize: vec4;
    /** return new vec4 ref */
    normalized(): vec4;
    static Random(): vec4;
    equals(v: vec4): boolean;
    set(v: vec4): void;
    static readonly zero: vec4;
    static readonly one: vec4;
}
export declare class vec3 {
    raw: number[];
    x: number;
    y: number;
    z: number;
    readonly length: number;
    readonly length2: number;
    constructor(v?: number[]);
    add(v: number | vec3 | vec4 | number[]): vec3;
    addToRef(v: number | vec3 | vec4 | number[]): vec3;
    sub(v: number | vec3 | vec4 | number[]): vec3;
    subToRef(v: number | vec3 | vec4 | number[]): vec3;
    /**
     * multiply a number
     * @param n
     */
    mulNum(n: number): vec3;
    mulNumToRef(n: number): vec3;
    mul(v: number | vec3 | vec4 | mat3 | number[] | quat): vec3;
    mulToRef(v: number | vec3 | vec4 | mat3 | number[] | quat): vec3;
    div(v: number): vec3;
    divToRef(v: number): vec3;
    vec4(w?: number): vec4;
    dot(v: vec3): number;
    clone(): vec3;
    /**
     * CrossProduct operation
     * @param rhs right hand side
     */
    cross(rhs: vec3): vec3;
    /**
     * Reverse CrossProduct operation
     * @param lhs left hand side
     */
    crossRev(lhs: vec3): vec3;
    static Cross(lhs: vec3, rhs: vec3): vec3;
    static SafeCross(lhs: vec3, rhs: vec3): vec3;
    static Dot(v1: vec3, v2: vec3): number;
    static Add(v1: vec3, v2: vec3): vec3;
    readonly normalize: vec3;
    normalized(): vec3;
    static Random(allpositive?: boolean): vec3;
    static RandomNorm(): vec3;
    static Abs(v: vec3): vec3;
    set(v: vec3): void;
    static readonly zero: vec3;
    static readonly one: vec3;
    static readonly up: vec3;
    static readonly forward: vec3;
    static readonly back: vec3;
    static readonly left: vec3;
    static readonly right: vec3;
    static readonly down: vec3;
}
export declare class quat {
    raw: number[];
    x: number;
    y: number;
    z: number;
    w: number;
    constructor(v?: number[]);
    conjugate(): quat;
    /**
     * flip all component when w is negative;
     */
    fmt(): this;
    mul(r: quat): quat;
    /**
     * Multiply self with quat <param l>, then return self
     * @param l
     */
    selfRota(l: quat): quat;
    /**
     * Identity quaternion [0,0,0,1]
     */
    static readonly Identity: quat;
    /**
     * Rotate vector by self
     * @param v
     */
    rota(v: vec3): vec3;
    /**
     * Convert euler angle (degree) to quaternion
     * width order Z-X-Y
     * @param degx
     * @param degy
     * @param degz
     */
    static fromEulerDeg(degx: number, degy: number, degz: number): quat;
    /**
     * Convert euler angle (radians) to quaternion
     * width order Z-X-Y
     * @param rx
     * @param ry
     * @param rz
     */
    static fromEuler(rx: number, ry: number, rz: number): quat;
    /**
     * Convert quaternion to Euler angle (radians).
     * Z-X-Y order
     */
    toEuler(): vec3;
    /**
     * Covert quaternion to Euler angle (degree).
     */
    toEulerDeg(): vec3;
    static axisRotation(axis: vec3, angle: number): quat;
    readonly axis: vec3;
    static axisRotationDeg(axis: vec3, deg: number): quat;
    /**
     * Calculate quaternion of rotation of vec3[from] -> vec3[to]
     * @param from
     * @param to
     * @param normal
     */
    static FromToNormal(from: vec3, to: vec3, normal: vec3): quat;
    static Coordinate(forward: vec3, up: vec3): quat;
    static QuatToMtx(q: quat): mat3;
    static Conjugate(q: quat): quat;
    /**
     * Flip coordinate axis of quaternion
     * @param q
     * @param xaxis
     * @param yaxis
     * @param zaxis
     */
    static Flip(q: quat, xaxis: boolean, yaxis: boolean, zaxis: boolean): quat;
    /**
     * div LHS quat by default
     * @param p
     * @param q
     * @param rhs false: p = rq, true: p =qr
     */
    static Div(p: quat, q: quat, rhs?: boolean): quat;
    static MtxToQuat(mtx: mat3): quat;
    equals(q: quat): boolean;
    magnitude2(): number;
    magnitude(): number;
    clone(): quat;
    static Random(): quat;
    set(q: quat): void;
}
export declare class mat4 {
    raw: number[];
    constructor(v?: number[]);
    column(index: number): vec4;
    row(index: number): vec4;
    static readonly Identity: mat4;
    static lookAt(eye: vec3, target: vec3, up: vec3): mat4;
    /**
     * Build coordinate change matrix RH->RH LH->LH
     * @param pos pos
     * @param forward dir
     * @param up dir
     */
    static coord(pos: vec3, forward: vec3, up: vec3): mat4;
    /**
     * Build matrix for coordinate conversion RH->LH LH->RH
     * @param pos pos
     * @param forward dir
     * @param up dir
     */
    static coordCvt(pos: vec3, forward: vec3, up: vec3): mat4;
    /**
     * Left Hand Coordinate
     * @param w
     * @param h
     * @param n
     * @param f
     */
    static perspective(w: number, h: number, n: number, f: number): mat4;
    /**
     * Left Hand Coordinate
     * @param fov
     * @param aspect
     * @param n
     * @param f
     */
    static perspectiveFoV(fov: number, aspect: number, n: number, f: number): mat4;
    /**
     * Left Hand coordinate
     * @param w
     * @param h
     * @param n
     * @param f
     */
    static orthographic(w: number, h: number, n: number, f: number): mat4;
    inverse(): mat4;
    static inverse(mtx: mat4): mat4;
    static Translate(v: vec3): mat4;
    static mul(mtx: mat4, v: vec4): vec4;
    add(m: mat4): mat4;
    static add(m1: mat4, m2: mat4): mat4;
    mulvec(v: vec4): vec4;
    mulnum(n: number): mat4;
    static Scale(scale: vec3): mat4;
    static Rotation(q: quat): mat4;
    static RotationEler(rad: vec3): mat4;
    static TRS(translate: vec3, rota: quat, scale: vec3): mat4;
    setTRS(translate: vec3, rota: quat, scale: vec3): void;
    set(mtx: mat4): void;
    mul(rhs: mat4): mat4;
    transpose(): mat4;
    static Transpose(m: mat4): mat4;
    /**
     * Decompose mat4 into translation rotation and scale, No shear
     * @param mat
     * @param hasNegativeScale optimize when scale components all positive.
     * @returns [T,R,S]
     */
    static Decompose(mat: mat4, hasNegativeScale?: boolean): [vec3, quat, vec3];
    static DecomposeTRS(mat: mat4, t: vec3, q: quat, s: vec3): void;
    /**
     * Decompose Affine Matrix
     * https://matthew-brett.github.io/transforms3d/reference/transforms3d.affines.html#decompose44?tdsourcetag=s_pctim_aiomsg
     * @param mat
     * @param t translate
     * @param q rotation
     * @param s zoom [sx,sy,sz]
     * @param sk skew [sxy,sxz,syz]
     */
    static DecomposeAffine(mat: mat4, t: vec3, q: mat3, s: vec3, sk?: vec3): void;
    clone(): mat4;
    static RandomTRS(): mat4;
    mat3Determinant(): number;
    toMat3(): mat3;
}
export declare class mat3 {
    raw: number[];
    constructor(v?: number[]);
    /**
     * Get column of matrix
     * @param index
     */
    column(index: number): vec3;
    setColumn(c: vec3, index: number): void;
    /**
     * Get row of matrix
     * @param index
     */
    row(index: number): vec3;
    setRow(r: vec3, index: number): void;
    static readonly Identity: mat3;
    static Transpose(m: mat3): mat3;
    transpose(): mat3;
    toMat4(): mat4;
    static ToMat4(m: mat3): mat4;
    static Cross(lhs: vec3): mat3;
    static Diagonal(v: vec3): mat3;
    static UpperTri(v: vec3): mat3;
    /**
     * Create mat3 from composition of scaling and rotation.
     * @param rota
     * @param s
     */
    static fromRS(rota: quat, s: vec3): mat3;
    static Mul(lhs: mat3, rhs: mat3): mat3;
    static fromRows(r0: vec3, r1: vec3, r2: vec3): mat3;
    static fromColumns(c0: vec3, c1: vec3, c2: vec3): mat3;
    mul(rhs: mat3): mat3;
    static CrossRHS(rhs: vec3): mat3;
    mulvec(v: vec3): vec3;
    static MulVec(mat: mat3, v: vec3): vec3;
    static Scale(sx: number, sy: number, sz: number): mat3;
    static Shear(sk: vec3): mat3;
    /**
     * Scale(s) * Shear(sk)
     * @param s
     * @param sk
     */
    static ScaleShear(s: vec3, sk: vec3): mat3;
    /**
     * Convert quaternion rotation to Matrix
     * @param q
     */
    static Rotation(q: quat): mat3;
    clone(): mat3;
    determinant(): number;
    /**
     * Decompose Affine matrix
     * @param mat
     * @param q Rotation matrix
     * @param s Diagonal matrix
     * @param sk Shear matrix
     */
    static DecomposeAffine(mat: mat3, q: mat3, s: vec3, sk?: vec3): void;
    static DecomposeRS(mat: mat3, q: quat, s: vec3): void;
}
