"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var DEG2RAD_HALF = Math.PI / 360.0;
var DEG2RAD = Math.PI / 180.0;
var RAD2DEG = 180.0 / Math.PI;
var glmath = /** @class */ (function () {
    function glmath() {
    }
    glmath.vec3 = function (x, y, z) {
        return new vec3([x, y, z]);
    };
    glmath.vec4 = function (x, y, z, w) {
        return new vec4([x, y, z, w]);
    };
    glmath.vec2 = function (x, y) {
        return new vec2([x, y]);
    };
    glmath.quat = function (x, y, z, w) {
        return new quat([x, y, z, w]);
    };
    glmath.clamp = function (v, min, max) {
        return v > max ? max : (v < min ? min : v);
    };
    glmath.Deg2Rad = DEG2RAD;
    glmath.Rad2Deg = RAD2DEG;
    return glmath;
}());
exports.glmath = glmath;
var vec2 = /** @class */ (function () {
    function vec2(v) {
        this.raw = [0, 0];
        if (v != null) {
            this.raw[0] = v[0];
            this.raw[1] = v[1];
        }
    }
    Object.defineProperty(vec2.prototype, "x", {
        get: function () { return this.raw[0]; },
        set: function (v) { this.raw[0] = v; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(vec2.prototype, "y", {
        get: function () { return this.raw[1]; },
        set: function (v) { this.raw[1] = v; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(vec2.prototype, "maginatude", {
        get: function () {
            var x = this.raw[0];
            var y = this.raw[1];
            return Math.sqrt(x * x + y * y);
        },
        enumerable: true,
        configurable: true
    });
    vec2.prototype.add = function (v) {
        this.x += v.x;
        this.y += v.y;
        return this;
    };
    vec2.prototype.addToRef = function (v, ref) {
        if (ref == null) {
            return new vec2([this.x + v.x, this.y + v.y]);
        }
        else {
            ref.x = this.x + v.x;
            ref.y = this.y + v.y;
            return ref;
        }
    };
    vec2.prototype.sub = function (v) {
        this.x -= v.x;
        this.y -= v.y;
        return this;
    };
    vec2.prototype.subToRef = function (v, ref) {
        if (ref == null) {
            return new vec2([this.x - v.x, this.y - v.y]);
        }
        else {
            ref.x = this.x - v.x;
            ref.y = this.y - v.y;
            return ref;
        }
    };
    vec2.prototype.mulNum = function (v) {
        this.x *= v;
        this.y *= v;
    };
    vec2.prototype.mulNumToRef = function (v, ref) {
        if (ref) {
            ref.x = this.x * v;
            ref.y = this.y * v;
            return ref;
        }
        return new vec2([this.x * v, this.y * v]);
    };
    vec2.prototype.mul = function (v) {
        this.x *= v.x;
        this.y *= v.y;
    };
    vec2.prototype.mulToRef = function (v, ref) {
        if (ref) {
            ref.x = this.x * v.x;
            ref.y = this.y * v.y;
            return ref;
        }
        return new vec2([this.x * v.x, this.y * v.y]);
    };
    vec2.prototype.div = function (v) {
        this.x /= v.x;
        this.y /= v.y;
    };
    vec2.prototype.divToRef = function (v, ref) {
        if (ref) {
            ref.x = this.x / v.x;
            ref.y = this.y / v.y;
            return ref;
        }
        return new vec2([this.x / v.x, this.y / v.y]);
    };
    vec2.prototype.dot = function (v) {
        return this.x * v.x + this.y * v.y;
    };
    vec2.prototype.clone = function () {
        return new vec2([this.x, this.y]);
    };
    Object.defineProperty(vec2.prototype, "normalize", {
        get: function () {
            this.mulNum(1.0 / this.maginatude);
            return this;
        },
        enumerable: true,
        configurable: true
    });
    vec2.prototype.normalized = function () {
        return this.mulNumToRef(this.maginatude);
    };
    vec2.prototype.set = function (v) {
        this.x = v.x;
        this.y = v.y;
    };
    Object.defineProperty(vec2, "zero", {
        get: function () { return new vec2(); },
        enumerable: true,
        configurable: true
    });
    ;
    Object.defineProperty(vec2, "one", {
        get: function () { return new vec2([1, 1]); },
        enumerable: true,
        configurable: true
    });
    ;
    return vec2;
}());
exports.vec2 = vec2;
var vec4 = /** @class */ (function () {
    function vec4(v) {
        if (v != null) {
            this.raw = v;
        }
        else {
            this.raw = [0, 0, 0, 0];
        }
    }
    Object.defineProperty(vec4.prototype, "x", {
        get: function () { return this.raw[0]; },
        set: function (v) { this.raw[0] = v; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(vec4.prototype, "y", {
        get: function () { return this.raw[1]; },
        set: function (v) { this.raw[1] = v; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(vec4.prototype, "z", {
        get: function () { return this.raw[2]; },
        set: function (v) { this.raw[2] = v; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(vec4.prototype, "w", {
        get: function () { return this.raw[3]; },
        set: function (v) { this.raw[3] = v; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(vec4.prototype, "length", {
        get: function () {
            return Math.sqrt(this.dot(this));
        },
        enumerable: true,
        configurable: true
    });
    vec4.prototype.add = function (v) {
        if (v instanceof vec3 || v instanceof vec4) {
            this.x += v.x;
            this.y += v.y;
            this.z += v.z;
            this.w += ((v instanceof vec4) ? v.w : 0);
        }
        else if (v instanceof Array) {
            this.x += v[0];
            this.y += v[1];
            this.z += v[2];
            this.w += v[3];
        }
        else {
            this.x += v;
            this.y += v;
            this.z += v;
            this.w += v;
        }
        return this;
    };
    vec4.prototype.addToRef = function (v) {
        if (v instanceof vec3 || v instanceof vec4) {
            var x = v.x + this.x;
            var y = v.y + this.y;
            var z = v.z + this.z;
            var w = ((v instanceof vec4) ? v.w : 0) + this.w;
            return glmath.vec4(x, y, z, w);
        }
        else if (v instanceof Array) {
            var x = v[0] + this.x;
            var y = v[1] + this.y;
            var z = v[2] + this.z;
            var w = v[3] + this.w;
            return glmath.vec4(x, y, z, w);
        }
        else {
            return glmath.vec4(this.x + v, this.y + v, this.z + v, this.w + v);
        }
    };
    vec4.prototype.sub = function (v) {
        if (v instanceof vec3 || v instanceof vec4) {
            this.x -= v.x;
            this.y -= v.y;
            this.z -= v.z;
            this.w -= ((v instanceof vec4) ? v.w : 0);
        }
        else if (v instanceof Array) {
            this.x -= v[0];
            this.y -= v[1];
            this.z -= v[2];
            this.w -= v[3];
        }
        else {
            this.x -= v;
            this.y -= v;
            this.z -= v;
            this.w -= v;
        }
        return this;
    };
    vec4.prototype.subToRef = function (v) {
        if (v instanceof vec3 || v instanceof vec4) {
            var x = v.x - this.x;
            var y = v.y - this.y;
            var z = v.z - this.z;
            var w = ((v instanceof vec4) ? v.w : 0) - this.w;
            return glmath.vec4(x, y, z, w);
        }
        else if (v instanceof Array) {
            var x = v[0] - this.x;
            var y = v[1] - this.y;
            var z = v[2] - this.z;
            var w = v[3] - this.w;
            return glmath.vec4(x, y, z, w);
        }
        else {
            return glmath.vec4(this.x - v, this.y - v, this.z - v, this.w - v);
        }
    };
    vec4.prototype.mulNum = function (v) {
        this.x *= v;
        this.y *= v;
        this.z *= v;
        this.w *= v;
        return this;
    };
    vec4.prototype.mulNumToRef = function (v) {
        return glmath.vec4(this.x * v, this.y * v, this.z * v, this.w * v);
    };
    vec4.prototype.mul = function (v) {
        if (v instanceof vec4) {
            this.x *= v.x;
            this.y *= v.y;
            this.z *= v.z;
            this.w *= v.w;
        }
        else if (v instanceof Array) {
            this.x *= v[0];
            this.y *= v[1];
            this.z *= v[2];
            this.w *= v[3];
        }
        else if (v instanceof mat4) {
            this.raw = v.mulvec(this).raw;
        }
        else if (v instanceof quat) {
            this.raw = v.rota(new vec3([this.x, this.y, this.z])).raw;
        }
        else {
            this.x *= v;
            this.y *= v;
            this.z *= v;
            this.w *= v;
        }
        return this;
    };
    vec4.prototype.mulToRef = function (v) {
        if (v instanceof vec4) {
            return glmath.vec4(v.x * this.x, v.y * this.y, v.z * this.z, v.w * this.w);
        }
        else if (v instanceof Array) {
            return glmath.vec4(v[0] * this.x, v[1] * this.y, v[2] * this.z, v[3] * this.w);
        }
        else if (v instanceof mat4) {
            return v.mulvec(this);
        }
        else if (v instanceof quat) {
            return null;
        }
        else {
            return glmath.vec4(this.x * v, this.y * v, this.z * v, this.w * v);
        }
    };
    vec4.prototype.div = function (v) {
        this.x /= v;
        this.y /= v;
        this.z /= v;
        this.w /= v;
        return this;
    };
    vec4.prototype.divToRef = function (v) {
        return glmath.vec4(this.x / v, this.y / v, this.z / v, this.w / v);
    };
    vec4.prototype.vec3 = function () {
        return glmath.vec3(this.x, this.y, this.z);
    };
    vec4.prototype.dot = function (v) {
        return this.x * v.x + this.y * v.y + this.z * v.z + this.w * v.w;
    };
    vec4.prototype.clone = function () {
        return new vec4([this.x, this.y, this.z, this.w]);
    };
    Object.defineProperty(vec4.prototype, "normalize", {
        get: function () {
            return this.div(this.length);
        },
        enumerable: true,
        configurable: true
    });
    /** return new vec4 ref */
    vec4.prototype.normalized = function () {
        return this.divToRef(this.length);
    };
    vec4.Random = function () {
        return new vec4([Math.random(), Math.random(), Math.random(), Math.random()]);
    };
    vec4.prototype.equals = function (v) {
        var r = this.raw;
        var rv = v.raw;
        for (var i = 0; i < 4; i++) {
            if (r[i] != rv[i])
                return false;
        }
        return true;
    };
    vec4.prototype.set = function (v) {
        this.x = v.x;
        this.y = v.y;
        this.z = v.z;
        this.w = v.w;
    };
    Object.defineProperty(vec4, "zero", {
        get: function () { return new vec4(); },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(vec4, "one", {
        get: function () { return new vec4([1, 1, 1, 1]); },
        enumerable: true,
        configurable: true
    });
    return vec4;
}());
exports.vec4 = vec4;
var vec3 = /** @class */ (function () {
    function vec3(v) {
        if (v != null) {
            this.raw = v;
        }
        else {
            this.raw = [0, 0, 0];
        }
    }
    Object.defineProperty(vec3.prototype, "x", {
        get: function () { return this.raw[0]; },
        set: function (v) { this.raw[0] = v; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(vec3.prototype, "y", {
        get: function () { return this.raw[1]; },
        set: function (v) { this.raw[1] = v; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(vec3.prototype, "z", {
        get: function () { return this.raw[2]; },
        set: function (v) { this.raw[2] = v; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(vec3.prototype, "length", {
        get: function () {
            return Math.sqrt(this.dot(this));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(vec3.prototype, "length2", {
        get: function () {
            var x = this.x;
            var y = this.y;
            var z = this.z;
            return x * x + y * y + z * z;
        },
        enumerable: true,
        configurable: true
    });
    vec3.prototype.add = function (v) {
        if (v instanceof vec3 || v instanceof vec4) {
            this.x += v.x;
            this.y += v.y;
            this.z += v.z;
        }
        else if (v instanceof Array) {
            this.x += v[0];
            this.x += v[1];
            this.x += v[2];
        }
        else {
            this.x += v;
            this.y += v;
            this.z += v;
        }
        return this;
    };
    vec3.prototype.addToRef = function (v) {
        if (v instanceof vec3 || v instanceof vec4) {
            var x = v.x + this.x;
            var y = v.y + this.y;
            var z = v.z + this.z;
            return glmath.vec3(x, y, z);
        }
        else if (v instanceof Array) {
            var x = v[0] + this.x;
            var y = v[1] + this.y;
            var z = v[2] + this.z;
            return glmath.vec3(x, y, z);
        }
        else {
            return glmath.vec3(this.x + v, this.y + v, this.z + v);
        }
    };
    vec3.prototype.sub = function (v) {
        if (v instanceof vec3 || v instanceof vec4) {
            this.x -= v.x;
            this.y -= v.y;
            this.z -= v.z;
        }
        else if (v instanceof Array) {
            this.x -= v[0];
            this.y -= v[1];
            this.z -= v[2];
        }
        else {
            this.x -= v;
            this.y -= v;
            this.z -= v;
        }
        return this;
    };
    vec3.prototype.subToRef = function (v) {
        if (v instanceof vec3 || v instanceof vec4) {
            var x = this.x - v.x;
            var y = this.y - v.y;
            var z = this.z - v.z;
            return glmath.vec3(x, y, z);
        }
        else if (v instanceof Array) {
            var x = this.x - v[0];
            var y = this.y - v[1];
            var z = this.z - v[2];
            return glmath.vec3(x, y, z);
        }
        else {
            return glmath.vec3(this.x - v, this.y - v, this.z - v);
        }
    };
    /**
     * multiply a number
     * @param n
     */
    vec3.prototype.mulNum = function (n) {
        this.x *= n;
        this.y *= n;
        this.z *= n;
        return this;
    };
    vec3.prototype.mulNumToRef = function (n) {
        return glmath.vec3(this.x * n, this.y * n, this.z * n);
    };
    vec3.prototype.mul = function (v) {
        if (v instanceof vec3 || v instanceof vec4) {
            this.x *= v.x;
            this.y *= v.y;
            this.z *= v.z;
        }
        else if (v instanceof Array) {
            this.x *= v[0];
            this.y *= v[1];
            this.z *= v[2];
        }
        else if (v instanceof mat3) {
            this.raw = v.mulvec(this).raw;
        }
        else if (v instanceof quat) {
            this.raw = v.rota(this).raw;
        }
        else {
            this.x *= v;
            this.y *= v;
            this.z *= v;
        }
        return this;
    };
    vec3.prototype.mulToRef = function (v) {
        if (v instanceof vec3 || v instanceof vec4) {
            var x = v.x * this.x;
            var y = v.y * this.y;
            var z = v.z * this.z;
            return glmath.vec3(x, y, z);
        }
        else if (v instanceof Array) {
            var x = v[0] * this.x;
            var y = v[1] * this.y;
            var z = v[2] * this.z;
            return glmath.vec3(x, y, z);
        }
        else if (v instanceof mat3) {
            return vec3.zero;
        }
        else if (v instanceof quat) {
            return v.rota(this);
        }
        else {
            return glmath.vec3(this.x * v, this.y * v, this.z * v);
        }
    };
    vec3.prototype.div = function (v) {
        this.x /= v;
        this.y /= v;
        this.z /= v;
        return this;
    };
    vec3.prototype.divToRef = function (v) {
        return glmath.vec3(this.x / v, this.y / v, this.z / v);
    };
    vec3.prototype.vec4 = function (w) {
        if (w === void 0) { w = 0; }
        return glmath.vec4(this.x, this.y, this.z, w);
    };
    vec3.prototype.dot = function (v) {
        return this.x * v.x + this.y * v.y + this.z * v.z;
    };
    vec3.prototype.clone = function () {
        return new vec3([this.x, this.y, this.z]);
    };
    /**
     * CrossProduct operation
     * @param rhs right hand side
     */
    vec3.prototype.cross = function (rhs) {
        return vec3.Cross(this, rhs);
    };
    /**
     * Reverse CrossProduct operation
     * @param lhs left hand side
     */
    vec3.prototype.crossRev = function (lhs) {
        return vec3.Cross(lhs, this);
    };
    vec3.Cross = function (lhs, rhs) {
        return new vec3([
            lhs.y * rhs.z - lhs.z * rhs.y,
            lhs.z * rhs.x - lhs.x * rhs.z,
            lhs.x * rhs.y - lhs.y * rhs.x
        ]);
    };
    vec3.SafeCross = function (lhs, rhs) {
        var c = new vec3([
            lhs.y * rhs.z - lhs.z * rhs.y,
            lhs.z * rhs.x - lhs.x * rhs.z,
            lhs.x * rhs.y - lhs.y * rhs.x
        ]);
        if (c.x == 0 && c.y == 0 && c.z == 0) {
            return this.Cross(lhs.addToRef([0.0000001, 0.0000001, 0.0000001]), rhs);
        }
        return c;
    };
    vec3.Dot = function (v1, v2) {
        return v1.x * v2.x + v1.y * v2.y + v1.z * v2.z;
    };
    vec3.Add = function (v1, v2) {
        return new vec3([v1.x + v2.x, v1.y + v2.y, v1.z + v2.z]);
    };
    Object.defineProperty(vec3.prototype, "normalize", {
        get: function () {
            return this.div(this.length);
        },
        enumerable: true,
        configurable: true
    });
    vec3.prototype.normalized = function () {
        return this.divToRef(this.length);
    };
    vec3.Random = function (allpositive) {
        if (allpositive === void 0) { allpositive = false; }
        if (allpositive) {
            return new vec3([Math.random(), Math.random(), Math.random()]);
        }
        else {
            return new vec3([Math.random() - 0.5, Math.random() - 0.5, Math.random() - 0.5]);
        }
    };
    vec3.RandomNorm = function () {
        return quat.Random().rota(vec3.right);
    };
    vec3.Abs = function (v) {
        return new vec3([Math.abs(v.x), Math.abs(v.y), Math.abs(v.z)]);
    };
    vec3.prototype.set = function (v) {
        this.x = v.x;
        this.y = v.y;
        this.z = v.z;
    };
    Object.defineProperty(vec3, "zero", {
        get: function () { return new vec3([0, 0, 0]); },
        enumerable: true,
        configurable: true
    });
    ;
    Object.defineProperty(vec3, "one", {
        get: function () { return new vec3([1, 1, 1]); },
        enumerable: true,
        configurable: true
    });
    ;
    Object.defineProperty(vec3, "up", {
        get: function () { return new vec3([0, 1, 0]); },
        enumerable: true,
        configurable: true
    });
    ;
    Object.defineProperty(vec3, "forward", {
        get: function () { return new vec3([0, 0, 1]); },
        enumerable: true,
        configurable: true
    });
    ;
    Object.defineProperty(vec3, "back", {
        get: function () { return new vec3([0, 0, -1]); },
        enumerable: true,
        configurable: true
    });
    ;
    Object.defineProperty(vec3, "left", {
        get: function () { return new vec3([-1, 0, 0]); },
        enumerable: true,
        configurable: true
    });
    ;
    Object.defineProperty(vec3, "right", {
        get: function () { return new vec3([1, 0, 0]); },
        enumerable: true,
        configurable: true
    });
    ;
    Object.defineProperty(vec3, "down", {
        get: function () { return new vec3([0, -1, 0]); },
        enumerable: true,
        configurable: true
    });
    return vec3;
}());
exports.vec3 = vec3;
var quat = /** @class */ (function () {
    function quat(v) {
        if (v != null) {
            this.raw = v;
        }
        else {
            this.raw = [0, 0, 0, 0];
        }
    }
    Object.defineProperty(quat.prototype, "x", {
        get: function () { return this.raw[0]; },
        set: function (v) { this.raw[0] = v; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(quat.prototype, "y", {
        get: function () { return this.raw[1]; },
        set: function (v) { this.raw[1] = v; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(quat.prototype, "z", {
        get: function () { return this.raw[2]; },
        set: function (v) { this.raw[2] = v; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(quat.prototype, "w", {
        get: function () { return this.raw[3]; },
        set: function (v) { this.raw[3] = v; },
        enumerable: true,
        configurable: true
    });
    quat.prototype.conjugate = function () {
        return quat.Conjugate(this);
    };
    /**
     * flip all component when w is negative;
     */
    quat.prototype.fmt = function () {
        if (this.w < 0) {
            var r = this.raw;
            r[0] = -r[0];
            r[1] = -r[1];
            r[2] = -r[2];
            r[3] = -r[3];
        }
        return this;
    };
    quat.prototype.mul = function (r) {
        var l = this;
        var rw = r.w;
        var rx = r.x;
        var ry = r.y;
        var rz = r.z;
        var lx = l.x;
        var ly = l.y;
        var lz = l.z;
        var lw = l.w;
        return new quat([
            rw * lx + lw * rx + ly * rz - lz * ry,
            rw * ly + lw * ry + lz * rx - lx * rz,
            rw * lz + lw * rz + lx * ry - ly * rx,
            rw * lw - lx * rx - ly * ry - lz * rz
        ]).fmt();
    };
    /**
     * Multiply self with quat <param l>, then return self
     * @param l
     */
    quat.prototype.selfRota = function (l) {
        var rw = this.w;
        var rx = this.x;
        var ry = this.y;
        var rz = this.z;
        var lx = l.x;
        var ly = l.y;
        var lz = l.z;
        var lw = l.w;
        this.x = rw * lx + lw * rx + ly * rz - lz * ry;
        this.y = rw * ly + lw * ry + lz * rx - lx * rz;
        this.z = rw * lz + lw * rz + lx * ry - ly * rx;
        this.w = rw * lw - lx * rx - ly * ry - lz * rz;
        return this;
    };
    Object.defineProperty(quat, "Identity", {
        /**
         * Identity quaternion [0,0,0,1]
         */
        get: function () {
            return new quat([
                0, 0, 0, 1
            ]);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Rotate vector by self
     * @param v
     */
    quat.prototype.rota = function (v) {
        var q = new vec3([this.x, this.y, this.z]);
        var t = q.cross(v).mul(2);
        return v.clone().add(t.mulToRef(this.w)).add(q.cross(t));
    };
    /**
     * Convert euler angle (degree) to quaternion
     * width order Z-X-Y
     * @param degx
     * @param degy
     * @param degz
     */
    quat.fromEulerDeg = function (degx, degy, degz) {
        var rx = degx * DEG2RAD_HALF;
        var ry = degy * DEG2RAD_HALF;
        var rz = degz * DEG2RAD_HALF;
        var cosx = Math.cos(rx);
        var cosy = Math.cos(ry);
        var cosz = Math.cos(rz);
        var sinx = Math.sin(rx);
        var siny = Math.sin(ry);
        var sinz = Math.sin(rz);
        return new quat([
            sinx * cosy * cosz + cosx * siny * sinz,
            cosx * siny * cosz - sinx * cosy * sinz,
            cosx * cosy * sinz - sinx * siny * cosz,
            cosx * cosy * cosz + sinx * siny * sinz
        ]);
    };
    /**
     * Convert euler angle (radians) to quaternion
     * width order Z-X-Y
     * @param rx
     * @param ry
     * @param rz
     */
    quat.fromEuler = function (rx, ry, rz) {
        var rxh = rx / 2.0;
        var ryh = ry / 2.0;
        var rzh = rz / 2.0;
        var cosx = Math.cos(rxh);
        var cosy = Math.cos(ryh);
        var cosz = Math.cos(rzh);
        var sinx = Math.sin(rxh);
        var siny = Math.sin(ryh);
        var sinz = Math.sin(rzh);
        return new quat([
            sinx * cosy * cosz + cosx * siny * sinz,
            cosx * siny * cosz - sinx * cosy * sinz,
            cosx * cosy * sinz - sinx * siny * cosz,
            cosx * cosy * cosz + sinx * siny * sinz
        ]);
    };
    /**
     * Convert quaternion to Euler angle (radians).
     * Z-X-Y order
     */
    quat.prototype.toEuler = function () {
        var v = new vec3();
        var x = this.z;
        var y = this.x;
        var z = this.y;
        var w = this.w;
        var t0 = 2.0 * (w * x + y * z);
        var t1 = 1.0 - 2.0 * (x * x + y * y);
        v.z = Math.atan2(t0, t1);
        var t2 = 2.0 * (w * y - z * x);
        if (t2 > 1.0) {
            t2 = 1.0;
        }
        else if (t2 < -1.0) {
            t2 = -1.0;
        }
        v.x = Math.asin(t2);
        var t3 = 2.0 * (w * z + x * y);
        var t4 = 1.0 - 2.0 * (y * y + z * z);
        v.y = Math.atan2(t3, t4);
        return v;
    };
    /**
     * Covert quaternion to Euler angle (degree).
     */
    quat.prototype.toEulerDeg = function () {
        var v = new vec3();
        var x = this.z;
        var y = this.x;
        var z = this.y;
        var w = this.w;
        var t0 = 2.0 * (w * x + y * z);
        var t1 = 1.0 - 2.0 * (x * x + y * y);
        v.z = Math.atan2(t0, t1) * RAD2DEG;
        var t2 = 2.0 * (w * y - z * x);
        if (t2 > 1.0) {
            t2 = 1.0;
        }
        else if (t2 < -1.0) {
            t2 = -1.0;
        }
        v.x = Math.asin(t2) * RAD2DEG;
        var t3 = 2.0 * (w * z + x * y);
        var t4 = 1.0 - 2.0 * (y * y + z * z);
        v.y = Math.atan2(t3, t4) * RAD2DEG;
        return v;
    };
    quat.axisRotation = function (axis, angle) {
        var d = 1.0 / axis.length;
        var sin = Math.sin(angle / 2);
        var cos = Math.cos(angle / 2);
        var v4 = axis.mulToRef(d * sin).vec4(cos);
        return new quat(v4.raw);
    };
    Object.defineProperty(quat.prototype, "axis", {
        get: function () {
            var magnitude = this.magnitude2();
            if (magnitude >= 1.0000001) {
                throw new Error();
            }
            var w = this.w;
            var sin = Math.sqrt(1.0 - w * w);
            return glmath.vec3(this.x / sin, this.y / sin, this.z / sin);
        },
        enumerable: true,
        configurable: true
    });
    quat.axisRotationDeg = function (axis, deg) {
        var angle = deg * DEG2RAD;
        var d = 1.0 / axis.length;
        var sin = Math.sin(angle / 2);
        var cos = Math.cos(angle / 2);
        var v4 = axis.mulToRef(d * sin).vec4(cos);
        return new quat(v4.raw);
    };
    /**
     * Calculate quaternion of rotation of vec3[from] -> vec3[to]
     * @param from
     * @param to
     * @param normal
     */
    quat.FromToNormal = function (from, to, normal) {
        var f = from.normalized();
        var t = to.normalized();
        var n = normal.normalized();
        var cross = vec3.Cross(f, t);
        var croosLen2 = cross.length2;
        if (croosLen2 == 0) {
            var dot = f.dot(t);
            if (dot == 1) {
                return quat.Identity;
            }
            var cr = vec3.Cross(n, f);
            var cu = vec3.Cross(f, cr);
            var nor = cu.normalize;
            return new quat([nor.x, nor.y, nor.z, 0]);
        }
        cross.div(Math.sqrt(croosLen2));
        var cos = f.dot(t);
        var cosh = Math.sqrt((1 + cos) / 2.0);
        var sinh = Math.sqrt((1 - cos) / 2.0);
        var cdotn = cross.dot(n);
        if (cdotn < 0) {
            cross.mul(-1.0);
            cosh *= -1.0;
        }
        return new quat([cross.x * sinh, cross.y * sinh, cross.z * sinh, cosh]);
    };
    quat.Coordinate = function (forward, up) {
        if (forward.dot(up) > Number.EPSILON) {
            throw new Error("<forward> must be perpendicular ot <up>");
        }
        var f = forward.normalized();
        var u = up.normalized();
        var qf = quat.FromToNormal(vec3.forward, f, u);
        var u1 = qf.rota(vec3.up);
        var qu = quat.FromToNormal(u1, up, f);
        return qu.mul(qf);
    };
    quat.QuatToMtx = function (q) {
        var x = q.x;
        var y = q.y;
        var z = q.z;
        var w = q.w;
        var x2 = 2 * x * x;
        var y2 = 2 * y * y;
        var z2 = 2 * z * z;
        var xy2 = x * y * 2;
        var yz2 = y * z * 2;
        var zx2 = z * x * 2;
        var wx2 = w * x * 2;
        var wy2 = w * y * 2;
        var wz2 = w * z * 2;
        return new mat3([
            1 - y2 - z2, xy2 + wz2, zx2 - wy2,
            xy2 - wz2, 1 - x2 - z2, yz2 + wx2,
            zx2 + wy2, yz2 - wx2, 1 - x2 - y2
        ]);
    };
    quat.Conjugate = function (q) {
        return new quat([-q.x, -q.y, -q.z, q.w]);
    };
    /**
     * Flip coordinate axis of quaternion
     * @param q
     * @param xaxis
     * @param yaxis
     * @param zaxis
     */
    quat.Flip = function (q, xaxis, yaxis, zaxis) {
        var r = q.clone();
        var c = 0;
        if (xaxis) {
            r.x = -r.x;
            c++;
        }
        if (yaxis) {
            r.y = -r.y;
            c++;
        }
        if (zaxis) {
            r.z = -r.z;
            c++;
        }
        if (c % 2 == 1) {
            r.x = -r.x;
            r.y = -r.y;
            r.z = -r.z;
        }
        return r;
    };
    /**
     * div LHS quat by default
     * @param p
     * @param q
     * @param rhs false: p = rq, true: p =qr
     */
    quat.Div = function (p, q, rhs) {
        if (rhs === void 0) { rhs = false; }
        if (rhs) {
            return p.mul(q.conjugate());
        }
        else {
            return q.conjugate().mul(p);
        }
    };
    quat.MtxToQuat = function (mtx) {
        var raw = mtx.raw;
        var a1 = raw[1];
        var a2 = raw[2];
        var a3 = raw[3];
        var a5 = raw[5];
        var a7 = raw[7];
        var a8 = raw[8];
        var a0 = raw[0];
        var a4 = raw[4];
        var a6 = raw[6];
        var w2 = (a0 + a4 + 1 + a8) / 4;
        var x2 = (a0 + 1 - 2 * w2) / 2;
        var x = Math.sqrt(x2);
        var y = 0;
        var z = 0;
        var w = 0;
        if (x < 0.000001) {
            x = 0;
            var y2 = 1 - a8;
            if (y2 == 0) {
                y = 0;
                var zw = -a3 / 2.0;
                w = Math.sqrt((a0 + 1) / 2.0);
                z = w == 0 ? 1.0 : zw / w;
            }
            else {
                y = Math.sqrt(y2 / 2.0);
                z = a7 / 2.0 / y;
                w = a6 / 2.0 / y;
            }
        }
        else {
            y = (a1 + a3) / 4.0 / x;
            if (y == 0) {
                z = a2 / 2.0 / x;
                w = a7 / -2.0 / x;
            }
            else {
                z = (a5 + a7) / 4.0 / y;
                w = z == 0 ? (a7 / -2.0 / x) : ((a1 - a3) / 4.0 / z);
            }
        }
        return new quat([x, y, z, w]);
    };
    quat.prototype.equals = function (q) {
        var qraw = (q.w * this.w < 0) ? [-q.x, -q.y, -q.z, -q.w] : q.raw;
        for (var i = 0; i < 4; i++) {
            if (Math.abs(qraw[i] - this.raw[i]) > 0.001)
                return false;
        }
        return true;
    };
    quat.prototype.magnitude2 = function () {
        var x = this.x;
        var y = this.y;
        var z = this.z;
        var w = this.w;
        return x * x + y * y + z * z + w * w;
    };
    quat.prototype.magnitude = function () {
        return Math.sqrt(this.magnitude2());
    };
    quat.prototype.clone = function () {
        return new quat([this.x, this.y, this.z, this.w]);
    };
    quat.Random = function () {
        var q = quat.axisRotation(glmath.vec3(Math.random(), Math.random(), Math.random()), Math.PI * 2 * Math.random());
        q.fmt();
        return q;
    };
    quat.prototype.set = function (q) {
        this.x = q.x;
        this.y = q.y;
        this.z = q.z;
        this.w = q.w;
    };
    return quat;
}());
exports.quat = quat;
var mat4 = /** @class */ (function () {
    function mat4(v) {
        if (v != null) {
            this.raw = v;
        }
        else {
            this.raw = new Array(16);
        }
    }
    mat4.prototype.column = function (index) {
        var raw = this.raw;
        var o = index * 4;
        return new vec4([raw[o], raw[o + 1], raw[o + 2], raw[o + 3]]);
    };
    mat4.prototype.row = function (index) {
        var raw = this.raw;
        var o = index;
        return new vec4([raw[o], raw[o + 4], raw[o + 8], raw[o + 12]]);
    };
    Object.defineProperty(mat4, "Identity", {
        get: function () {
            return new mat4([
                1, 0, 0, 0,
                0, 1, 0, 0,
                0, 0, 1, 0,
                0, 0, 0, 1
            ]);
        },
        enumerable: true,
        configurable: true
    });
    mat4.lookAt = function (eye, target, up) {
        var vz = eye.subToRef(target).normalize;
        var vx = up.cross(vz).normalize;
        var vy = vz.cross(vx).normalize;
        return mat4.inverse(new mat4([
            vx.x, vx.y, vx.z, 0,
            vy.x, vy.y, vy.z, 0,
            vz.x, vz.y, vz.z, 0,
            -vx.dot(eye),
            -vy.dot(eye),
            -vz.dot(eye),
            1
        ]));
    };
    /**
     * Build coordinate change matrix RH->RH LH->LH
     * @param pos pos
     * @param forward dir
     * @param up dir
     */
    mat4.coord = function (pos, forward, up) {
        var f = forward.normalized();
        var u = up.normalized();
        var r = u.cross(f).normalize;
        u = f.cross(r).normalize;
        return new mat4([
            r.x, u.x, f.x, 0,
            r.y, u.y, f.y, 0,
            r.z, u.z, f.z, 0,
            -r.dot(pos), -u.dot(pos), -f.dot(pos), 1
        ]);
    };
    /**
     * Build matrix for coordinate conversion RH->LH LH->RH
     * @param pos pos
     * @param forward dir
     * @param up dir
     */
    mat4.coordCvt = function (pos, forward, up) {
        var f = forward.normalized();
        var u = up.normalized();
        var r = u.crossRev(f).normalize;
        u = f.crossRev(r).normalize;
        return new mat4([
            r.x, u.x, f.x, 0,
            r.y, u.y, f.y, 0,
            r.z, u.z, f.z, 0,
            -r.dot(pos), -u.dot(pos), -f.dot(pos), 1
        ]);
    };
    /**
     * Left Hand Coordinate
     * @param w
     * @param h
     * @param n
     * @param f
     */
    mat4.perspective = function (w, h, n, f) {
        return new mat4([
            2 * n / w, 0, 0, 0,
            0, 2 * n / h, 0, 0,
            0, 0, (n + f) / (n - f), -1,
            0, 0, 2 * n * f / (n - f), 0
        ]);
    };
    /**
     * Left Hand Coordinate
     * @param fov
     * @param aspect
     * @param n
     * @param f
     */
    mat4.perspectiveFoV = function (fov, aspect, n, f) {
        var h = Math.tan(fov / 360.0 * Math.PI) * n * 2;
        var w = h * aspect;
        return this.perspective(w, h, n, f);
    };
    /**
     * Left Hand coordinate
     * @param w
     * @param h
     * @param n
     * @param f
     */
    mat4.orthographic = function (w, h, n, f) {
        var d = f - n;
        return new mat4([
            2.0 / w, 0, 0, 0,
            0, 2.0 / h, 0, 0,
            0, 0, 2.0 / d, 0,
            0, 0, -(n + f) / d, 1
        ]);
    };
    mat4.prototype.inverse = function () {
        return mat4.inverse(this);
    };
    mat4.inverse = function (mtx) {
        var m = mtx.raw;
        var dst = new Array(16);
        var m00 = m[0 * 4 + 0];
        var m01 = m[0 * 4 + 1];
        var m02 = m[0 * 4 + 2];
        var m03 = m[0 * 4 + 3];
        var m10 = m[1 * 4 + 0];
        var m11 = m[1 * 4 + 1];
        var m12 = m[1 * 4 + 2];
        var m13 = m[1 * 4 + 3];
        var m20 = m[2 * 4 + 0];
        var m21 = m[2 * 4 + 1];
        var m22 = m[2 * 4 + 2];
        var m23 = m[2 * 4 + 3];
        var m30 = m[3 * 4 + 0];
        var m31 = m[3 * 4 + 1];
        var m32 = m[3 * 4 + 2];
        var m33 = m[3 * 4 + 3];
        var tmp_0 = m22 * m33;
        var tmp_1 = m32 * m23;
        var tmp_2 = m12 * m33;
        var tmp_3 = m32 * m13;
        var tmp_4 = m12 * m23;
        var tmp_5 = m22 * m13;
        var tmp_6 = m02 * m33;
        var tmp_7 = m32 * m03;
        var tmp_8 = m02 * m23;
        var tmp_9 = m22 * m03;
        var tmp_10 = m02 * m13;
        var tmp_11 = m12 * m03;
        var tmp_12 = m20 * m31;
        var tmp_13 = m30 * m21;
        var tmp_14 = m10 * m31;
        var tmp_15 = m30 * m11;
        var tmp_16 = m10 * m21;
        var tmp_17 = m20 * m11;
        var tmp_18 = m00 * m31;
        var tmp_19 = m30 * m01;
        var tmp_20 = m00 * m21;
        var tmp_21 = m20 * m01;
        var tmp_22 = m00 * m11;
        var tmp_23 = m10 * m01;
        var t0 = (tmp_0 * m11 + tmp_3 * m21 + tmp_4 * m31) -
            (tmp_1 * m11 + tmp_2 * m21 + tmp_5 * m31);
        var t1 = (tmp_1 * m01 + tmp_6 * m21 + tmp_9 * m31) -
            (tmp_0 * m01 + tmp_7 * m21 + tmp_8 * m31);
        var t2 = (tmp_2 * m01 + tmp_7 * m11 + tmp_10 * m31) -
            (tmp_3 * m01 + tmp_6 * m11 + tmp_11 * m31);
        var t3 = (tmp_5 * m01 + tmp_8 * m11 + tmp_11 * m21) -
            (tmp_4 * m01 + tmp_9 * m11 + tmp_10 * m21);
        var d = 1.0 / (m00 * t0 + m10 * t1 + m20 * t2 + m30 * t3);
        dst[0] = d * t0;
        dst[1] = d * t1;
        dst[2] = d * t2;
        dst[3] = d * t3;
        dst[4] = d * ((tmp_1 * m10 + tmp_2 * m20 + tmp_5 * m30) -
            (tmp_0 * m10 + tmp_3 * m20 + tmp_4 * m30));
        dst[5] = d * ((tmp_0 * m00 + tmp_7 * m20 + tmp_8 * m30) -
            (tmp_1 * m00 + tmp_6 * m20 + tmp_9 * m30));
        dst[6] = d * ((tmp_3 * m00 + tmp_6 * m10 + tmp_11 * m30) -
            (tmp_2 * m00 + tmp_7 * m10 + tmp_10 * m30));
        dst[7] = d * ((tmp_4 * m00 + tmp_9 * m10 + tmp_10 * m20) -
            (tmp_5 * m00 + tmp_8 * m10 + tmp_11 * m20));
        dst[8] = d * ((tmp_12 * m13 + tmp_15 * m23 + tmp_16 * m33) -
            (tmp_13 * m13 + tmp_14 * m23 + tmp_17 * m33));
        dst[9] = d * ((tmp_13 * m03 + tmp_18 * m23 + tmp_21 * m33) -
            (tmp_12 * m03 + tmp_19 * m23 + tmp_20 * m33));
        dst[10] = d * ((tmp_14 * m03 + tmp_19 * m13 + tmp_22 * m33) -
            (tmp_15 * m03 + tmp_18 * m13 + tmp_23 * m33));
        dst[11] = d * ((tmp_17 * m03 + tmp_20 * m13 + tmp_23 * m23) -
            (tmp_16 * m03 + tmp_21 * m13 + tmp_22 * m23));
        dst[12] = d * ((tmp_14 * m22 + tmp_17 * m32 + tmp_13 * m12) -
            (tmp_16 * m32 + tmp_12 * m12 + tmp_15 * m22));
        dst[13] = d * ((tmp_20 * m32 + tmp_12 * m02 + tmp_19 * m22) -
            (tmp_18 * m22 + tmp_21 * m32 + tmp_13 * m02));
        dst[14] = d * ((tmp_18 * m12 + tmp_23 * m32 + tmp_15 * m02) -
            (tmp_22 * m32 + tmp_14 * m02 + tmp_19 * m12));
        dst[15] = d * ((tmp_22 * m22 + tmp_16 * m02 + tmp_21 * m12) -
            (tmp_20 * m12 + tmp_23 * m22 + tmp_17 * m02));
        return new mat4(dst);
    };
    mat4.Translate = function (v) {
        return new mat4([
            1, 0, 0, 0,
            0, 1, 0, 0,
            0, 0, 1, 0,
            v.x, v.y, v.z, 1
        ]);
    };
    mat4.mul = function (mtx, v) {
        return new vec4([
            v.dot(mtx.row(0)),
            v.dot(mtx.row(1)),
            v.dot(mtx.row(2)),
            v.dot(mtx.row(3))
        ]);
    };
    mat4.prototype.add = function (m) {
        return mat4.add(this, m);
    };
    mat4.add = function (m1, m2) {
        var ary = [];
        var raw1 = m1.raw;
        var raw2 = m2.raw;
        for (var i = 0; i < 16; i++) {
            ary.push(raw1[i] + raw2[i]);
        }
        return new mat4(ary);
    };
    mat4.prototype.mulvec = function (v) {
        return mat4.mul(this, v);
    };
    mat4.prototype.mulnum = function (n) {
        var nary = [];
        var raw = this.raw;
        for (var i = 0; i < 16; i++) {
            nary.push(raw[i] * n);
        }
        return new mat4(nary);
    };
    mat4.Scale = function (scale) {
        return new mat4([
            scale.x, 0, 0, 0,
            0, scale.y, 0, 0,
            0, 0, scale.z, 0,
            0, 0, 0, 1
        ]);
    };
    mat4.Rotation = function (q) {
        var mtx = quat.QuatToMtx(q).toMat4();
        return mtx;
    };
    mat4.RotationEler = function (rad) {
        return mat3.Rotation(quat.fromEuler(rad.x, rad.y, rad.z)).toMat4();
    };
    mat4.TRS = function (translate, rota, scale) {
        var mtxr = quat.QuatToMtx(rota).raw;
        var x = scale.x;
        var y = scale.y;
        var z = scale.z;
        return new mat4([
            mtxr[0] * x, mtxr[1] * x, mtxr[2] * x, 0,
            mtxr[3] * y, mtxr[4] * y, mtxr[5] * y, 0,
            mtxr[6] * z, mtxr[7] * z, mtxr[8] * z, 0,
            translate.x, translate.y, translate.z, 1
        ]);
    };
    mat4.prototype.setTRS = function (translate, rota, scale) {
        var raw = this.raw;
        var r = quat.QuatToMtx(rota).raw;
        var x = scale.x;
        var y = scale.y;
        var z = scale.z;
        raw[0] = r[0] * x;
        raw[1] = r[1] * x;
        raw[2] = r[2] * x;
        raw[3] = 0;
        raw[4] = r[3] * y;
        raw[5] = r[4] * y;
        raw[6] = r[5] * y;
        raw[7] = 0;
        raw[8] = r[6] * z;
        raw[9] = r[7] * z;
        raw[10] = r[8] * z;
        raw[11] = 0;
        raw[12] = translate.x;
        raw[13] = translate.y;
        raw[14] = translate.z;
        raw[15] = 1;
    };
    mat4.prototype.set = function (mtx) {
        var raw = this.raw;
        var mraw = mtx.raw;
        for (var i = 0; i < 16; i++) {
            raw[i] = mraw[i];
        }
    };
    mat4.prototype.mul = function (rhs) {
        var m0 = this.row(0);
        var m1 = this.row(1);
        var m2 = this.row(2);
        var m3 = this.row(3);
        var n0 = rhs.column(0);
        var n1 = rhs.column(1);
        var n2 = rhs.column(2);
        var n3 = rhs.column(3);
        return new mat4([
            m0.dot(n0), m1.dot(n0), m2.dot(n0), m3.dot(n0),
            m0.dot(n1), m1.dot(n1), m2.dot(n1), m3.dot(n1),
            m0.dot(n2), m1.dot(n2), m2.dot(n2), m3.dot(n2),
            m0.dot(n3), m1.dot(n3), m2.dot(n3), m3.dot(n3)
        ]);
    };
    mat4.prototype.transpose = function () {
        return mat4.Transpose(this);
    };
    mat4.Transpose = function (m) {
        var raw = m.raw;
        return new mat4([
            raw[0], raw[4], raw[8], raw[10],
            raw[1], raw[5], raw[7], raw[11],
            raw[2], raw[6], raw[8], raw[12],
            raw[3], raw[7], raw[9], raw[13],
        ]);
    };
    /**
     * Decompose mat4 into translation rotation and scale, No shear
     * @param mat
     * @param hasNegativeScale optimize when scale components all positive.
     * @returns [T,R,S]
     */
    mat4.Decompose = function (mat, hasNegativeScale) {
        var raw = mat.raw;
        var t = glmath.vec3(raw[12], raw[13], raw[14]);
        var r0 = raw[0];
        var r1 = raw[1];
        var r2 = raw[2];
        var r4 = raw[4];
        var r5 = raw[5];
        var r6 = raw[6];
        var r8 = raw[8];
        var r9 = raw[9];
        var r10 = raw[10];
        var c0 = glmath.vec3(r0, r1, r2);
        var c1 = glmath.vec3(r4, r5, r6);
        var c2 = glmath.vec3(r8, r9, r10);
        var scale = glmath.vec3(c0.length, c1.length, c2.length);
        if (hasNegativeScale == null || hasNegativeScale == true) {
            var determinant = r0 * (r5 * r10 - r6 * r9) - r1 * (r4 * r10 - r8 * r6) + r2 * (r4 * r9 - r5 * r8);
            if (determinant < 0) {
                scale.z = -scale.z; //set z to negative ensures that MtxToQuat not get the NaN value.
            }
        }
        var mtx3 = mat3.fromColumns(c0.div(scale.x), c1.div(scale.y), c2.div(scale.z));
        var rota = quat.MtxToQuat(mtx3);
        if (isNaN(rota.w) || isNaN(rota.x) || isNaN(rota.y) || isNaN(rota.z)) {
            throw new Error("quat is NaN, " + mtx3.raw);
        }
        return [t, rota, scale];
    };
    mat4.DecomposeTRS = function (mat, t, q, s) {
        var smtx = new mat3();
        mat4.DecomposeAffine(mat, t, smtx, s);
        q.set(quat.MtxToQuat(smtx));
    };
    /**
     * Decompose Affine Matrix
     * https://matthew-brett.github.io/transforms3d/reference/transforms3d.affines.html#decompose44?tdsourcetag=s_pctim_aiomsg
     * @param mat
     * @param t translate
     * @param q rotation
     * @param s zoom [sx,sy,sz]
     * @param sk skew [sxy,sxz,syz]
     */
    mat4.DecomposeAffine = function (mat, t, q, s, sk) {
        var raw = mat.raw;
        t.x = raw[12];
        t.y = raw[13];
        t.z = raw[14];
        var mtx3 = mat.toMat3();
        mat3.DecomposeAffine(mtx3, q, s, sk);
    };
    mat4.prototype.clone = function () {
        var ary = this.raw.slice(0);
        return new mat4(ary);
    };
    mat4.RandomTRS = function () {
        return mat4.TRS(vec3.Random(), quat.Random(), vec3.Random());
    };
    mat4.prototype.mat3Determinant = function () {
        var raw = this.raw;
        return raw[0] * (raw[5] * raw[10] - raw[6] * raw[9]) -
            raw[4] * (raw[1] * raw[10] - raw[2] * raw[9]) +
            raw[8] * (raw[1] * raw[6] - raw[2] * raw[5]);
    };
    mat4.prototype.toMat3 = function () {
        var raw = this.raw;
        return new mat3([raw[0], raw[1], raw[2],
            raw[4], raw[5], raw[6],
            raw[8], raw[9], raw[10]]);
    };
    return mat4;
}());
exports.mat4 = mat4;
var mat3 = /** @class */ (function () {
    function mat3(v) {
        if (v != null) {
            this.raw = v;
        }
        else {
            this.raw = new Array(9);
        }
    }
    /**
     * Get column of matrix
     * @param index
     */
    mat3.prototype.column = function (index) {
        var raw = this.raw;
        var o = index * 3;
        return new vec3([raw[o], raw[o + 1], raw[o + 2]]);
    };
    mat3.prototype.setColumn = function (c, index) {
        var raw = this.raw;
        var o = index * 3;
        raw[o] = c.x;
        raw[o + 1] = c.y;
        raw[o + 2] = c.z;
    };
    /**
     * Get row of matrix
     * @param index
     */
    mat3.prototype.row = function (index) {
        var raw = this.raw;
        var o = index;
        return new vec3([raw[o], raw[o + 3], raw[o + 6]]);
    };
    mat3.prototype.setRow = function (r, index) {
        var raw = this.raw;
        raw[index] = r.x;
        raw[index + 3] = r.y;
        raw[index + 6] = r.z;
    };
    Object.defineProperty(mat3, "Identity", {
        get: function () {
            return new mat3([1, 0, 0, 0, 1, 0, 0, 0, 1]);
        },
        enumerable: true,
        configurable: true
    });
    mat3.Transpose = function (m) {
        var raw = m.raw;
        return new mat3([
            raw[0], raw[3], raw[6],
            raw[1], raw[4], raw[7],
            raw[2], raw[5], raw[8]
        ]);
    };
    mat3.prototype.transpose = function () {
        return mat3.Transpose(this);
    };
    mat3.prototype.toMat4 = function () {
        return mat3.ToMat4(this);
    };
    mat3.ToMat4 = function (m) {
        var r = m.raw;
        return new mat4([
            r[0], r[1], r[2], 0,
            r[3], r[4], r[5], 0,
            r[6], r[7], r[8], 0,
            0, 0, 0, 1
        ]);
    };
    mat3.Cross = function (lhs) {
        return new mat3([
            0, lhs.z, -lhs.y,
            -lhs.z, 0, lhs.x,
            lhs.y, -lhs.x, 0
        ]);
    };
    mat3.Diagonal = function (v) {
        return new mat3([
            v.x, 0, 0,
            0, v.y, 0,
            0, 0, v.z
        ]);
    };
    mat3.UpperTri = function (v) {
        return new mat3([
            1, v.x, v.y,
            0, 1, v.z,
            0, 0, 1
        ]);
    };
    /**
     * Create mat3 from composition of scaling and rotation.
     * @param rota
     * @param s
     */
    mat3.fromRS = function (rota, s) {
        return mat3.Mul(quat.QuatToMtx(rota), mat3.Scale(s.x, s.y, s.z));
    };
    mat3.Mul = function (lhs, rhs) {
        var m0 = lhs.row(0);
        var m1 = lhs.row(1);
        var m2 = lhs.row(2);
        var n0 = rhs.column(0);
        var n1 = rhs.column(1);
        var n2 = rhs.column(2);
        return new mat3([
            m0.dot(n0), m1.dot(n0), m2.dot(n0),
            m0.dot(n1), m1.dot(n1), m2.dot(n1),
            m0.dot(n2), m1.dot(n2), m2.dot(n2)
        ]);
    };
    mat3.fromRows = function (r0, r1, r2) {
        return new mat3([
            r0.x, r1.x, r2.x,
            r0.y, r1.y, r2.y,
            r0.z, r1.z, r2.z
        ]);
    };
    mat3.fromColumns = function (c0, c1, c2) {
        return new mat3([
            c0.x, c0.y, c0.z,
            c1.x, c1.y, c1.z,
            c2.x, c2.y, c2.z,
        ]);
    };
    mat3.prototype.mul = function (rhs) {
        return mat3.Mul(this, rhs);
    };
    mat3.CrossRHS = function (rhs) {
        return new mat3([
            0, -rhs.z, rhs.y,
            rhs.z, 0, -rhs.x,
            -rhs.y, rhs.x, 0
        ]);
    };
    mat3.prototype.mulvec = function (v) {
        return mat3.MulVec(this, v);
    };
    mat3.MulVec = function (mat, v) {
        return new vec3([
            mat.row(0).dot(v),
            mat.row(1).dot(v),
            mat.row(2).dot(v),
        ]);
    };
    mat3.Scale = function (sx, sy, sz) {
        return new mat3([
            sx, 0, 0,
            0, sy, 0,
            0, 0, sz
        ]);
    };
    mat3.Shear = function (sk) {
        return new mat3([
            1, sk.x, sk.y,
            0, 1, sk.z,
            0, 0, 1
        ]);
    };
    /**
     * Scale(s) * Shear(sk)
     * @param s
     * @param sk
     */
    mat3.ScaleShear = function (s, sk) {
        var sx = s.x;
        var sy = s.y;
        return new mat3([
            sx, sx * sk.x, sx * sk.y,
            0, sy, sy * sk.z,
            0, 0, s.z
        ]);
    };
    /**
     * Convert quaternion rotation to Matrix
     * @param q
     */
    mat3.Rotation = function (q) {
        return quat.QuatToMtx(q);
    };
    mat3.prototype.clone = function () {
        var ary = this.raw.slice(0);
        return new mat3(ary);
    };
    mat3.prototype.determinant = function () {
        var raw = this.raw;
        return raw[0] * (raw[4] * raw[8] - raw[5] * raw[7]) -
            raw[3] * (raw[1] * raw[8] - raw[2] * raw[7]) +
            raw[6] * (raw[1] * raw[5] - raw[2] * raw[4]);
    };
    /**
     * Decompose Affine matrix
     * @param mat
     * @param q Rotation matrix
     * @param s Diagonal matrix
     * @param sk Shear matrix
     */
    mat3.DecomposeAffine = function (mat, q, s, sk) {
        var raw = mat.raw;
        var r0 = raw[0];
        var r1 = raw[1];
        var r2 = raw[2];
        var r3 = raw[3];
        var r4 = raw[4];
        var r5 = raw[5];
        var r6 = raw[6];
        var r7 = raw[7];
        var r8 = raw[8];
        var m0 = glmath.vec3(r0, r1, r2);
        var m1 = glmath.vec3(r3, r4, r5);
        var m2 = glmath.vec3(r6, r7, r8);
        var M0 = m0.clone();
        var M1 = m1.clone();
        var M2 = m2.clone();
        var sx = M0.length;
        M0.div(sx);
        var sx_sxy = M0.dot(M1);
        M1.sub(M0.mulNumToRef(sx_sxy));
        var sy = M1.length;
        M1.div(sy);
        var sx_sxz = M0.dot(M2);
        var sy_syz = M1.dot(M2);
        M2.sub(M0.mulNumToRef(sx_sxz).add(M1.mulNumToRef(sy_syz)));
        var sz = M2.length;
        M2.div(sz);
        q.setColumn(M0, 0);
        q.setColumn(M1, 1);
        q.setColumn(M2, 2);
        if (q.determinant() < 0) {
            var raw_1 = q.raw;
            for (var i = 0; i < 9; i++) {
                raw_1[i] = -raw_1[i];
            }
            M0.mulNum(-1);
            M1.mulNum(-1);
            M2.mulNum(-1);
        }
        s.x = M0.dot(m0);
        s.y = M1.dot(m1);
        s.z = M2.dot(m2);
        if (sk != null) {
            var invsx = 1.0 / s.x;
            sk.x = M0.dot(m1) * invsx;
            sk.y = M0.dot(m2) * invsx;
            sk.z = M1.dot(m2) / s.y;
        }
    };
    mat3.DecomposeRS = function (mat, q, s) {
        var smtx = new mat3();
        mat3.DecomposeAffine(mat, smtx, s);
        q.set(quat.MtxToQuat(smtx));
        q.fmt();
    };
    return mat3;
}());
exports.mat3 = mat3;
