/// <reference types="webgl2" />
import { Scene } from "../Scene";
import { ShaderDataUniformObj, ShaderDataUniformShadowMap, ShaderDataUniformLight, ShaderDataBasis } from "../shaderfx/ShaderFXLibs";
import { GraphicsRenderCreateInfo, GraphicsRender } from "../GraphicsRender";
import { RenderNodeList } from "../RenderNodeList";
import { BufferDebugInfo } from "../render/BufferDebugInfo";
import { PassDebug } from "../render/PassDebug";
import { Texture } from "../Texture";
import { ShadowMapData } from "../render/Shadow";
import { PipelineStateCache } from "../PipelineStateCache";
import { IRenderPipeline } from "./IRenderPipeline";
import { Mesh } from "../Mesh";
import { MeshRender } from "../MeshRender";
import { Material } from "../Material";
import { Camera } from "../Camera";
import { GLContext } from "../gl/GLContext";
import { GLFrameBuffer } from "../gl/GLFrameBuffer";
import { GLProgram } from "../gl/GLProgram";
import { mat4 } from "../math/GLMath";
export declare class PipelineBase implements IRenderPipeline {
    static readonly UNIFORMINDEX_OBJ: number;
    static readonly UNIFORMINDEX_BASIS: number;
    static readonly UNIFORMINDEX_SHADOWMAP: number;
    static readonly UNIFORMINDEX_LIGHT: number;
    static readonly UNIFORMINDEX_SHADER: number;
    readonly GLCtx: GLContext;
    readonly GL: WebGL2RenderingContext;
    protected glctx: GLContext;
    protected gl: WebGL2RenderingContext;
    private m_pipestateCache;
    protected m_inited: boolean;
    graphicRender: GraphicsRender;
    protected m_bufferDebugInfo: BufferDebugInfo[];
    readonly bufferDebugInfo: BufferDebugInfo[];
    protected m_mainDepthTexture: Texture;
    protected m_mainDepthFB: WebGLFramebuffer;
    readonly mainDepthTexture: Texture;
    readonly mainDepthFrameBuffer: WebGLFramebuffer;
    private m_uniformBufferBasis;
    private m_uniformBufferObj;
    private m_uniformBufferShadowMap;
    private m_uniformBufferLight;
    private m_shaderDataBasis;
    private m_shaderDataObj;
    private m_shaderDataShadowMap;
    private m_shaderDataLight;
    readonly shaderDataBasis: ShaderDataBasis;
    readonly shaderDataObj: ShaderDataUniformObj;
    readonly shaderDataLight: ShaderDataUniformLight;
    readonly shaderDataShadowMap: ShaderDataUniformShadowMap;
    protected m_shadowMapData: ShadowMapData;
    readonly shadowMapData: ShadowMapData;
    protected m_shadowEnabled: boolean;
    private m_nodelist;
    private m_nodelistIndex;
    private m_nodelistCur;
    readonly nodeList: RenderNodeList;
    protected m_mainFrameBuffer: GLFrameBuffer;
    protected m_mainFrameBufferInfo: GraphicsRenderCreateInfo;
    protected m_mainFrameBufferBinded: boolean;
    readonly mainFrameBufferWidth: number;
    readonly mainFrameBufferHeight: number;
    readonly mainFrameBufferAspect: number;
    readonly mainFrameBuffer: GLFrameBuffer;
    readonly stateCache: PipelineStateCache;
    protected m_mainFrameBufferAspect: number;
    protected m_mainFrameBufferWidth: number;
    protected m_mainFrameBufferHeight: number;
    /** for viewport update in @function <bindTargetFrameBuffer> */
    private m_mainFrameBufferResized;
    /** for shderDataBasis screnparam */
    private m_shaderDataScreenResized;
    protected m_passDebug: PassDebug;
    renderPassDebug: boolean;
    private m_fullscreenRender;
    private m_fullscreenMat;
    constructor();
    onSetupRender(glctx: GLContext, bufferinfo: GraphicsRenderCreateInfo): void;
    init(): void;
    private createUniformBuffers;
    renderBufferDebug(): void;
    resizeFrameBuffer(width: number, height: number): void;
    exec(scene: Scene): void;
    /**
     * draw main framebuffer to canvas buffer
     */
    onRenderToCanvas(): void;
    updateShaderDataBasis(camera: Camera, submit?: boolean): void;
    submitShaderDataBasis(): void;
    updateUniformCamera(camera: Camera, submit?: boolean): void;
    updateUniformBufferObject(data: ShaderDataUniformObj): void;
    updateUniformBufferShadowMap(data: ShaderDataUniformShadowMap): void;
    updateUniformBufferLight(data: ShaderDataUniformLight): void;
    activeDefaultTexture(): void;
    addBufferDebugInfo(info: BufferDebugInfo): void;
    removeBufferDebugInfo(info: BufferDebugInfo): void;
    bindTargetFrameBuffer(forece?: boolean): void;
    UnBindTargetFrameBuffer(): void;
    generateDrawList(scene: Scene): RenderNodeList;
    private traversalRenderNode;
    /**
     * Bind internal ShaderFX uniform block index to current program;
     * UNIFORM_BASIS, UNIFORM_OBJ, UNIFORM_LIGHT, UNIFORM_SM
     * @param program
     */
    uniformBindDefault(program: GLProgram): void;
    /** draw fullscreen tex */
    drawFullScreenTex(tex: Texture | WebGLTexture): void;
    /**
     * procedural drawing a mesh with material
     * @param mesh
     * @param mat
     * @param vao
     * @param objmtx
     * @param defUniformBlock
     */
    drawMeshWithMat(mesh: Mesh, mat: Material, vao: WebGLVertexArrayObject, objmtx?: mat4, defUniformBlock?: boolean): void;
    /**
     * draw a seperated MeshRender
     * @param meshrender
     * @param objmtx
     * @param defUniformBlock
     */
    drawMeshRender(meshrender: MeshRender, objmtx?: mat4, defUniformBlock?: boolean): void;
    release(): void;
    reload(): void;
}
