"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ShaderFXLibs_1 = require("../shaderfx/ShaderFXLibs");
var RenderNodeList_1 = require("../RenderNodeList");
var PassDebug_1 = require("../render/PassDebug");
var Shadow_1 = require("../render/Shadow");
var PipelineStateCache_1 = require("../PipelineStateCache");
var RenderPass_1 = require("../render/RenderPass");
var ShaderFX_1 = require("../shaderfx/ShaderFX");
var Mesh_1 = require("../Mesh");
var MeshRender_1 = require("../MeshRender");
var Material_1 = require("../Material");
var PipelineBase = /** @class */ (function () {
    function PipelineBase() {
        this.m_inited = false;
        //For debug textures and framebuffers
        this.m_bufferDebugInfo = [];
        this.m_shadowMapData = new Shadow_1.ShadowMapData();
        this.m_shadowEnabled = false;
        this.m_nodelist = [new RenderNodeList_1.RenderNodeList(), new RenderNodeList_1.RenderNodeList()];
        this.m_nodelistIndex = 0;
        this.m_mainFrameBufferBinded = false;
        this.m_mainFrameBufferAspect = 1.0;
        this.m_mainFrameBufferWidth = 0;
        this.m_mainFrameBufferHeight = 0;
        /** for viewport update in @function <bindTargetFrameBuffer> */
        this.m_mainFrameBufferResized = true;
        /** for shderDataBasis screnparam */
        this.m_shaderDataScreenResized = false;
        this.renderPassDebug = false;
    }
    Object.defineProperty(PipelineBase.prototype, "GLCtx", {
        get: function () { return this.glctx; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PipelineBase.prototype, "GL", {
        get: function () { return this.gl; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PipelineBase.prototype, "bufferDebugInfo", {
        get: function () {
            return this.m_bufferDebugInfo;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PipelineBase.prototype, "mainDepthTexture", {
        get: function () {
            return this.m_mainDepthTexture;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PipelineBase.prototype, "mainDepthFrameBuffer", {
        get: function () {
            return this.m_mainDepthFB;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PipelineBase.prototype, "shaderDataBasis", {
        get: function () {
            return this.m_shaderDataBasis;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PipelineBase.prototype, "shaderDataObj", {
        get: function () {
            return this.m_shaderDataObj;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PipelineBase.prototype, "shaderDataLight", {
        get: function () {
            return this.m_shaderDataLight;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PipelineBase.prototype, "shaderDataShadowMap", {
        get: function () {
            return this.m_shaderDataShadowMap;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PipelineBase.prototype, "shadowMapData", {
        get: function () { return this.m_shadowMapData; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PipelineBase.prototype, "nodeList", {
        get: function () {
            return this.m_nodelistCur;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PipelineBase.prototype, "mainFrameBufferWidth", {
        get: function () { return this.m_mainFrameBufferWidth; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PipelineBase.prototype, "mainFrameBufferHeight", {
        get: function () { return this.m_mainFrameBufferHeight; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PipelineBase.prototype, "mainFrameBufferAspect", {
        get: function () { return this.m_mainFrameBufferAspect; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PipelineBase.prototype, "mainFrameBuffer", {
        get: function () { return this.m_mainFrameBuffer; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(PipelineBase.prototype, "stateCache", {
        get: function () { return this.m_pipestateCache; },
        enumerable: true,
        configurable: true
    });
    PipelineBase.prototype.onSetupRender = function (glctx, bufferinfo) {
        this.glctx = glctx;
        this.gl = glctx.gl;
        this.m_mainFrameBufferInfo = bufferinfo;
        this.init();
    };
    PipelineBase.prototype.init = function () {
        if (this.m_inited)
            return;
        var glctx = this.glctx;
        var bufferinfo = this.m_mainFrameBufferInfo;
        this.m_pipestateCache = new PipelineStateCache_1.PipelineStateCache(glctx);
        var fb = this.glctx.createFrameBuffer(true, bufferinfo.colorFormat, bufferinfo.depthFormat);
        this.m_mainFrameBuffer = fb;
        this.m_mainFrameBufferWidth = fb.width;
        this.m_mainFrameBufferHeight = fb.height;
        this.m_mainFrameBufferAspect = fb.width / fb.height;
        this.createUniformBuffers();
        this.m_passDebug = new PassDebug_1.PassDebug(this);
        if (this.m_fullscreenMat == null) {
            this.m_fullscreenMat = new Material_1.Material(this.graphicRender.shaderLib.shaderBlit);
        }
        if (this.m_fullscreenRender == null) {
            this.m_fullscreenRender = new MeshRender_1.MeshRender(Mesh_1.Mesh.Quad, this.m_fullscreenMat);
        }
        this.m_inited = true;
    };
    PipelineBase.prototype.createUniformBuffers = function () {
        var CLASS = PipelineBase;
        var gl = this.gl;
        //create internal uniform buffer
        if (this.m_uniformBufferObj == null) {
            var data = new ShaderFXLibs_1.ShaderDataUniformObj();
            this.m_shaderDataObj = data;
            var buffer = gl.createBuffer();
            gl.bindBuffer(gl.UNIFORM_BUFFER, buffer);
            gl.bufferData(gl.UNIFORM_BUFFER, data.fxbuffer.raw, gl.DYNAMIC_DRAW);
            gl.bindBufferBase(gl.UNIFORM_BUFFER, CLASS.UNIFORMINDEX_OBJ, buffer);
            this.m_uniformBufferObj = buffer;
        }
        if (this.m_uniformBufferBasis == null) {
            var data = new ShaderFXLibs_1.ShaderDataBasis();
            this.m_shaderDataBasis = data;
            var buffer = gl.createBuffer();
            gl.bindBuffer(gl.UNIFORM_BUFFER, buffer);
            gl.bufferData(gl.UNIFORM_BUFFER, data.fxbuffer.raw, gl.DYNAMIC_DRAW);
            gl.bindBufferBase(gl.UNIFORM_BUFFER, CLASS.UNIFORMINDEX_BASIS, buffer);
            this.m_uniformBufferBasis = buffer;
        }
        if (this.m_uniformBufferShadowMap == null) {
            var data = new ShaderFXLibs_1.ShaderDataUniformShadowMap();
            this.m_shaderDataShadowMap = data;
            var buffer = gl.createBuffer();
            gl.bindBuffer(gl.UNIFORM_BUFFER, buffer);
            gl.bufferData(gl.UNIFORM_BUFFER, data.fxbuffer.raw, gl.DYNAMIC_DRAW);
            gl.bindBufferBase(gl.UNIFORM_BUFFER, CLASS.UNIFORMINDEX_SHADOWMAP, buffer);
            this.m_uniformBufferShadowMap = buffer;
        }
        if (this.m_uniformBufferLight == null) {
            var data = new ShaderFXLibs_1.ShaderDataUniformLight();
            this.m_shaderDataLight = data;
            var buffer = gl.createBuffer();
            gl.bindBuffer(gl.UNIFORM_BUFFER, buffer);
            gl.bufferData(gl.UNIFORM_BUFFER, data.fxbuffer.raw, gl.DYNAMIC_DRAW);
            gl.bindBufferBase(gl.UNIFORM_BUFFER, CLASS.UNIFORMINDEX_LIGHT, buffer);
            this.m_uniformBufferLight = buffer;
        }
        console.log('create uniform buffers');
    };
    PipelineBase.prototype.renderBufferDebug = function () {
        if (!this.renderPassDebug)
            return;
        var passdebug = this.m_passDebug;
        if (passdebug != null && this.m_bufferDebugInfo.length != 0)
            passdebug.render();
    };
    PipelineBase.prototype.resizeFrameBuffer = function (width, height) {
        var bufferInfo = this.m_mainFrameBufferInfo;
        this.m_mainFrameBuffer = this.glctx.createFrameBuffer(false, bufferInfo.colorFormat, bufferInfo.depthFormat, width, height, this.m_mainFrameBuffer);
        this.m_mainFrameBufferWidth = width;
        this.m_mainFrameBufferHeight = height;
        this.m_mainFrameBufferAspect = width / height;
        this.m_mainFrameBufferResized = true;
        this.m_shaderDataScreenResized = true;
    };
    PipelineBase.prototype.exec = function (scene) {
    };
    /**
     * draw main framebuffer to canvas buffer
     */
    PipelineBase.prototype.onRenderToCanvas = function () {
        var gl = this.gl;
        gl.viewport(0, 0, this.m_mainFrameBufferWidth, this.m_mainFrameBufferHeight);
        this.drawFullScreenTex(this.m_mainFrameBuffer.colorTex0);
    };
    PipelineBase.prototype.updateShaderDataBasis = function (camera, submit) {
        if (submit === void 0) { submit = true; }
        var grender = this.graphicRender;
        var databasis = this.m_shaderDataBasis;
        var databasic = databasis.render;
        databasic.setTime(grender.time, this.graphicRender.deltaTime);
        if (this.m_shaderDataScreenResized) {
            databasic.setScreenParam(this.mainFrameBufferWidth, this.mainFrameBufferHeight);
            this.m_shaderDataScreenResized = false;
        }
        var datacamera = databasis.camrea;
        if (camera.isDataTrsDirty) {
            datacamera.setCameraMtxView(camera.WorldMatrix);
            datacamera.setCameraPos(camera.transform.position);
            camera.isDataTrsDirty = false;
        }
        if (camera.isDataProjDirty) {
            datacamera.setCameraMtxProj(camera.ProjMatrix);
            datacamera.setProjParam(camera.near, camera.far);
            camera.isDataProjDirty = false;
        }
        if (submit) {
            this.submitShaderDataBasis();
        }
    };
    PipelineBase.prototype.submitShaderDataBasis = function () {
        var dataBasis = this.m_shaderDataBasis;
        var gl = this.gl;
        dataBasis.submitBuffer(gl, this.m_uniformBufferBasis);
    };
    PipelineBase.prototype.updateUniformCamera = function (camera, submit) {
        if (submit === void 0) { submit = false; }
        var data = this.m_shaderDataBasis.camrea;
        var ctrs = camera.transform;
        data.setCameraPos(ctrs.position);
        data.setCameraMtxProj(camera.ProjMatrix);
        data.setCameraMtxView(camera.WorldMatrix);
        if (submit) {
            this.submitShaderDataBasis();
        }
    };
    PipelineBase.prototype.updateUniformBufferObject = function (data) {
        var gl = this.gl;
        data.submitBuffer(gl, this.m_uniformBufferObj);
    };
    PipelineBase.prototype.updateUniformBufferShadowMap = function (data) {
        var gl = this.gl;
        data.submitBuffer(gl, this.m_uniformBufferShadowMap);
    };
    PipelineBase.prototype.updateUniformBufferLight = function (data) {
        var gl = this.gl;
        data.submitBuffer(gl, this.m_uniformBufferLight);
    };
    PipelineBase.prototype.activeDefaultTexture = function () {
        var gl = this.gl;
        gl.activeTexture(gl.TEXTURE3);
        gl.bindTexture(gl.TEXTURE_2D, this.graphicRender.defaultTexture.rawtexture);
    };
    PipelineBase.prototype.addBufferDebugInfo = function (info) {
        var curinfo = this.m_bufferDebugInfo;
        if (curinfo.indexOf(info) >= 0)
            return;
        curinfo.push(info);
    };
    PipelineBase.prototype.removeBufferDebugInfo = function (info) {
        var curinfo = this.m_bufferDebugInfo;
        var index = curinfo.indexOf(info);
        if (index < 0)
            return;
        curinfo = curinfo.splice(index, 1);
    };
    PipelineBase.prototype.bindTargetFrameBuffer = function (forece) {
        if (forece === void 0) { forece = false; }
        var mainfb = this.m_mainFrameBuffer;
        if (!this.m_mainFrameBufferBinded || forece) {
            mainfb.bind(this.gl);
            this.m_mainFrameBufferBinded = true;
            forece = true;
        }
        if (this.m_mainFrameBufferResized || forece) {
            this.gl.viewport(0, 0, mainfb.width, mainfb.height);
            this.m_mainFrameBufferResized = false;
        }
    };
    PipelineBase.prototype.UnBindTargetFrameBuffer = function () {
        if (!this.m_mainFrameBufferBinded)
            return;
        var gl = this.gl;
        gl.bindFramebuffer(gl.FRAMEBUFFER, null);
        this.m_mainFrameBufferBinded = false;
    };
    PipelineBase.prototype.generateDrawList = function (scene) {
        var nodelistIndex = this.m_nodelistIndex;
        var nodelist = this.m_nodelist[nodelistIndex];
        nodelist.reset();
        this.traversalRenderNode(nodelist, scene.transform);
        nodelist.sort();
        this.m_nodelistIndex = nodelistIndex == 0 ? 1 : 0;
        this.m_nodelistCur = nodelist;
        return nodelist;
    };
    PipelineBase.prototype.traversalRenderNode = function (drawlist, obj) {
        var children = obj.children;
        if (children == null)
            return;
        for (var i = 0, len = children.length; i < len; i++) {
            var c = children[i];
            var cobj = c.gameobject;
            if (!cobj.active)
                continue;
            var crender = cobj.render;
            if (crender != null) {
                drawlist.pushRenderNode(crender);
            }
            this.traversalRenderNode(drawlist, c);
        }
    };
    /**
     * Bind internal ShaderFX uniform block index to current program;
     * UNIFORM_BASIS, UNIFORM_OBJ, UNIFORM_LIGHT, UNIFORM_SM
     * @param program
     */
    PipelineBase.prototype.uniformBindDefault = function (program) {
        var CLASS = PipelineBase;
        var NAME_BASIS = ShaderFX_1.ShaderFX.UNIFORM_BASIS;
        var NAME_OBJ = ShaderFXLibs_1.ShaderDataUniformObj.UNIFORM_OBJ;
        var NAME_LIGHT = ShaderFXLibs_1.ShaderDataUniformLight.UNIFORM_LIGHT;
        var NAME_SM = ShaderFX_1.ShaderFX.UNIFORM_SHADOWMAP;
        var gl = this.gl;
        var ublock = program.UniformBlock;
        var glp = program.Program;
        //cam uniform buffer
        var indexBasis = ublock[NAME_BASIS];
        if (indexBasis != null)
            gl.uniformBlockBinding(glp, indexBasis, CLASS.UNIFORMINDEX_BASIS);
        //obj uniform buffer
        var indexObj = ublock[NAME_OBJ];
        if (indexObj != null)
            gl.uniformBlockBinding(glp, indexObj, CLASS.UNIFORMINDEX_OBJ);
        //light uniform buffer
        var indexLight = ublock[NAME_LIGHT];
        if (indexLight != null)
            gl.uniformBlockBinding(glp, indexLight, CLASS.UNIFORMINDEX_LIGHT);
        var indexSM = ublock[NAME_SM];
        if (indexSM != null) {
            gl.uniformBlockBinding(glp, indexSM, CLASS.UNIFORMINDEX_SHADOWMAP);
            var loc = program.Uniforms[ShaderFX_1.ShaderFX.UNIFORM_SHADOWMAP_SAMPLER];
            if (loc != null) {
                gl.uniform1i(loc, ShaderFX_1.ShaderFX.GL_SHADOWMAP_TEX0_ID);
            }
        }
    };
    /** draw fullscreen tex */
    PipelineBase.prototype.drawFullScreenTex = function (tex) {
        var mat = this.m_fullscreenMat;
        mat.setTexture(ShaderFX_1.ShaderFX.UNIFORM_MAIN_TEXTURE, tex);
        this.drawMeshRender(this.m_fullscreenRender);
    };
    /**
     * procedural drawing a mesh with material
     * @param mesh
     * @param mat
     * @param vao
     * @param objmtx
     * @param defUniformBlock
     */
    PipelineBase.prototype.drawMeshWithMat = function (mesh, mat, vao, objmtx, defUniformBlock) {
        if (defUniformBlock === void 0) { defUniformBlock = true; }
        var gl = this.gl;
        var program = mat.program;
        gl.useProgram(program.Program);
        if (defUniformBlock) {
            this.uniformBindDefault(program);
        }
        mat.apply(gl);
        var dataobj = this.m_shaderDataObj;
        if (objmtx != null) {
            dataobj.setMtxModel(objmtx);
            this.updateUniformBufferObject(dataobj);
        }
        if (vao == null)
            throw new Error('vertex array obj is null!');
        gl.bindVertexArray(vao);
        var indicedesc = mesh.indiceDesc;
        gl.drawElements(gl.TRIANGLES, indicedesc.indiceCount, indicedesc.type, indicedesc.offset);
        gl.bindVertexArray(null);
        mat.clean(gl);
    };
    /**
     * draw a seperated MeshRender
     * @param meshrender
     * @param objmtx
     * @param defUniformBlock
     */
    PipelineBase.prototype.drawMeshRender = function (meshrender, objmtx, defUniformBlock) {
        if (defUniformBlock === void 0) { defUniformBlock = true; }
        var gl = this.gl;
        var mat = meshrender.material;
        var mesh = meshrender.mesh;
        meshrender.refreshData(this.glctx);
        var program = mat.program;
        gl.useProgram(program.Program);
        if (defUniformBlock) {
            this.uniformBindDefault(program);
        }
        mat.apply(gl);
        var dataobj = this.m_shaderDataObj;
        if (objmtx != null) {
            dataobj.setMtxModel(objmtx);
            this.updateUniformBufferObject(dataobj);
        }
        meshrender.bindVertexArray(gl);
        var indicedesc = mesh.indiceDesc;
        gl.drawElements(gl.TRIANGLES, indicedesc.indiceCount, indicedesc.type, indicedesc.offset);
        meshrender.unbindVertexArray(gl);
        mat.clean(gl);
    };
    PipelineBase.prototype.release = function () {
        if (!this.m_inited)
            return;
        var glctx = this.glctx;
        var gl = glctx.gl;
        this.m_bufferDebugInfo = [];
        this.m_mainFrameBuffer.release(glctx);
        this.m_mainFrameBufferWidth = 0;
        this.m_mainFrameBufferHeight = 0;
        this.m_mainFrameBufferAspect = 1.0;
        this.m_mainFrameBuffer = null;
        this.m_pipestateCache.release();
        this.m_pipestateCache = null;
        gl.deleteBuffer(this.m_uniformBufferBasis);
        gl.deleteBuffer(this.m_uniformBufferLight);
        gl.deleteBuffer(this.m_uniformBufferObj);
        gl.deleteBuffer(this.m_uniformBufferShadowMap);
        this.m_uniformBufferBasis = null;
        this.m_uniformBufferLight = null;
        this.m_uniformBufferObj = null;
        this.m_uniformBufferShadowMap = null;
        this.m_shadowEnabled = false;
        this.m_shadowMapData = new Shadow_1.ShadowMapData();
        var passDebug = this.m_passDebug;
        if (passDebug != null) {
            this.m_passDebug = RenderPass_1.RenderPass.Release(passDebug);
        }
        this.m_inited = false;
    };
    PipelineBase.prototype.reload = function () {
        this.release();
        this.init();
    };
    PipelineBase.UNIFORMINDEX_OBJ = 0;
    PipelineBase.UNIFORMINDEX_BASIS = 1;
    PipelineBase.UNIFORMINDEX_SHADOWMAP = 2;
    PipelineBase.UNIFORMINDEX_LIGHT = 3;
    PipelineBase.UNIFORMINDEX_SHADER = 4;
    return PipelineBase;
}());
exports.PipelineBase = PipelineBase;
