"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var PassOpaque_1 = require("../render/PassOpaque");
var PassTransparent_1 = require("../render/PassTransparent");
var PassSkybox_1 = require("../render/PassSkybox");
var PassGizmos_1 = require("../render/PassGizmos");
var PassDepth_1 = require("../render/PassDepth");
var PassShadowMap_1 = require("../render/PassShadowMap");
var PipelineBase_1 = require("./PipelineBase");
var Texture_1 = require("../Texture");
var Shader_1 = require("../shaderfx/Shader");
var GL_1 = require("../gl/GL");
var RenderPass_1 = require("../render/RenderPass");
var PipelineForwardZPrePass = /** @class */ (function (_super) {
    __extends(PipelineForwardZPrePass, _super);
    function PipelineForwardZPrePass() {
        return _super.call(this) || this;
    }
    PipelineForwardZPrePass.prototype.init = function () {
        if (this.m_inited)
            return;
        _super.prototype.init.call(this);
        var gl = this.glctx.gl;
        gl.depthMask(true);
        gl.depthFunc(gl.LEQUAL);
        gl.enable(gl.DEPTH_TEST);
        var fb = this.m_mainFrameBuffer;
        this.createMainDepthFB(fb.width, fb.height);
        this.m_passGizmos = new PassGizmos_1.PassGizmos(this);
        this.m_passDepth = new PassDepth_1.PassDepth(this);
        this.m_passOpaque = new PassOpaque_1.PassOpaque(this);
        this.m_passTransparent = new PassTransparent_1.PassTransparent(this);
        this.m_passSkybox = new PassSkybox_1.PassSkybox(this);
        this.m_passShadowMap = new PassShadowMap_1.PassShadowMap(this);
        this.m_inited = true;
    };
    PipelineForwardZPrePass.prototype.createMainDepthFB = function (width, height) {
        var bufferinfo = this.m_mainFrameBufferInfo;
        var depthtexdesc = new Texture_1.TextureCreationDesc(null, bufferinfo.depthFormat, false, GL_1.GL.NEAREST, GL_1.GL.NEAREST);
        var tex = Texture_1.Texture.createTexture2D(width, height, depthtexdesc, this.glctx);
        this.m_mainDepthTexture = tex;
        var gl = this.gl;
        if (this.m_mainDepthFB == null) {
            var fb = gl.createFramebuffer();
            gl.bindFramebuffer(gl.DRAW_FRAMEBUFFER, fb);
            gl.framebufferTexture2D(gl.DRAW_FRAMEBUFFER, gl.DEPTH_ATTACHMENT, gl.TEXTURE_2D, tex.rawtexture, 0);
            gl.bindFramebuffer(gl.DRAW_FRAMEBUFFER, null);
            this.m_mainDepthFB = fb;
        }
    };
    PipelineForwardZPrePass.prototype.resizeFrameBuffer = function (width, height) {
        _super.prototype.resizeFrameBuffer.call(this, width, height);
        var glctx = this.glctx;
        var gl = this.gl;
        //resize depth framebuffer
        if (this.m_mainDepthFB != null) {
            gl.deleteFramebuffer(this.m_mainDepthFB);
        }
        var depthtex = this.m_mainDepthTexture;
        depthtex.resize(width, height, glctx);
        var fb = gl.createFramebuffer();
        gl.bindFramebuffer(gl.DRAW_FRAMEBUFFER, fb);
        gl.framebufferTexture2D(gl.DRAW_FRAMEBUFFER, gl.DEPTH_ATTACHMENT, gl.TEXTURE_2D, depthtex.rawtexture, 0);
        gl.bindFramebuffer(gl.DRAW_FRAMEBUFFER, null);
        this.m_mainDepthFB = fb;
    };
    PipelineForwardZPrePass.prototype.exec = function (scene) {
        var cam = scene.mainCamera;
        if (cam == null)
            return;
        cam.aspect = this.mainFrameBufferAspect;
        this.updateShaderDataBasis(cam);
        this.generateDrawList(scene);
        this.bindTargetFrameBuffer();
        var gl = this.gl;
        gl.clearColor(1, 0, 0, 1);
        gl.clearDepth(10.0);
        gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);
        //do rendering
        var passDepth = this.m_passDepth;
        passDepth.render(scene);
        //sm
        var passSM = this.m_passShadowMap;
        passSM.render(scene);
        var passOpaque = this.m_passOpaque;
        passOpaque.render(scene);
        var passSkybox = this.m_passSkybox;
        passSkybox.render(scene);
        var passTransparent = this.m_passTransparent;
        passTransparent.render(scene);
        var passGizmos = this.m_passGizmos;
        passGizmos.render();
        this.renderBufferDebug();
        this.UnBindTargetFrameBuffer();
        var state = this.stateCache;
        state.setBlend(false);
        state.setZTest(Shader_1.Comparison.ALWAYS);
        state.setZWrite(true);
    };
    PipelineForwardZPrePass.prototype.release = function () {
        if (this.m_inited)
            return;
        var gl = this.gl;
        gl.deleteFramebuffer(this.m_mainDepthFB);
        this.m_mainDepthTexture.release(this.glctx);
        this.m_mainDepthTexture = null;
        this.m_passDebug = RenderPass_1.RenderPass.Release(this.m_passDepth);
        this.m_passGizmos = RenderPass_1.RenderPass.Release(this.m_passGizmos);
        this.m_passOpaque = RenderPass_1.RenderPass.Release(this.m_passOpaque);
        this.m_passShadowMap = RenderPass_1.RenderPass.Release(this.m_passShadowMap);
        this.m_passSkybox = RenderPass_1.RenderPass.Release(this.m_passSkybox);
        this.m_passTransparent = RenderPass_1.RenderPass.Release(this.m_passTransparent);
    };
    PipelineForwardZPrePass.prototype.reload = function () {
        this.release();
        this.init();
    };
    return PipelineForwardZPrePass;
}(PipelineBase_1.PipelineBase));
exports.PipelineForwardZPrePass = PipelineForwardZPrePass;
