"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var PipelineBase_1 = require("../pipeline/PipelineBase");
var Shader_1 = require("../shaderfx/Shader");
var MeshRender_1 = require("../MeshRender");
var BufferDebugInfo_1 = require("./BufferDebugInfo");
var RenderPass_1 = require("./RenderPass");
var ShaderFX_1 = require("../shaderfx/ShaderFX");
var GLMath_1 = require("../math/GLMath");
/**
 * Pre-rendering Depth Pass
 */
var PassDepth = /** @class */ (function (_super) {
    __extends(PassDepth, _super);
    function PassDepth(pipeline) {
        var _this = _super.call(this, pipeline) || this;
        var deftags = new Shader_1.ShaderTags();
        deftags.blendOp = null;
        deftags.blend = false;
        deftags.zwrite = true;
        deftags.ztest = Shader_1.Comparison.LEQUAL;
        deftags.culling = Shader_1.CullingMode.Back;
        deftags.fillDefaultVal();
        _this.m_tags = deftags;
        var shader = pipeline.graphicRender.shaderLib.shaderDepth;
        _this.m_program = shader.defaultProgram;
        //debug depth texture
        var debuginfo = new BufferDebugInfo_1.BufferDebugInfo(pipeline.mainDepthTexture, GLMath_1.glmath.vec4(0, 0, 200, 200));
        _this.m_bufferDebugInfo = debuginfo;
        pipeline.addBufferDebugInfo(debuginfo);
        return _this;
    }
    PassDepth.prototype.render = function (scene) {
        var CLASS = PipelineBase_1.PipelineBase;
        var pipe = this.pipeline;
        var gl = pipe.GL;
        var glctx = pipe.GLCtx;
        var deftags = this.m_tags;
        var NAME_BASIS = ShaderFX_1.ShaderFX.UNIFORM_BASIS;
        var NAME_OBJ = ShaderFX_1.ShaderFX.UNIFORM_OBJ;
        var cam = scene.mainCamera;
        var queue = pipe.nodeList.nodeOpaque;
        if (queue.length == 0)
            return;
        //diable color buffer
        gl.colorMask(false, false, false, false);
        //state
        var state = pipe.stateCache;
        state.reset(deftags);
        pipe.activeDefaultTexture();
        //do draw
        var len = queue.length;
        var program = this.m_program;
        var glp = program.Program;
        gl.useProgram(this.m_program.Program);
        var ublock = program.UniformBlock;
        var indexCam = ublock[NAME_BASIS];
        gl.uniformBlockBinding(glp, indexCam, CLASS.UNIFORMINDEX_BASIS);
        var indexObj = ublock[NAME_OBJ];
        gl.uniformBlockBinding(glp, indexObj, CLASS.UNIFORMINDEX_OBJ);
        var dataobj = pipe.shaderDataObj;
        for (var i = 0; i < len; i++) {
            var node = queue[i];
            if (node instanceof MeshRender_1.MeshRender) {
                var mat = node.material;
                var mesh = node.mesh;
                node.refreshData(glctx);
                dataobj.setMtxModel(node.object.transform.objMatrix);
                pipe.updateUniformBufferObject(dataobj);
                node.bindVertexArray(gl);
                var indicedesc = mesh.indiceDesc;
                gl.drawElements(indicedesc.topology, indicedesc.indiceCount, indicedesc.type, indicedesc.offset);
                node.unbindVertexArray(gl);
                mat.clean(gl);
            }
        }
        gl.colorMask(true, true, true, true);
        //copy depth buffer to seperated depth texture
        var mainfb = pipe.mainFrameBuffer;
        gl.bindFramebuffer(gl.READ_FRAMEBUFFER, mainfb.frambuffer);
        gl.bindFramebuffer(gl.DRAW_FRAMEBUFFER, pipe.mainDepthFrameBuffer);
        var w = mainfb.width;
        var h = mainfb.height;
        gl.blitFramebuffer(0, 0, w, h, 0, 0, w, h, gl.DEPTH_BUFFER_BIT, gl.NEAREST);
        gl.bindFramebuffer(gl.READ_FRAMEBUFFER, null);
        gl.bindFramebuffer(gl.DRAW_FRAMEBUFFER, null);
        pipe.bindTargetFrameBuffer(true);
        this.m_bufferDebugInfo.setTexture(pipe.mainDepthTexture);
    };
    return PassDepth;
}(RenderPass_1.RenderPass));
exports.PassDepth = PassDepth;
