"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Shader_1 = require("../shaderfx/Shader");
var RenderPass_1 = require("./RenderPass");
var MeshRender_1 = require("../MeshRender");
var PassOpaque = /** @class */ (function (_super) {
    __extends(PassOpaque, _super);
    function PassOpaque(pipeline) {
        var _this = _super.call(this, pipeline) || this;
        console.log('init opaque');
        var deftags = new Shader_1.ShaderTags();
        deftags.blendOp = null;
        deftags.blend = false;
        deftags.zwrite = false;
        deftags.ztest = Shader_1.Comparison.LEQUAL;
        deftags.culling = Shader_1.CullingMode.Back;
        deftags.fillDefaultVal();
        _this.m_tags = deftags;
        var gl = pipeline.GL;
        gl.polygonOffset(-1, -1);
        return _this;
    }
    PassOpaque.prototype.render = function (scene) {
        var queue = this.pipeline.nodeList.nodeOpaque;
        var pipe = this.pipeline;
        var gl = pipe.GL;
        var glctx = pipe.GLCtx;
        var deftags = this.m_tags;
        var cam = scene.mainCamera;
        if (queue.length == 0)
            return;
        gl.enable(gl.POLYGON_OFFSET_FILL);
        //light
        var light = scene.lights[0];
        if (light != null && light.isDirty) {
            var datalight = pipe.shaderDataLight;
            datalight.setLightData(light.lightPosData, light.lightType, 0);
            datalight.setLightColorIntensity(light.lightColor, light.intensity, 0);
            datalight.setAmbientColor(cam.ambientColor);
            pipe.updateUniformBufferLight(datalight);
        }
        //sm
        var state = pipe.stateCache;
        state.reset(deftags);
        pipe.activeDefaultTexture();
        //do draw
        var len = queue.length;
        var curprogram = null;
        var dataobj = pipe.shaderDataObj;
        for (var i = 0; i < len; i++) {
            var node = queue[i];
            if (node instanceof MeshRender_1.MeshRender) {
                var mat = node.material;
                var mesh = node.mesh;
                var program = mat.program;
                node.refreshData(glctx);
                if (program != curprogram) {
                    var glp = program.Program;
                    gl.useProgram(glp);
                    pipe.uniformBindDefault(program);
                    curprogram = program;
                }
                state.apply(mat.shaderTags);
                mat.apply(gl);
                dataobj.setMtxModel(node.object.transform.objMatrix);
                pipe.updateUniformBufferObject(dataobj);
                node.bindVertexArray(gl);
                var indicedesc = mesh.indiceDesc;
                gl.drawElements(gl.TRIANGLES, indicedesc.indiceCount, indicedesc.type, indicedesc.offset);
                node.unbindVertexArray(gl);
                mat.clean(gl);
            }
        }
        gl.disable(gl.POLYGON_OFFSET_FILL);
    };
    return PassOpaque;
}(RenderPass_1.RenderPass));
exports.PassOpaque = PassOpaque;
