"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var PipelineBase_1 = require("../pipeline/PipelineBase");
var MeshRender_1 = require("../MeshRender");
var GLMath_1 = require("../math/GLMath");
var ShaderFX_1 = require("../shaderfx/ShaderFX");
var Shadow_1 = require("./Shadow");
var Texture_1 = require("../Texture");
var Light_1 = require("../Light");
var BufferDebugInfo_1 = require("./BufferDebugInfo");
var Mesh_1 = require("../Mesh");
var Material_1 = require("../Material");
var RenderPass_1 = require("./RenderPass");
var PassShadowMap = /** @class */ (function (_super) {
    __extends(PassShadowMap, _super);
    function PassShadowMap(pipeline) {
        var _this = _super.call(this, pipeline) || this;
        _this.initShadowMaps();
        return _this;
    }
    PassShadowMap.prototype.initShadowMaps = function () {
        var pipe = this.pipeline;
        var gl = pipe.GL;
        var glctx = pipe.GLCtx;
        var config = pipe.graphicRender.shadowConfig;
        var shader = pipe.graphicRender.shaderLib.shaderShadowMap;
        var program = shader.defaultProgram;
        this.m_shader = shader;
        this.m_program = program;
        this.m_uniformLightVP = program.Uniforms['uLightVP'];
        var ublocks = program.UniformBlock;
        var indexCam = ublocks[ShaderFX_1.ShaderFX.UNIFORM_BASIS];
        var indexObj = ublocks[ShaderFX_1.ShaderFX.UNIFORM_OBJ];
        this.m_blockIndexBasis = indexCam;
        this.m_blockIndexObj = indexObj;
        var size = config.shadowmapSize;
        var smheight = size;
        var smwidth = size;
        if (config.cascade == Shadow_1.ShadowCascade.TwoCascade) {
            smwidth *= 2;
        }
        this.m_smheight = smheight;
        this.m_smwidth = smwidth;
        //depth texture and framebuffer
        var smtexdesc = new Texture_1.TextureCreationDesc(null, gl.DEPTH_COMPONENT24, false, gl.NEAREST, gl.NEAREST);
        var smtex = Texture_1.Texture.createTexture2D(smwidth, smheight, smtexdesc, glctx);
        this.m_smtex = smtex;
        gl.activeTexture(gl.TEXTURE12);
        gl.bindTexture(gl.TEXTURE_2D, smtex.rawtexture);
        var smfb = gl.createFramebuffer();
        this.m_smfb = smfb;
        gl.bindFramebuffer(gl.DRAW_FRAMEBUFFER, smfb);
        gl.framebufferTexture2D(gl.DRAW_FRAMEBUFFER, gl.DEPTH_ATTACHMENT, gl.TEXTURE_2D, smtex.rawtexture, 0);
        var status = gl.checkFramebufferStatus(gl.DRAW_FRAMEBUFFER);
        if (status != gl.FRAMEBUFFER_COMPLETE) {
            console.error('fb status incomplete ' + status.toString(16));
        }
        gl.bindFramebuffer(gl.DRAW_FRAMEBUFFER, null);
        var debuginfo = new BufferDebugInfo_1.BufferDebugInfo(this.m_smtex, GLMath_1.glmath.vec4(200, 0, 200, 200));
        pipe.addBufferDebugInfo(debuginfo);
        //shadow gather
        if (PassShadowMap.s_shadowGatherShader == null) {
            var gathersh = ShaderFX_1.ShaderFX.compileShaders(glctx, PassShadowMap.SH_shadowGather);
            PassShadowMap.s_shadowGatherShader = gathersh;
        }
        var gathermat = new Material_1.Material(PassShadowMap.s_shadowGatherShader);
        var gatherProj = gathermat.program;
        this.m_gatherMat = gathermat;
        gathermat.setTexture("uDepthTexure", pipe.mainDepthTexture);
        gathermat.setTexture("uShadowMap", this.m_smtex);
        this.m_quadMesh = Mesh_1.Mesh.Quad;
        this.m_quadVAO = MeshRender_1.MeshRender.CreateVertexArrayObj(glctx, this.m_quadMesh, gatherProj);
        var texdesc = new Texture_1.TextureCreationDesc(gl.RGB, gl.RGB8, false, gl.LINEAR, gl.LINEAR);
        var stex = Texture_1.Texture.createTexture2D(pipe.mainFrameBufferWidth, pipe.mainFrameBufferHeight, texdesc, glctx);
        this.m_shadowTexture = stex;
        var sfb = gl.createFramebuffer();
        gl.bindFramebuffer(gl.DRAW_FRAMEBUFFER, sfb);
        gl.framebufferTexture2D(gl.DRAW_FRAMEBUFFER, gl.COLOR_ATTACHMENT0, gl.TEXTURE_2D, stex.rawtexture, 0);
        gl.bindFramebuffer(gl.DRAW_FRAMEBUFFER, null);
        this.m_shadowFB = sfb;
        var debugshadows = new BufferDebugInfo_1.BufferDebugInfo(stex, GLMath_1.glmath.vec4(0, 200, 200, 200));
        pipe.addBufferDebugInfo(debugshadows);
    };
    PassShadowMap.prototype.render = function (scene) {
        var CLASS = PipelineBase_1.PipelineBase;
        var queue = this.pipeline.nodeList.nodeOpaque;
        var cam = scene.mainCamera;
        if (cam == null)
            return;
        var pipe = this.pipeline;
        cam.aspect = pipe.mainFrameBufferAspect;
        var config = pipe.graphicRender.shadowConfig;
        //use program 
        var program = this.m_program;
        var gl = pipe.GL;
        var glp = program.Program;
        gl.useProgram(glp);
        gl.uniformBlockBinding(glp, this.m_blockIndexBasis, CLASS.UNIFORMINDEX_BASIS);
        gl.uniformBlockBinding(glp, this.m_blockIndexObj, CLASS.UNIFORMINDEX_OBJ);
        var lights = scene.lights;
        for (var i = 0, lcount = lights.length; i < lcount; i++) {
            this.renderLightShadowMap(lights[i], cam, queue, config);
        }
        //update shadowmap uniform buffer
        var smdata = this.pipeline.shaderDataShadowMap;
        pipe.updateUniformBufferShadowMap(smdata);
        pipe.bindTargetFrameBuffer(true);
    };
    PassShadowMap.prototype.renderLightShadowMap = function (light, camera, queue, config) {
        if (light.lightType != Light_1.LightType.direction)
            return;
        var pipe = this.pipeline;
        var gl = pipe.GL;
        var smdata = pipe.shaderDataShadowMap;
        gl.bindFramebuffer(gl.DRAW_FRAMEBUFFER, this.m_smfb);
        gl.enable(gl.DEPTH_TEST);
        gl.depthFunc(gl.LEQUAL);
        gl.clearDepth(1.0);
        gl.clear(gl.DEPTH_BUFFER_BIT);
        gl.viewport(0, 0, this.m_smwidth, this.m_smheight);
        var lightMtxs = this.calculateLightMatrix(light, camera, config);
        var _a = lightMtxs[0], lightworldMtx = _a[0], lightProjMtx = _a[1];
        var lightMtxVP = lightProjMtx.mul(lightworldMtx);
        smdata.setLightMtx(lightMtxVP, 0);
        pipe.shadowMapData.lightMtx0 = lightMtxVP;
        var cascades = config.cascade;
        var size = this.m_smheight;
        if (cascades == 1) {
            this.renderShadowCascade(GLMath_1.glmath.vec4(0, 0, size, size), queue, lightMtxVP);
        }
        else if (cascades == 2) {
            this.renderShadowCascade(GLMath_1.glmath.vec4(0, 0, size, size), queue, lightMtxVP);
            this.renderShadowCascade(GLMath_1.glmath.vec4(size, 0, size, size), queue, lightMtxVP);
        }
        else {
            this.renderShadowCascade(GLMath_1.glmath.vec4(0, 0, size, size), queue, lightMtxVP);
            this.renderShadowCascade(GLMath_1.glmath.vec4(size, 0, size, size), queue, lightMtxVP);
            this.renderShadowCascade(GLMath_1.glmath.vec4(0, size, size, size), queue, lightMtxVP);
            this.renderShadowCascade(GLMath_1.glmath.vec4(size, size, size, size), queue, lightMtxVP);
        }
        gl.bindFramebuffer(gl.DRAW_FRAMEBUFFER, null);
    };
    PassShadowMap.prototype.calculateLightMatrix = function (light, camera, config) {
        var ctrs = camera.transform;
        var near = camera.near;
        var far = camera.far;
        var camdist = far - near;
        var shadowDis = Math.min(camdist, config.shadowDistance);
        var cascades = config.cascade;
        var cascadeSplit = config.cascadeSplit;
        var fardist = near;
        var neardist = near;
        var campos = ctrs.position;
        var camforward = ctrs.worldForward;
        var hCoefficient = Math.tan(camera.fov / 2.0 * GLMath_1.glmath.Deg2Rad);
        var wCoefficient = hCoefficient * camera.aspect;
        var ldir = light.lightPosData;
        var lup = GLMath_1.vec3.up;
        if (Math.abs(GLMath_1.vec3.Dot(lup, ldir)) > 0.99) {
            lup = GLMath_1.glmath.vec3(0, 1, 0.001);
        }
        var ret = [];
        for (var i = 0; i < cascades; i++) {
            var dist = cascadeSplit[i] * shadowDis;
            fardist += dist;
            var d = dist * 0.5;
            var cdist = neardist + d;
            var cpos = campos.clone().sub(camforward.clone().mul(cdist));
            var h = fardist * hCoefficient;
            var w = fardist * wCoefficient;
            var r = Math.sqrt(h * h + d * d + w * w);
            var lpos = cpos.sub(ldir.mulToRef(r));
            var vmtx = GLMath_1.mat4.coordCvt(lpos, ldir, lup);
            var pmtx = GLMath_1.mat4.orthographic(r, r, 0.1, r * 2.0);
            ret.push([vmtx, pmtx]);
            //next frausta
            neardist += dist;
        }
        return ret;
    };
    PassShadowMap.prototype.renderShadowCascade = function (vp, queue, lmtxVP) {
        var pipe = this.pipeline;
        var glctx = pipe.GLCtx;
        var gl = glctx.gl;
        gl.uniformMatrix4fv(this.m_uniformLightVP, false, lmtxVP.raw);
        gl.viewport(vp.x, vp.y, vp.z, vp.w);
        var objdata = pipe.shaderDataObj;
        var queueLen = queue.length;
        for (var i = 0; i < queueLen; i++) {
            var node = queue[i];
            if (!node.castShadow)
                continue;
            if (node instanceof MeshRender_1.MeshRender) {
                var mat = node.material;
                var mesh = node.mesh;
                if (mat == null || mesh == null)
                    continue;
                node.refreshData(glctx);
                var trs = node.object.transform;
                objdata.setMtxModel(trs.objMatrix);
                pipe.updateUniformBufferObject(objdata);
                node.bindVertexArray(gl);
                var indicesDesc = mesh.indiceDesc;
                gl.drawElements(indicesDesc.topology, indicesDesc.indiceCount, indicesDesc.type, indicesDesc.offset);
                node.unbindVertexArray(gl);
            }
        }
    };
    PassShadowMap.prototype.shadowGathering = function (light) {
        var CLASS = PipelineBase_1.PipelineBase;
        var dataSM = this.pipeline.shaderDataShadowMap;
        dataSM.setLightMtx(this.pipeline.shadowMapData.lightMtx0, 0);
        this.pipeline.updateUniformBufferShadowMap(dataSM);
        var gl = this.pipeline.GL;
        gl.bindFramebuffer(gl.DRAW_FRAMEBUFFER, this.m_shadowFB);
        gl.clearColor(0, 0, 0, 0);
        gl.clear(gl.COLOR_BUFFER_BIT);
        var shadowtex = this.m_shadowTexture;
        gl.viewport(0, 0, shadowtex.width, shadowtex.height);
        var mat = this.m_gatherMat;
        var program = mat.program;
        var glp = program.Program;
        gl.useProgram(glp);
        var blocks = program.UniformBlock;
        var indexSM = blocks[ShaderFX_1.ShaderFX.UNIFORM_SHADOWMAP];
        if (indexSM != null) {
            gl.uniformBlockBinding(glp, indexSM, CLASS.UNIFORMINDEX_SHADOWMAP);
        }
        var indexBasis = blocks[ShaderFX_1.ShaderFX.UNIFORM_BASIS];
        if (indexBasis != null) {
            gl.uniformBlockBinding(glp, indexBasis, CLASS.UNIFORMINDEX_BASIS);
        }
        var indexObj = blocks[ShaderFX_1.ShaderFX.UNIFORM_OBJ];
        if (indexObj != null) {
            gl.uniformBlockBinding(glp, indexObj, CLASS.UNIFORMINDEX_OBJ);
        }
        mat.apply(gl);
        var mesh = this.m_quadMesh;
        var vao = this.m_quadVAO;
        gl.bindVertexArray(vao);
        var indices = mesh.indiceDesc;
        gl.drawElements(gl.TRIANGLES, indices.indiceCount, indices.type, indices.offset);
        gl.bindVertexArray(null);
        gl.bindFramebuffer(gl.DRAW_FRAMEBUFFER, null);
    };
    __decorate([
        ShaderFX_1.ShaderFile("shadowsGather")
    ], PassShadowMap, "SH_shadowGather", void 0);
    return PassShadowMap;
}(RenderPass_1.RenderPass));
exports.PassShadowMap = PassShadowMap;
