import { ShaderSource } from "./ShaderSource";
import { ShaderOptionsConfig } from "./ShaderVariant";
import { GLProgram } from "../gl/GLProgram";
import { GLContext } from "../gl/GLContext";
export declare enum RenderQueue {
    Opaque = 0,
    Transparent = 1,
    Skybox = 2,
    Image = 3,
    Other = 4
}
export declare enum Comparison {
    NEVER = 512,
    LESS = 513,
    EQUAL = 514,
    LEQUAL = 515,
    GREATER = 516,
    NOTEQUAL = 517,
    GEQUAL = 518,
    ALWAYS = 519
}
export declare enum BlendOperator {
    ADD = 32774,
    MIN = 32775,
    MAX = 32776,
    SUBTRACT = 32778,
    RESERVE_SUBSTRACT = 32779
}
export declare enum BlendFactor {
    ONE = 1,
    ZERO = 0,
    SRC_COLOR = 768,
    ONE_MINUS_SRC_COLOR = 769,
    SRC_ALPHA = 770,
    ONE_MINUS_SRC_ALPHA = 771,
    DST_ALPHA = 772,
    ONE_MINUS_DST_ALPHA = 773,
    DST_COLOR = 774,
    ONE_MINUS_DST_COLOR = 775,
    SRC_ALPHA_SATURATE = 776,
    CONSTANT_ALPHA = 32771,
    ONE_MINUS_CONSTANT_ALPHA = 32772,
    CONSTANT_COLOR = 32769,
    ONE_MINUS_CONSTANT_COLOR = 32770
}
export declare enum CullingMode {
    Front = 1028,
    Back = 1029,
    FRONT_AND_BACK = 1032,
    None = 2884
}
export declare class ShaderTags {
    queue?: RenderQueue;
    ztest?: Comparison;
    zwrite?: boolean;
    blend: boolean;
    blendOp?: BlendOperator;
    blendFactorSrc?: BlendFactor;
    blendFactorDst?: BlendFactor;
    culling?: CullingMode;
    clone(): ShaderTags;
    fillDefaultVal(): void;
    toString(): string;
}
export declare class ShaderProgram {
    hash: string;
    private glprogram;
    readonly glProgram: WebGLProgram;
}
export declare class ShaderPass {
    name: string;
    program: ShaderProgram;
}
export declare class Shader {
    private m_source;
    private m_defaultProgram;
    private m_compiledPrograms;
    tags: ShaderTags;
    m_defaultOptionsConfig: ShaderOptionsConfig;
    private m_glctx;
    constructor(source: ShaderSource, defaultProgram: GLProgram, defOptConfig: ShaderOptionsConfig, glctx: GLContext);
    readonly source: ShaderSource;
    readonly defaultProgram: GLProgram;
    getVariantProgram(optconfig: ShaderOptionsConfig): GLProgram;
    release(): void;
    static ParseShaderInfo(source: string, info: string): void;
    static CreateProgram(glctx: GLContext, vsource: string, psource: string): GLProgram;
}
