"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var GLProgram_1 = require("../gl/GLProgram");
var RenderQueue;
(function (RenderQueue) {
    RenderQueue[RenderQueue["Opaque"] = 0] = "Opaque";
    RenderQueue[RenderQueue["Transparent"] = 1] = "Transparent";
    RenderQueue[RenderQueue["Skybox"] = 2] = "Skybox";
    RenderQueue[RenderQueue["Image"] = 3] = "Image";
    RenderQueue[RenderQueue["Other"] = 4] = "Other";
})(RenderQueue = exports.RenderQueue || (exports.RenderQueue = {}));
var Comparison;
(function (Comparison) {
    Comparison[Comparison["NEVER"] = 512] = "NEVER";
    Comparison[Comparison["LESS"] = 513] = "LESS";
    Comparison[Comparison["EQUAL"] = 514] = "EQUAL";
    Comparison[Comparison["LEQUAL"] = 515] = "LEQUAL";
    Comparison[Comparison["GREATER"] = 516] = "GREATER";
    Comparison[Comparison["NOTEQUAL"] = 517] = "NOTEQUAL";
    Comparison[Comparison["GEQUAL"] = 518] = "GEQUAL";
    Comparison[Comparison["ALWAYS"] = 519] = "ALWAYS";
})(Comparison = exports.Comparison || (exports.Comparison = {}));
var BlendOperator;
(function (BlendOperator) {
    BlendOperator[BlendOperator["ADD"] = 32774] = "ADD";
    BlendOperator[BlendOperator["MIN"] = 32775] = "MIN";
    BlendOperator[BlendOperator["MAX"] = 32776] = "MAX";
    BlendOperator[BlendOperator["SUBTRACT"] = 32778] = "SUBTRACT";
    BlendOperator[BlendOperator["RESERVE_SUBSTRACT"] = 32779] = "RESERVE_SUBSTRACT";
})(BlendOperator = exports.BlendOperator || (exports.BlendOperator = {}));
var BlendFactor;
(function (BlendFactor) {
    BlendFactor[BlendFactor["ONE"] = 1] = "ONE";
    BlendFactor[BlendFactor["ZERO"] = 0] = "ZERO";
    BlendFactor[BlendFactor["SRC_COLOR"] = 768] = "SRC_COLOR";
    BlendFactor[BlendFactor["ONE_MINUS_SRC_COLOR"] = 769] = "ONE_MINUS_SRC_COLOR";
    BlendFactor[BlendFactor["SRC_ALPHA"] = 770] = "SRC_ALPHA";
    BlendFactor[BlendFactor["ONE_MINUS_SRC_ALPHA"] = 771] = "ONE_MINUS_SRC_ALPHA";
    BlendFactor[BlendFactor["DST_ALPHA"] = 772] = "DST_ALPHA";
    BlendFactor[BlendFactor["ONE_MINUS_DST_ALPHA"] = 773] = "ONE_MINUS_DST_ALPHA";
    BlendFactor[BlendFactor["DST_COLOR"] = 774] = "DST_COLOR";
    BlendFactor[BlendFactor["ONE_MINUS_DST_COLOR"] = 775] = "ONE_MINUS_DST_COLOR";
    BlendFactor[BlendFactor["SRC_ALPHA_SATURATE"] = 776] = "SRC_ALPHA_SATURATE";
    BlendFactor[BlendFactor["CONSTANT_ALPHA"] = 32771] = "CONSTANT_ALPHA";
    BlendFactor[BlendFactor["ONE_MINUS_CONSTANT_ALPHA"] = 32772] = "ONE_MINUS_CONSTANT_ALPHA";
    BlendFactor[BlendFactor["CONSTANT_COLOR"] = 32769] = "CONSTANT_COLOR";
    BlendFactor[BlendFactor["ONE_MINUS_CONSTANT_COLOR"] = 32770] = "ONE_MINUS_CONSTANT_COLOR";
})(BlendFactor = exports.BlendFactor || (exports.BlendFactor = {}));
var CullingMode;
(function (CullingMode) {
    CullingMode[CullingMode["Front"] = 1028] = "Front";
    CullingMode[CullingMode["Back"] = 1029] = "Back";
    CullingMode[CullingMode["FRONT_AND_BACK"] = 1032] = "FRONT_AND_BACK";
    CullingMode[CullingMode["None"] = 2884] = "None";
})(CullingMode = exports.CullingMode || (exports.CullingMode = {}));
var ShaderTags = /** @class */ (function () {
    function ShaderTags() {
        this.blend = false;
    }
    ShaderTags.prototype.clone = function () {
        var tags = new ShaderTags();
        tags.queue = this.queue;
        tags.ztest = this.ztest;
        tags.zwrite = this.zwrite;
        tags.blendOp = this.blendOp;
        tags.blendFactorDst = this.blendFactorDst;
        tags.blendFactorSrc = this.blendFactorSrc;
        tags.culling = this.culling;
        return tags;
    };
    ShaderTags.prototype.fillDefaultVal = function () {
        if (this.queue == null)
            this.queue = RenderQueue.Opaque;
        if (this.zwrite == null)
            this.zwrite = true;
        if (this.ztest == null)
            this.ztest = Comparison.LEQUAL;
        if (this.culling == null)
            this.culling = CullingMode.Back;
        if (this.blendOp == null)
            this.blendOp = BlendOperator.ADD;
        if (this.blendFactorSrc == null)
            this.blendFactorSrc = BlendFactor.SRC_ALPHA;
        if (this.blendFactorDst == null)
            this.blendFactorDst = BlendFactor.ONE_MINUS_SRC_ALPHA;
    };
    ShaderTags.prototype.toString = function () {
        return "\n            queue:" + RenderQueue[this.queue] + "\n            ztest:" + Comparison[this.ztest] + "\n            zwrite:" + this.zwrite + "\n            blend:" + this.blend + "\n            blendOp:" + BlendOperator[this.blendOp] + "\n            blendSrc:" + BlendFactor[this.blendFactorSrc] + "\n            blendDst:" + BlendFactor[this.blendFactorDst] + "\n            culling:" + CullingMode[this.culling] + "\n        ";
    };
    return ShaderTags;
}());
exports.ShaderTags = ShaderTags;
var ShaderProgram = /** @class */ (function () {
    function ShaderProgram() {
    }
    Object.defineProperty(ShaderProgram.prototype, "glProgram", {
        get: function () {
            return this.glprogram.Program;
        },
        enumerable: true,
        configurable: true
    });
    return ShaderProgram;
}());
exports.ShaderProgram = ShaderProgram;
var ShaderPass = /** @class */ (function () {
    function ShaderPass() {
    }
    return ShaderPass;
}());
exports.ShaderPass = ShaderPass;
var Shader = /** @class */ (function () {
    function Shader(source, defaultProgram, defOptConfig, glctx) {
        this.m_compiledPrograms = {};
        this.m_source = source;
        this.m_defaultProgram = defaultProgram;
        this.m_defaultOptionsConfig = defOptConfig;
        this.m_compiledPrograms[defOptConfig.hashCode] = defaultProgram;
        this.m_glctx = glctx;
    }
    Object.defineProperty(Shader.prototype, "source", {
        get: function () {
            return this.m_source;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Shader.prototype, "defaultProgram", {
        get: function () {
            return this.m_defaultProgram;
        },
        enumerable: true,
        configurable: true
    });
    Shader.prototype.getVariantProgram = function (optconfig) {
        if (optconfig == null)
            throw new Error('optconfig is null');
        var hash = optconfig.hashCode;
        var cachedProgram = this.m_compiledPrograms[hash];
        if (cachedProgram != null) {
            return cachedProgram;
        }
        else {
            var source = this.source;
            var _a = source.injectCompileFlags(optconfig.compileFlag), vs = _a[0], ps = _a[1];
            var program = Shader.CreateProgram(this.m_glctx, vs, ps);
            if (program == null)
                throw new Error("compile program failed");
            this.m_compiledPrograms[hash] = program;
            console.log("program hash " + hash);
            return program;
        }
    };
    Shader.prototype.release = function () {
        this.m_glctx = null;
    };
    Shader.ParseShaderInfo = function (source, info) {
        var regexp = /ERROR: 0:([\d]+):/g;
        var split = source.split(/\r\n|\r|\n/);
        var ary = null;
        while ((ary = regexp.exec(info)) !== null) {
            console.error(split[Number(ary[1]) - 1]);
        }
    };
    Shader.CreateProgram = function (glctx, vsource, psource) {
        var gl = glctx.gl;
        var vs = gl.createShader(gl.VERTEX_SHADER);
        gl.shaderSource(vs, vsource);
        gl.compileShader(vs);
        if (!gl.getShaderParameter(vs, gl.COMPILE_STATUS)) {
            console.error(vsource);
            var infolog = gl.getShaderInfoLog(vs);
            console.error('compile vertex shader failed: ' + infolog);
            Shader.ParseShaderInfo(vsource, infolog);
            gl.deleteShader(vs);
            return null;
        }
        var ps = gl.createShader(gl.FRAGMENT_SHADER);
        gl.shaderSource(ps, psource);
        gl.compileShader(ps);
        if (!gl.getShaderParameter(ps, gl.COMPILE_STATUS)) {
            console.error(psource);
            var infolog = gl.getShaderInfoLog(ps);
            console.error('compile fragment shader failed: ' + infolog);
            Shader.ParseShaderInfo(psource, infolog);
            gl.deleteShader(ps);
            return null;
        }
        var program = gl.createProgram();
        gl.attachShader(program, vs);
        gl.attachShader(program, ps);
        gl.linkProgram(program);
        if (!gl.getProgramParameter(program, gl.LINK_STATUS)) {
            console.error('link shader program failed!:' + gl.getProgramInfoLog(program));
            gl.deleteProgram(program);
            gl.deleteShader(vs);
            gl.deleteShader(ps);
            return null;
        }
        if (program == null) {
            throw new Error('compile shader error');
        }
        var p = new GLProgram_1.GLProgram(gl, program);
        return p;
    };
    return Shader;
}());
exports.Shader = Shader;
