"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ShaderBuffer = /** @class */ (function () {
    function ShaderBuffer(bytesize) {
        this.minoff = 0;
        this.maxoff = 0;
        this.m_isdirty = false;
        this.maxoff = 0;
        this.minoff = bytesize;
        var buffer = new ArrayBuffer(bytesize);
        this.rawbuffer = buffer;
        this.dataView = new DataView(buffer);
        this.byteArray = new Uint8Array(buffer);
    }
    Object.defineProperty(ShaderBuffer.prototype, "isDirty", {
        get: function () { return this.m_isdirty; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ShaderBuffer.prototype, "offsetMin", {
        get: function () { return this.minoff; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ShaderBuffer.prototype, "offsetMax", {
        get: function () { return this.maxoff; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ShaderBuffer.prototype, "raw", {
        get: function () {
            return this.byteArray;
        },
        enumerable: true,
        configurable: true
    });
    ShaderBuffer.prototype.setDirty = function (d) {
        this.m_isdirty = d;
        if (!d) {
            this.minoff = this.rawbuffer.byteLength;
            this.maxoff = 0;
        }
    };
    ShaderBuffer.prototype.set = function (fxbuffer, off) {
        this.byteArray.set(fxbuffer.byteArray, off);
    };
    ShaderBuffer.prototype.setOfView = function (fxview) {
        this.set(fxview.buffer, fxview.viewbase);
    };
    ShaderBuffer.prototype.setOfSubData = function (fxsubdata) {
        this.set(fxsubdata.bufferView.buffer, fxsubdata.baseOffset);
    };
    ShaderBuffer.prototype.setMinMaxOffset = function (byteOffset, length) {
        if (byteOffset < this.minoff)
            this.minoff = byteOffset;
        var max = byteOffset + length;
        if (max > this.maxoff)
            this.maxoff = max;
        this.m_isdirty = true;
    };
    ShaderBuffer.prototype.setFloat = function (byteOffset, value) {
        this.setMinMaxOffset(byteOffset, 4);
        this.dataView.setFloat32(byteOffset, value, true);
    };
    ShaderBuffer.prototype.setUint32 = function (byteOffset, value) {
        this.setMinMaxOffset(byteOffset, 4);
        this.dataView.setUint32(byteOffset, value);
    };
    ShaderBuffer.prototype.setVec3 = function (byteOffset, value) {
        this.setMinMaxOffset(byteOffset, 12);
        var raw = value.raw;
        var dv = this.dataView;
        var off = byteOffset;
        dv.setFloat32(off, raw[0], true);
        off += 4;
        dv.setFloat32(off, raw[1], true);
        off += 4;
        dv.setFloat32(off, raw[2], true);
    };
    ShaderBuffer.prototype.setVec4 = function (byteOffset, value) {
        this.setMinMaxOffset(byteOffset, 16);
        var raw = value.raw;
        var dv = this.dataView;
        var off = byteOffset;
        dv.setFloat32(off, raw[0], true);
        off += 4;
        dv.setFloat32(off, raw[1], true);
        off += 4;
        dv.setFloat32(off, raw[2], true);
        off += 4;
        dv.setFloat32(off, raw[3], true);
    };
    ShaderBuffer.prototype.setMat4 = function (byteOffset, value) {
        this.setMinMaxOffset(byteOffset, 64);
        var raw = value.raw;
        var dv = this.dataView;
        var len = raw.length;
        var off = byteOffset;
        for (var i = 0; i < len; i++) {
            dv.setFloat32(off, raw[i], true);
            off += 4;
        }
    };
    ShaderBuffer.prototype.setMat3 = function (byteOffset, value) {
        this.setMinMaxOffset(byteOffset, 36);
        var raw = value.raw;
        var dv = this.dataView;
        var len = raw.length;
        var off = byteOffset;
        for (var i = 0; i < len; i++) {
            dv.setFloat32(off, raw[i], true);
            off += 4;
        }
    };
    return ShaderBuffer;
}());
var ShaderBufferView = /** @class */ (function () {
    function ShaderBufferView(buffer, byteoffset, bytelength) {
        this.base = 0;
        this.fxbuffer = buffer;
        this.base = byteoffset;
    }
    Object.defineProperty(ShaderBufferView.prototype, "buffer", {
        get: function () {
            return this.fxbuffer;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ShaderBufferView.prototype, "viewbase", {
        get: function () {
            return this.base;
        },
        enumerable: true,
        configurable: true
    });
    ShaderBufferView.prototype.setFloat = function (byteoffset, val) {
        var base = this.base;
        this.fxbuffer.setFloat(base + byteoffset, val);
    };
    ShaderBufferView.prototype.setUint32 = function (byteoffset, val) {
        var base = this.base;
        this.fxbuffer.setUint32(base + byteoffset, val);
    };
    ShaderBufferView.prototype.setVec3 = function (byteoffset, val) {
        var base = this.base;
        this.fxbuffer.setVec3(base + byteoffset, val);
    };
    ShaderBufferView.prototype.setVec4 = function (byteoffset, val) {
        var base = this.base;
        this.fxbuffer.setVec4(base + byteoffset, val);
    };
    ShaderBufferView.prototype.setMat4 = function (byteoffset, val) {
        var base = this.base;
        this.fxbuffer.setMat4(base + byteoffset, val);
    };
    ShaderBufferView.prototype.setMat3 = function (byteoffset, val) {
        var base = this.base;
        this.fxbuffer.setMat3(base + byteoffset, val);
    };
    return ShaderBufferView;
}());
var ShaderSubData = /** @class */ (function () {
    function ShaderSubData(data, length, offset) {
        this.m_seperated = false;
        this.byteLength = length;
        if (data == null) {
            var buffer = new ShaderBuffer(length);
            this.view = new ShaderBufferView(buffer, 0, length);
            this.m_seperated = true;
        }
        else {
            this.view = new ShaderBufferView(data.fxbuffer, offset, length);
        }
        this.baseOffset = offset;
    }
    Object.defineProperty(ShaderSubData.prototype, "isSeperated", {
        get: function () { return this.m_seperated; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ShaderSubData.prototype, "bufferView", {
        get: function () {
            return this.view;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ShaderSubData.prototype, "isDirty", {
        get: function () {
            return this.view.buffer.isDirty;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ShaderSubData.prototype, "setDirty", {
        set: function (v) {
            this.view.buffer.setDirty(v);
        },
        enumerable: true,
        configurable: true
    });
    return ShaderSubData;
}());
exports.ShaderSubData = ShaderSubData;
var ShaderData = /** @class */ (function () {
    function ShaderData(bytelength) {
        this.buffer = new ShaderBuffer(bytelength);
    }
    Object.defineProperty(ShaderData.prototype, "fxbuffer", {
        get: function () {
            return this.buffer;
        },
        enumerable: true,
        configurable: true
    });
    ShaderData.prototype.submitBuffer = function (gl, glbuffer) {
        var fxbuffer = this.buffer;
        if (!fxbuffer.isDirty)
            return false;
        var minoff = fxbuffer.offsetMin;
        var maxoff = fxbuffer.offsetMax;
        if (minoff >= maxoff) {
            fxbuffer.setDirty(false);
            return false;
        }
        gl.bindBuffer(gl.UNIFORM_BUFFER, glbuffer);
        gl.bufferSubData(gl.UNIFORM_BUFFER, minoff, fxbuffer.raw, minoff, maxoff - minoff);
        gl.bindBuffer(gl.UNIFORM_BUFFER, null);
        fxbuffer.setDirty(false);
        return true;
    };
    return ShaderData;
}());
exports.ShaderData = ShaderData;
