"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ShaderVariant_1 = require("./ShaderVariant");
var ShaderSource_1 = require("./ShaderSource");
var Shader_1 = require("./Shader");
var ShaderGenerated_1 = require("./ShaderGenerated");
var ShaderFX = /** @class */ (function () {
    function ShaderFX() {
    }
    ShaderFX.registVariant = function (variant) {
        this.variants[variant.variantName] = variant;
    };
    ShaderFX.linkAllVariant = function () {
        var variants = ShaderFX.variants;
        for (var key in variants) {
            var v = variants[key];
            if (v != null && v instanceof ShaderVariant_1.ShaderVariant) {
                v.link(variants);
            }
        }
    };
    ShaderFX.getShader = function () {
    };
    ShaderFX.compileShaders = function (glctx, source) {
        if (!ShaderFX.s_variantsLinked) {
            ShaderFX.linkAllVariant();
            ShaderFX.s_variantsLinked = true;
        }
        if (source == null)
            return null;
        source.buildShader(ShaderFX.variants);
        if (!source.isBuilt)
            return null;
        var optconfig = new ShaderVariant_1.ShaderOptionsConfig(source.optionsList);
        var compileFlags = optconfig.compileFlag;
        var _a = source.injectCompileFlags(compileFlags), vs = _a[0], ps = _a[1];
        var p = Shader_1.Shader.CreateProgram(glctx, vs, ps);
        if (p == null)
            return null;
        var shader = new Shader_1.Shader(source, p, optconfig, glctx);
        var tags = source.tags;
        if (tags == null) {
            tags = new Shader_1.ShaderTags();
        }
        tags.fillDefaultVal();
        shader.tags = tags;
        return shader;
    };
    ShaderFX.isInternalUniformBlockName = function (blockname) {
        var CLASS = ShaderFX;
        if (blockname == CLASS.UNIFORM_BASIS || blockname == CLASS.UNIFORM_OBJ || blockname == CLASS.UNIFORM_SHADOWMAP)
            return true;
        return false;
    };
    ShaderFX.variants = {};
    ShaderFX.s_variantsLinked = false;
    ShaderFX.VARIANT_SHADERFX_OBJ = "SHADERFX_OBJ";
    ShaderFX.VARIANT_SHADERFX_CAMERA = "SHADERFX_CAMERA";
    ShaderFX.VARIANT_SHADERFX_BASIS = "SHADERFX_BASIS";
    ShaderFX.VARIANT_SHADERFX_LIGHT = "SHADERFX_LIGHT";
    ShaderFX.VARIANT_SHADERFX_SHADOWMAP = "SHADERFX_SHADOWMAP";
    ShaderFX.VARIANT_SHADERFX_LIGHTING = "SHADERFX_LIGHTING";
    ShaderFX.OPT_SHADOWMAP_SHADOW = "SHADOW";
    ShaderFX.OPT_SHADOWMAP_SHADOW_ON = "ON";
    ShaderFX.OPT_SHADOWMAP_SHADOW_OFF = "OFF";
    ShaderFX.ATTR_aPosition = "aPosition";
    ShaderFX.ATTR_aUV = "aUV";
    ShaderFX.ATTR_aNormal = "aNormal";
    ShaderFX.UNIFORM_BASIS = "UNIFORM_BASIS";
    ShaderFX.UNIFORM_OBJ = "UNIFORM_OBJ";
    ShaderFX.UNIFORM_LIGHT = "UNIFORM_LIGHT";
    ShaderFX.UNIFORM_SHADOWMAP = "UNIFORM_SHADOWMAP";
    ShaderFX.UNIFORM_SHADOWMAP_SAMPLER = "uShadowMap";
    ShaderFX.UNIFORM_MAIN_COLOR = "uColor";
    ShaderFX.UNIFORM_MAIN_TEXTURE = "uSampler";
    ShaderFX.GL_TEXTURE_FB = 0x84C0;
    ShaderFX.GL_TEXTURE_DEPTH = 0x84C1;
    ShaderFX.GL_TEXTURE_TEMP = 0x84C2;
    ShaderFX.GL_TEXTURE_DEF_TEX = 0x84C3;
    ShaderFX.GL_TEXTURE_FB_ID = 0;
    ShaderFX.GL_TEXTURE_TEMP_ID = 2;
    ShaderFX.GL_TEXTURE_DEF_TEX_ID = 3;
    //Texture used in shader sampler
    ShaderFX.GL_SH_TEXTURE0 = 0x84C4;
    ShaderFX.GL_SH_TEXTURE1 = 0x84C5;
    ShaderFX.GL_SH_TEXTURE2 = 0x84C6;
    ShaderFX.GL_SH_TEXTURE3 = 0x84C7;
    ShaderFX.GL_SH_TEXTURE4 = 0x84C8;
    ShaderFX.GL_SH_TEXTURE5 = 0x84C9;
    ShaderFX.GL_SH_TEXTURE6 = 0x84C10;
    ShaderFX.GL_SH_TEXTURE7 = 0x84C11;
    ShaderFX.GL_SH_TEXTURE0_ID = 4;
    ShaderFX.GL_SH_TEXTURE1_ID = 5;
    ShaderFX.GL_SH_TEXTURE2_ID = 6;
    ShaderFX.GL_SH_TEXTURE3_ID = 7;
    ShaderFX.GL_SH_TEXTURE4_ID = 8;
    ShaderFX.GL_SH_TEXTURE5_ID = 9;
    ShaderFX.GL_SH_TEXTURE6_ID = 10;
    ShaderFX.GL_SH_TEXTURE7_ID = 11;
    ShaderFX.GL_SHADOWMAP_TEX0 = 0x84CC;
    ShaderFX.GL_SHADOWMAP_TEX1 = 0x84CD;
    ShaderFX.GL_SHADOWMAP_TEX2 = 0x84CE;
    ShaderFX.GL_SHADOWMAP_TEX3 = 0x84CF;
    ShaderFX.GL_SHADOWMAP_TEX0_ID = 12;
    ShaderFX.GL_SHADOWMAP_TEX1_ID = 13;
    ShaderFX.GL_SHADOWMAP_TEX2_ID = 14;
    ShaderFX.GL_SHADOWMAP_TEX3_ID = 15;
    return ShaderFX;
}());
exports.ShaderFX = ShaderFX;
function ShaderFile(vsfile, psfile) {
    return function (target, key) {
        if (psfile == null) {
            psfile = vsfile;
        }
        target[key] = getShaderSource(vsfile, psfile, key);
    };
}
exports.ShaderFile = ShaderFile;
function ShaderInc(filename) {
    return function (target, key) {
        var variant = getShaderInclude(filename);
        target[key] = variant;
        ShaderFX.registVariant(variant);
    };
}
exports.ShaderInc = ShaderInc;
function getShaderSource(vss, pss, name) {
    var unified = ShaderGenerated_1.ShaderGen[vss];
    if (unified != null) {
        return ShaderSource_1.ShaderSource.create(unified, name);
    }
    var vs = ShaderGenerated_1.ShaderGen[vss + '_vs'];
    var ps = ShaderGenerated_1.ShaderGen[pss + '_ps'];
    if (vs != null && ps != null) {
        return new ShaderSource_1.ShaderSource(vs, ps, name);
    }
    throw new Error("shader source invalid : " + vss + " " + pss);
}
function getShaderInclude(src) {
    var inc = ShaderGenerated_1.ShaderGen[src];
    if (inc == null) {
        throw new Error("shader include invalid : " + src);
    }
    return new ShaderVariant_1.ShaderVariant(src.toUpperCase(), inc);
}
