/// <reference types="webgl2" />
import { ShaderSource } from "./ShaderSource";
import { ShaderVariant } from "./ShaderVariant";
import { ShaderData, ShaderSubData } from "./ShaderBuffer";
import { Shader } from "./Shader";
import { vec3, vec4, mat4 } from "../math/GLMath";
import { GLContext } from "../gl/GLContext";
export declare class ShaderFXLibs {
    private glctx;
    constructor(glctx: GLContext);
    private m_unlitColor;
    private m_unlitTexture;
    private m_uvValue;
    private m_diffuse;
    private m_skybox;
    private m_depth;
    private m_shadermap;
    private m_blit;
    private m_sprite;
    private m_pbrMetallicRoughness;
    static SH_unitColor: ShaderSource;
    static SH_unlitTexture: ShaderSource;
    static SH_uvValue: ShaderSource;
    static SH_diffuse: ShaderSource;
    static SH_skybox: ShaderSource;
    static SH_pbrMetallicRoughness: ShaderSource;
    static SH_depth: ShaderSource;
    static SH_shadowmap: ShaderSource;
    static SH_blit: ShaderSource;
    static SH_sprite: ShaderSource;
    static SHADERFX_BASIS: ShaderVariant;
    static SHADERFX_LIGHT: ShaderVariant;
    static SHADERFX_LIGHTING: ShaderVariant;
    static SHADERFX_SHADOWMAP: ShaderVariant;
    readonly shaderUnlitColor: Shader;
    readonly shaderUnlitTexture: Shader;
    readonly shaderUVvalue: Shader;
    readonly shaderDiffuse: Shader;
    readonly shaderSkybox: Shader;
    readonly shaderPbrMetallicRoughness: Shader;
    readonly shaderDepth: Shader;
    readonly shaderShadowMap: Shader;
    readonly shaderBlit: Shader;
    readonly shaderSprite: Shader;
    release(): void;
    reload(): void;
    static readonly SH_PBR_BaseColorFactor: string;
    static readonly SH_PBR_BaseColorTexture: string;
    static readonly SH_PBR_MetallicFactor: string;
    static readonly SH_PBR_RoughnessFactor: string;
    static readonly SH_PBR_MetallicRoughnessTexture: string;
    static readonly SH_PBR_EmissiveFactor: string;
    static readonly SH_PBR_EmissiveTexture: string;
}
/** Shader DataBuffer */
export declare class ShaderDataUniformObj extends ShaderData {
    static readonly UNIFORM_OBJ: string;
    constructor();
    setMtxModel(mtx: mat4): void;
}
export declare class ShaderDataUniformLight extends ShaderData {
    static readonly UNIFORM_LIGHT: string;
    constructor();
    setLightData(pos: vec3, type: number, index: number): void;
    setLightColorIntensity(col: vec3, intensity: number, index: number): void;
    setAmbientColor(ambient: vec4): void;
}
export declare class ShaderDataUniformShadowMap extends ShaderData {
    constructor();
    setLightMtx(mtx: mat4, index: number): void;
    setShadowDistance(dist: null): void;
    setCascadeCount(count: number): void;
}
export declare class ShaderDataBasis extends ShaderData {
    readonly render: ShaderDataRender;
    readonly camrea: ShaderDataCamera;
    readonly ambientfog: ShaderDataAmbientFog;
    constructor();
    updateDataBasic(data: ShaderDataRender): void;
    updateDataCamera(data: ShaderDataCamera): void;
    updateDateAmbeintFog(data: ShaderDataAmbientFog): void;
    submitBuffer(gl: WebGL2RenderingContext, glbuffer: WebGLBuffer): boolean;
}
export declare class ShaderDataRender extends ShaderSubData {
    constructor(data?: ShaderData);
    setScreenParam(width: number, height: number): void;
    setTime(t: number, ts: number): void;
}
export declare class ShaderDataCamera extends ShaderSubData {
    constructor(data?: ShaderData);
    setProjParam(near: number, far: number): void;
    setCameraPos(pos: vec3): void;
    setCameraMtxView(view: mat4): void;
    setCameraMtxProj(proj: mat4, invproj?: mat4): void;
}
/**
 *
 */
export declare class ShaderDataAmbientFog extends ShaderSubData {
    constructor(data?: ShaderData);
    setAmbientColor(col: vec4): void;
    setFogColor(col: vec4): void;
    setFogParam(param: vec4): void;
}
