"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ShaderVariant_1 = require("./ShaderVariant");
var ShaderPreprocessor = /** @class */ (function () {
    function ShaderPreprocessor() {
    }
    ShaderPreprocessor.processVariantInclude = function (line, lineindex) {
        var match = ShaderPreprocessor.REGEX_INCLUDE;
        var matchInc = line.match(match);
        if (matchInc != null) {
            return { key: matchInc[1], line: lineindex };
        }
        return null;
    };
    ShaderPreprocessor.processUnifiedSource = function (unified, name) {
        var _a, _b, _c, _d;
        var pattern_pragma = /#pragma ([\w]+) ([\w]+)/g;
        var pattern_entry = /void ([\w\d]+)[\s]*\(/g;
        var vs_entry = null;
        var ps_entry = null;
        var matchPragma = unified.match(pattern_pragma);
        if (matchPragma != null) {
            for (var i = 0, mlen = matchPragma.length; i < mlen; i++) {
                var pragmastr = matchPragma[i];
                var splits = pragmastr.split(/\s+/);
                var key = splits[1];
                var val = splits[2];
                if (key === 'vs')
                    vs_entry = val;
                if (key === 'ps')
                    ps_entry = val;
            }
        }
        if (vs_entry == null)
            throw new Error("vs entry is not definded, shader:" + name);
        if (ps_entry == null)
            throw new Error("ps entry is not definded, shader:" + name);
        var vs_func = null;
        var ps_func = null;
        var matchEntry = unified.match(pattern_entry);
        if (matchEntry != null) {
            var len = matchEntry.length;
            for (var i = 0; i < len; i++) {
                var entry = matchEntry[i];
                var part = entry.split(/\s+/)[1].replace('(', '').trim();
                if (part === vs_entry) {
                    vs_func = ShaderPreprocessor.getFunctionPosIndex(unified, entry);
                    continue;
                }
                if (part === ps_entry) {
                    ps_func = ShaderPreprocessor.getFunctionPosIndex(unified, entry);
                    continue;
                }
            }
        }
        //porcess lines
        var main = unified;
        var vs_main = null;
        var ps_main = null;
        if (vs_func[0] > ps_func[0]) {
            _a = ShaderPreprocessor.Seperate(main, vs_func[0], vs_func[1]), main = _a[0], vs_main = _a[1];
            _b = ShaderPreprocessor.Seperate(main, ps_func[0], ps_func[1]), main = _b[0], ps_main = _b[1];
        }
        else {
            _c = ShaderPreprocessor.Seperate(main, ps_func[0], ps_func[1]), main = _c[0], ps_main = _c[1];
            _d = ShaderPreprocessor.Seperate(main, vs_func[0], vs_func[1]), main = _d[0], vs_main = _d[1];
        }
        var lines = main.split('\n');
        var vs_lines = [];
        var ps_lines = [];
        var pattern_varying = /^(inout|in|out)\s+[\d\w]+\s+/;
        for (var i = 0, len = lines.length; i < len; i++) {
            var line = lines[i].trimLeft();
            if (line == null || line === '')
                continue;
            var match = line.match(pattern_varying);
            if (match != null) {
                var decorator = match[1];
                if (decorator === 'inout') {
                    line = line.substr(5);
                    vs_lines.push('out' + line);
                    ps_lines.push('in' + line);
                }
                else if (decorator === 'in') {
                    vs_lines.push(line);
                }
                else if (decorator === 'out') {
                    ps_lines.push(line);
                }
            }
            else {
                vs_lines.push(line);
                ps_lines.push(line);
            }
        }
        vs_main = vs_main.replace(vs_entry, 'main');
        ps_main = ps_main.replace(ps_entry, 'main');
        var vs_final = vs_lines.join('\n') + '\n' + vs_main;
        var ps_final = ps_lines.join('\n') + '\n' + ps_main;
        return [vs_final, ps_final];
    };
    ShaderPreprocessor.getFunctionPosIndex = function (text, entry) {
        var startindex = text.indexOf(entry);
        var len = entry.length + startindex;
        var subtext = text.substr(len);
        var slen = subtext.length;
        var endpos = -1;
        var t = 0;
        for (var i = 0; i < slen; i++) {
            var c = subtext[i];
            if (c === '{') {
                t++;
            }
            else if (c === '}') {
                t--;
                if (t == 0) {
                    endpos = i;
                    break;
                }
            }
        }
        if (endpos == -1)
            throw new Error('invalid function');
        return [startindex, len + endpos + 1];
    };
    ShaderPreprocessor.Seperate = function (str, spos, epos) {
        var f = str.substr(0, spos);
        var c = str.substr(spos, epos - spos);
        var e = str.substr(epos);
        return [f + e, c];
    };
    ShaderPreprocessor.TrimEmptyLine = function (str) {
        return str.replace(/^([\s]*)$/gm, '');
    };
    /**
     * Process shader source #include macros
     * @param line
     * @param variants
     * @returns line:string variantName:string
     */
    ShaderPreprocessor.processSourceInclude = function (line, variants) {
        var match = /#include ([\w]+)/;
        var matchInc = line.match(match);
        if (matchInc != null) {
            var vname = matchInc[1];
            var variant = variants[vname];
            if (variant == null) {
                throw new Error("shader variant [" + vname + "] not found!");
            }
            if (!variant.linked)
                throw new Error("shader variant [" + vname + "] not linked!");
            return [variant.sources, vname];
        }
        return null;
    };
    ShaderPreprocessor.processOptions = function (line) {
        var match = ShaderPreprocessor.REGEX_OPTIONS;
        if (!line.match(match)) {
            return null;
        }
        var linet = line.trim();
        var options = linet.substr(8);
        var parts = options.split(' ');
        if (parts == null || parts.length == 0)
            throw new Error("invalid #options " + line);
        var validopts = [];
        for (var i = 0, len = parts.length; i < len; i++) {
            var item = parts[i].trim();
            if (item == '')
                continue;
            validopts.push(item);
        }
        var validlen = validopts.length;
        if (validlen == 0 || validlen == 1)
            throw new Error("invalid #options " + line);
        if (validlen == 2) {
            var val = validopts[1];
            if (val != 'ON' && val != 'OFF')
                throw new Error("invalid #options " + line);
            var opt = new ShaderVariant_1.ShaderOptions();
            opt.flag = validopts[0];
            opt.default = val;
            opt.values = ['ON', 'OFF'];
            return ['//' + line, opt];
        }
        else {
            var opt = new ShaderVariant_1.ShaderOptions();
            opt.flag = validopts[0];
            opt.default = validopts[1];
            opt.values = validopts.slice(1);
            return ['//' + line, opt];
        }
    };
    ShaderPreprocessor.REGEX_INCLUDE = /#include ([\w]+)/;
    ShaderPreprocessor.REGEX_OPTIONS = /^[\s]*#options/;
    return ShaderPreprocessor;
}());
exports.ShaderPreprocessor = ShaderPreprocessor;
