"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ShaderGen = /** @class */ (function () {
    function ShaderGen() {
    }
    ShaderGen.SHADERFX_BASIS = "#define PI 3.1415926\n#define PI_2 6.2831852\n#define PI_HALF 1.5707963\n#define EPSILON 1e-5\n\nuniform UNIFORM_OBJ{\n    mat4 _obj2world_;\n};\n#define MATRIX_M _obj2world_\nuniform UNIFORM_BASIS{\n    //basic region\n    vec4 _screenparam_;//[width,height,1/wdith,1/height]\n    highp vec4 _time_;//[Time,deltaTime,sinTime,cosTime]\n    //camera\n    vec4 _camera_pos_;\n    mat4 _camera_mtx_view_;\n    vec4 _camera_projparam_;//[near,far,1/near,1/far]\n    mat4 _camera_mtx_proj_;\n    mat4 _camera_mtx_invproj_;\n    //Ambient And Fog\n    lowp vec4 _ambientcolor_;\n    vec4 _fogcolor_;\n    vec4 _fogparam_;\n};\n#define TIME _time_\n#define SCREEN _screenparam_\n\n#define MATRIX_V _camera_mtx_view_\n#define MATRIX_P _camera_mtx_proj_\n#define MATRIX_VP MATRIX_P * MATRIX_V\n#define MATRIX_MV MATRIX_V * MATRIX_M\n#define MATRIX_IT_MV transpose(inverse(MATRIX_MV))\n#define MATRIX_MVP MATRIX_P * MATRIX_MV\n#define MATRIX_INV_P _camera_mtx_invproj_\n#define MATRIX_WORLD2OBJ inverse(MATRIX_M)\n#define CAMERA_POS _camera_pos_\n#define CAMERA_NEAR _camera_projparam_.x\n#define CAMERA_FAR _camera_projparam_.y\n#define CAMERA_NEAR_INV _camera_projparam_.z\n#define CAMERA_FAR_INV _camera_projparam_.w\n#define SCREEN_WIDTH _screenparam_.x\n#define SCREEN_HEIGHT _screenparam_.y\n#define SCREEN_WIDTH_INV _screenparam_.z\n#define SCREEN_HEIGHT_INV _screenparam_.w\nvec3 ObjToWorldDir(in vec3 dir){\n    return normalize(dir * mat3(MATRIX_WORLD2OBJ));\n}\nfloat SAMPLE_DEPTH_TEXTURE(sampler2D depthtex,vec2 uv){\n    return texture(depthtex,uv).r;\n}\nfloat DECODE_VIEWDEPTH(float d){\n    return 1.0/ ((CAMERA_NEAR_INV - CAMERA_FAR_INV) * d  - CAMERA_NEAR_INV);\n}\nvec4 ClipToWorld(in vec4 clippoint){\n    return inverse(MATRIX_VP) * clippoint;\n}\n\n\n#define saturate(x) clamp(x,0.0,1.0)\n";
    ShaderGen.SHADERFX_LIGHT = "struct LIGHT_DATA{\n    vec4 pos_type;\n    vec4 col_intensity;\n};\nuniform UNIFORM_LIGHT{\n    LIGHT_DATA light_source[4];\n    vec4 ambient_color;\n};\n#define LIGHT_COLOR0 light_source[0].col_intensity.xyz\n#define LIGHT_COLOR1 light_source[1].col_intensity.xyz\n#define LIGHT_COLOR2 light_source[2].col_intensity.xyz\n#define LIGHT_COLOR3 light_source[3].col_intensity.xyz\n\n#define LIGHT_INTENSITY0 light_source[0].col_intensity.w\n#define LIGHT_INTENSITY1 light_source[1].col_intensity.w\n#define LIGHT_INTENSITY2 light_source[2].col_intensity.w\n#define LIGHT_INTENSITY3 light_source[3].col_intensity.w\n\n#define LIGHT_DIR0 light_source[0].pos_type.xyz\n#define LIGHT_DIR1 light_source[1].pos_type.xyz";
    ShaderGen.SHADERFX_LIGHTING = "vec3 LightModel_Lambert(vec3 lightdir,vec3 lightColor,vec3 normal,vec3 albedo){\n    float diff = max(.0,dot(-lightdir,normal));\n    return albedo * lightColor * diff;\n}";
    ShaderGen.SHADERFX_SHADOWMAP = "#options SMCASCADE NONE TWO FOUR\n#options SHADOW ON OFF\n\n#ifdef SHADOW_ON\n\nuniform UNIFORM_SHADOWMAP{\n    mat4 uLightMtx[4];\n    float uShadowDist;\n};\nuniform sampler2D uShadowMap;\n\nfloat computeShadow(vec4 vLightPos,sampler2D shadowsampler){\n    vec3 clipspace = vLightPos.xyz / vLightPos.w;\n    clipspace = clipspace *0.5 + 0.5;\n    float shadowDep = texture(shadowsampler,vec2(clipspace.xy)).x;\n    \n    //fix shadowmpa edge clamp\n    vec2 border = step(clipspace.xy,vec2(0.002));\n    border += step(vec2(0.998),clipspace.xy);\n    shadowDep += (border.x + border.y);\n\n    return step(clipspace.z- 0.01,shadowDep);\n}\n\nfloat computeShadowPoisson(vec4 vLightPos,sampler2D shadowsampler){\n    vec3 clipspace = vLightPos.xyz / vLightPos.w;\n    clipspace = clipspace *0.5 + 0.5;\n\n    vec2 coord = clipspace.xy;\n    float curdepth = clipspace.z - 0.01;\n    float visibility = 1.0;\n\n    float mapsize = 1.0/1024.0;\n\n    vec2 poissonDisk[4];\n        poissonDisk[0] = vec2(-0.94201624, -0.39906216);\n        poissonDisk[1] = vec2(0.94558609, -0.76890725);\n        poissonDisk[2] = vec2(-0.094184101, -0.92938870);\n        poissonDisk[3] = vec2(0.34495938, 0.29387760);\n\n    if(texture(shadowsampler,coord + poissonDisk[0] * mapsize).r <curdepth) visibility -=0.25;\n    if(texture(shadowsampler,coord + poissonDisk[1] * mapsize).r <curdepth) visibility -=0.25;\n    if(texture(shadowsampler,coord + poissonDisk[2] * mapsize).r <curdepth) visibility -=0.25;\n    if(texture(shadowsampler,coord + poissonDisk[3] * mapsize).r <curdepth) visibility -=0.25;\n    return visibility;\n}\n\nfloat computeShadowPCF3(vec4 vLightPos,sampler2DShadow shadowsampler){\n    vec3 clipspace = vLightPos.xyz / vLightPos.w;\n    clipspace = clipspace *0.5 + 0.5;\n\n    vec2 shadowMapSizeInv = vec2(1024.0,1.0/1024.0);\n\n    float curdepth = clipspace.z;\n\n    vec2 uv = clipspace.xy *shadowMapSizeInv.x;\n    uv += 0.5;\n    vec2 st = fract(uv);\n    vec2 base_uv = floor(uv) - 0.5;\n    base_uv *= shadowMapSizeInv.y;\n\n    vec2 uvw0 = 3. - 2. * st;\n    vec2 uvw1 = 1. + 2. * st;\n    vec2 u = vec2((2. - st.x) / uvw0.x - 1., st.x / uvw1.x + 1.) * shadowMapSizeInv.y;\n    vec2 v = vec2((2. - st.y) / uvw0.y - 1., st.y / uvw1.y + 1.) * shadowMapSizeInv.y;\n\n    float shadow = 0.;\n    shadow += uvw0.x * uvw0.y * texture(shadowsampler, vec3(base_uv.xy + vec2(u[0], v[0]), curdepth));\n    shadow += uvw1.x * uvw0.y * texture(shadowsampler, vec3(base_uv.xy + vec2(u[1], v[0]), curdepth));\n    shadow += uvw0.x * uvw1.y * texture(shadowsampler, vec3(base_uv.xy + vec2(u[0], v[1]), curdepth));\n    shadow += uvw1.x * uvw1.y * texture(shadowsampler, vec3(base_uv.xy + vec2(u[1], v[1]), curdepth));\n    shadow = shadow / 16.;\n    return shadow;\n}\n\n#endif";
    ShaderGen.blit = "#version 300 es\n\nprecision mediump float;\n#queue opaque\ninout vec2 vUV;\n#pragma vs vertex\nin vec4 aPosition;\nin vec2 aUV;\nvoid vertex(){\n    vec4 pos = aPosition;\n    pos.xy *=2.0;\n    vUV = vec2(aUV.x,1.0 -aUV.y);\n    gl_Position = pos;\n}\n#pragma ps fragment\nuniform sampler2D uSampler;\nout vec4 fragColor;\nvoid fragment(){\n    fragColor = texture(uSampler,vUV);\n}";
    ShaderGen.depth = "#version 300 es\nprecision mediump float;\n#include SHADERFX_BASIS\n#queue other\n#pragma vs vertex\n#pragma ps fragment\n\nin vec4 aPosition;\nvoid vertex(){\n    gl_Position = MATRIX_MVP * aPosition;\n}\n\nvoid fragment(){\n}";
    ShaderGen.diffuse = "#version 300 es\nprecision mediump float;\n#include SHADERFX_BASIS\n#include SHADERFX_LIGHT\n#include SHADERFX_LIGHTING\n\n#queue opaque\n#pragma vs vertex\n#pragma ps fragment\n\nin vec4 aPosition;\nin vec2 aUV;\nin vec4 aNormal;\n\nstruct V2F{\n    vec3 pos;\n    vec3 normal;\n};\ninout V2F v2f;\n\nvoid vertex(){\n    vec4 wpos = MATRIX_M * aPosition;\n    v2f.pos = wpos.xyz;\n    vec4 pos = MATRIX_VP * wpos;\n    gl_Position = pos;\n    v2f.normal = ObjToWorldDir(aNormal.xyz);\n}\n\nout lowp vec4 fragColor;\nuniform vec4 uColor;\nvoid fragment(){\n    vec3 lcolor = LightModel_Lambert(LIGHT_DIR0,LIGHT_COLOR0,v2f.normal,uColor.xyz);\n    fragColor = vec4(lcolor +0.1,1.0);\n}";
    ShaderGen.gizmos = "#version 300 es\n\nprecision mediump float;\n#include SHADERFX_BASIS\n#queue other\n#pragma vs vertex\n#pragma ps fragment\n\nin vec4 aPosition;\n\nvoid vertex(){\n    vec4 vpos = aPosition;\n    gl_Position = MATRIX_MVP * vpos;\n}\n\nout vec4 fragColor;\nvoid fragment(){\n    fragColor = vec4(1.0);\n}";
    ShaderGen.pbrMetallicRoughness = "#version 300 es\n\nprecision mediump float;\n#include SHADERFX_BASIS\n#include SHADERFX_LIGHT\n#include SHADERFX_SHADOWMAP\n#queue opaque\n\nstruct V2F{\n    vec4 wpos;\n    vec2 uv;\n    vec3 normal;\n    #ifdef SHADOW_ON\n    vec4 lpos;\n    #endif\n};\n\ninout V2F v2f;\n\n#pragma vs vertex\n\nin vec3 aPosition;\nin vec3 aNormal;\nin vec2 aUV;\n\nvoid vertex(){\n\n    vec4 wpos = MATRIX_M * vec4(aPosition,1.0);\n    v2f.wpos = wpos;\n    #ifdef SHADOW_ON\n    v2f.lpos = uLightMtx[0] * wpos;\n    #endif\n    v2f.normal = ObjToWorldDir(aNormal.xyz);\n    gl_Position = MATRIX_VP * wpos;\n    v2f.uv = aUV;\n}\n\n#pragma ps fragment\n\nuniform uPBR{\n    vec4 uColor;\n    float uMetallic;\n    float uRoughness;\n    float uEmissive;\n};\n\nuniform sampler2D uSampler;\nuniform sampler2D uTexMetallicRoughness;\nuniform sampler2D uTexEmissive;\n\nout vec4 fragColor;\n\nvec3 diffuse(vec3 diffcolor){\n    return diffcolor / PI;\n}\n\nvec3 specularReflection(float metallic,float vdoth){\n    return metallic + (vec3(1.0) - metallic) * pow(1.0- vdoth,5.0);\n}\n\nfloat microfacetDistribution(float alphaRoughness,float NdotH)\n{\n    float roughnessSq = alphaRoughness * alphaRoughness;\n    float f = (NdotH * roughnessSq - NdotH) * NdotH + 1.0;\n    return roughnessSq / (PI * f * f);\n}\n\nfloat geometricOcclusion(float NdotV, float NdotH,float VdotH,float NdotL)\n{\n    return min(min(2.0 * NdotV * NdotH / VdotH, 2.0 * NdotL * NdotH / VdotH), 1.0);\n}\n\n\nvoid fragment(){\n\n    vec2 uv = v2f.uv;\n    vec3 wpos = v2f.wpos.xyz;\n    vec3 diffColor = texture(uSampler,uv).xyz;\n\n    vec3 n = v2f.normal;\n    vec3 v = normalize(CAMERA_POS.xyz - wpos);\n    vec3 l = -LIGHT_DIR0;\n    vec3 h = normalize(v+l);\n\n    float roughness = uRoughness;\n    float metallic = uMetallic;\n\n    float NdotL = clamp(dot(n, l), 0.001, 1.0);\n    float NdotV = clamp(abs(dot(n, v)), 0.001, 1.0);\n    float NdotH = clamp(dot(n, h), 0.0, 1.0);\n    //float LdotH = clamp(dot(l, h), 0.0, 1.0);\n    float VdotH = clamp(dot(v, h), 0.0, 1.0);\n\n    vec3 F = specularReflection(metallic,VdotH);\n    float D = microfacetDistribution(roughness,NdotH);\n    float G = geometricOcclusion(NdotV,NdotH,VdotH,NdotL);\n\n    vec3 diff = (vec3(1.0) - F) * diffColor;\n    vec3 spec = F  * (G * D / (4.0 * NdotL * NdotV));\n\n    vec3 col = NdotL * LIGHT_COLOR0 * (diff + spec);\n\n    vec3 ambient = ambient_color.a * ambient_color.rgb;\n\n    col += ambient;\n\n    #ifdef SHADOW_ON\n    float shadow = computeShadow(v2f.lpos,uShadowMap);\n    shadow = clamp(shadow+0.5,0.0,1.0);\n    col = mix(min(col,ambient),col,shadow);\n    #endif\n\n    fragColor = vec4(col *1.2,1.0);\n\n}";
    ShaderGen.shadowmap = "#version 300 es\n\nprecision mediump float;\n#include SHADERFX_BASIS\n#queue other\n#pragma vs vertex\n#pragma ps fragment\n\nuniform mat4 uLightVP;\n\nin vec4 aPosition;\nvoid vertex(){\n    gl_Position = (uLightVP * MATRIX_M) * aPosition;\n}\n\nvoid fragment(){\n}";
    ShaderGen.shadowsGather = "#version 300 es\n\nprecision mediump float;\n#include SHADERFX_BASIS\n#include SHADERFX_SHADOWMAP\n\ninout vec2 vUV;\ninout vec3 vvdir;\n\n#pragma vs vertex\n\nin vec4 aPosition;\nvoid vertex(){\n    vec4 pos = aPosition;\n    vUV = pos.xy +0.5;\n    pos.xy *=2.0;\n\n    vec4 clippos = vec4(pos.xy *2.0,1.0,1.0);\n    vec4 cwpos = ClipToWorld(clippos);\n\n    vvdir = (cwpos.xyz / cwpos.w) - CAMERA_POS.xyz;\n    \n    gl_Position = pos;\n}\n\n#pragma ps fragment\n\nout vec4 fragColor;\nuniform sampler2D uDepthTexure;\nvoid fragment(){\n    float eyedepth = DECODE_VIEWDEPTH(SAMPLE_DEPTH_TEXTURE(uDepthTexure,vUV));\n    vec3 dir = normalize(vvdir);\n    vec3 wpos = dir * eyedepth + CAMERA_POS.xyz;\n    vec4 lpos = uLightMtx[0] * vec4(wpos,1.0);\n    vec3 lcpos = lpos.xyz / lpos.w;\n    lcpos = lpos.xyz *0.5 +0.5;\n    vec2 coord=  lcpos.xy;\n    float shadowDep = texture(uShadowMap,coord).x;\n    float d = shadowDep;// lcpos.z;\n    fragColor = vec4(lcpos.z -1.0,0,0,1.0);\n}";
    ShaderGen.skybox = "#version 300 es\n\nprecision mediump float;\n#include SHADERFX_BASIS\n#options ENVMAP_TYPE CUBE TEX PCG\n#queue skybox\n\ninout vec3 vWorldDir;\n\n#ifdef ENVMAP_TYPE_CUBE\nuniform samplerCube uSampler;\n#elif defined(ENVMAP_TYPE_TEX)\nuniform sampler2D uSampler;\n#elif defined(ENVMAP_TYPE_PCG)\n\n#define SAMPLES_NUMS 16\n// https://www.shadertoy.com/view/XlBfRD\n// License (MIT) Copyright (C) 2017-2018 Rui. All rights reserved.\nstruct ScatteringParams\n{\n    float sunRadius;\n\tfloat sunRadiance;\n\n\tfloat mieG;\n\tfloat mieHeight;\n\n\tfloat rayleighHeight;\n\n\tvec3 waveLambdaMie;\n\tvec3 waveLambdaOzone;\n\tvec3 waveLambdaRayleigh;\n\n\tfloat earthRadius;\n\tfloat earthAtmTopRadius;\n\tvec3 earthCenter;\n};\n\nvec3 ComputeSphereNormal(vec2 coord, float phiStart, float phiLength, float thetaStart, float thetaLength)\n{\n\tvec3 normal;\n\tnormal.x = -sin(thetaStart + coord.y * thetaLength) * sin(phiStart + coord.x * phiLength);\n\tnormal.y = -cos(thetaStart + coord.y * thetaLength);\n\tnormal.z = -sin(thetaStart + coord.y * thetaLength) * cos(phiStart + coord.x * phiLength);\n\treturn normalize(normal);\n}\n\nvec2 ComputeRaySphereIntersection(vec3 position, vec3 dir, vec3 center, float radius)\n{\n\tvec3 origin = position - center;\n\tfloat B = dot(origin, dir);\n\tfloat C = dot(origin, origin) - radius * radius;\n\tfloat D = B * B - C;\n\n\tvec2 minimaxIntersections;\n\tif (D < 0.0)\n\t{\n\t\tminimaxIntersections = vec2(-1.0, -1.0);\n\t}\n\telse\n\t{\n\t\tD = sqrt(D);\n\t\tminimaxIntersections = vec2(-B - D, -B + D);\n\t}\n\n\treturn minimaxIntersections;\n}\n\nvec3 ComputeWaveLambdaRayleigh(vec3 lambda)\n{\n\tconst float n = 1.0003;\n\tconst float N = 2.545E25;\n\tconst float pn = 0.035;\n\tconst float n2 = n * n;\n\tconst float pi3 = PI * PI * PI;\n\tconst float rayleighConst = (8.0 * pi3 * pow(n2 - 1.0,2.0)) / (3.0 * N) * ((6.0 + 3.0 * pn) / (6.0 - 7.0 * pn));\n\treturn rayleighConst / (lambda * lambda * lambda * lambda);\n}\n\nfloat ComputePhaseMie(float theta, float g)\n{\n\tfloat g2 = g * g;\n\treturn (1.0 - g2) / pow(1.0 + g2 - 2.0 * g * saturate(theta), 1.5) / (4.0 * PI);\n}\n\nfloat ComputePhaseRayleigh(float theta)\n{\n\tfloat theta2 = theta * theta;\n\treturn (theta2 * 0.75 + 0.75) / (4.0 * PI);\n}\n\nfloat ChapmanApproximation(float X, float h, float cosZenith)\n{\n\tfloat c = sqrt(X + h);\n\tfloat c_exp_h = c * exp(-h);\n\n\tif (cosZenith >= 0.0)\n\t{\n\t\treturn c_exp_h / (c * cosZenith + 1.0);\n\t}\n\telse\n\t{\n\t\tfloat x0 = sqrt(1.0 - cosZenith * cosZenith) * (X + h);\n\t\tfloat c0 = sqrt(x0);\n\n\t\treturn 2.0 * c0 * exp(X - x0) - c_exp_h / (1.0 - c * cosZenith);\n\t}\n}\n\nfloat GetOpticalDepthSchueler(float h, float H, float earthRadius, float cosZenith)\n{\n\treturn H * ChapmanApproximation(earthRadius / H, h / H, cosZenith);\n}\n\nvec3 GetTransmittance(ScatteringParams setting, vec3 L, vec3 V)\n{\n\tfloat ch = GetOpticalDepthSchueler(L.y, setting.rayleighHeight, setting.earthRadius, V.y);\n\treturn exp(-(setting.waveLambdaMie + setting.waveLambdaRayleigh) * ch);\n}\n\nvec2 ComputeOpticalDepth(ScatteringParams setting, vec3 samplePoint, vec3 V, vec3 L, float neg)\n{\n\tfloat rl = length(samplePoint);\n\tfloat h = rl - setting.earthRadius;\n\tvec3 r = samplePoint / rl;\n\n\tfloat cos_chi_sun = dot(r, L);\n\tfloat cos_chi_ray = dot(r, V * neg);\n\n\tfloat opticalDepthSun = GetOpticalDepthSchueler(h, setting.rayleighHeight, setting.earthRadius, cos_chi_sun);\n\tfloat opticalDepthCamera = GetOpticalDepthSchueler(h, setting.rayleighHeight, setting.earthRadius, cos_chi_ray) * neg;\n\n\treturn vec2(opticalDepthSun, opticalDepthCamera);\n}\n\nvoid AerialPerspective(ScatteringParams setting, vec3 start, vec3 end, vec3 V, vec3 L, bool infinite, out vec3 transmittance, out vec3 insctrMie, out vec3 insctrRayleigh)\n{\n\tfloat inf_neg = infinite ? 1.0 : -1.0;\n\n\tvec3 sampleStep = (end - start) / float(SAMPLES_NUMS);\n\tvec3 samplePoint = end - sampleStep;\n\tvec3 sampleLambda = setting.waveLambdaMie + setting.waveLambdaRayleigh + setting.waveLambdaOzone;\n\n\tfloat sampleLength = length(sampleStep);\n\n\tvec3 scattering = vec3(0.0);\n\tvec2 lastOpticalDepth = ComputeOpticalDepth(setting, end, V, L, inf_neg);\n\n\tfor (int i = 1; i < SAMPLES_NUMS; i++, samplePoint -= sampleStep)\n\t{\n\t\tvec2 opticalDepth = ComputeOpticalDepth(setting, samplePoint, V, L, inf_neg);\n\n\t\tvec3 segment_s = exp(-sampleLambda * (opticalDepth.x + lastOpticalDepth.x));\n\t\tvec3 segment_t = exp(-sampleLambda * (opticalDepth.y - lastOpticalDepth.y));\n\t\t\n\t\ttransmittance *= segment_t;\n\t\t\n\t\tscattering = scattering * segment_t;\n\t\tscattering += exp(-(length(samplePoint) - setting.earthRadius) / setting.rayleighHeight) * segment_s;\n\n\t\tlastOpticalDepth = opticalDepth;\n\t}\n\n\tinsctrMie = scattering * setting.waveLambdaMie * sampleLength;\n\tinsctrRayleigh = scattering * setting.waveLambdaRayleigh * sampleLength;\n}\n\nfloat ComputeSkyboxChapman(ScatteringParams setting, vec3 eye, vec3 V, vec3 L, out vec3 transmittance, out vec3 insctrMie, out vec3 insctrRayleigh)\n{\n\tbool neg = true;\n\n\tvec2 outerIntersections = ComputeRaySphereIntersection(eye, V, setting.earthCenter, setting.earthAtmTopRadius);\n\tif (outerIntersections.y < 0.0) return 0.0;\n\n\tvec2 innerIntersections = ComputeRaySphereIntersection(eye, V, setting.earthCenter, setting.earthRadius);\n\tif (innerIntersections.x > 0.0)\n\t{\n\t\tneg = false;\n\t\touterIntersections.y = innerIntersections.x;\n\t}\n\n\teye -= setting.earthCenter;\n\n\tvec3 start = eye + V * max(0.0, outerIntersections.x);\n\tvec3 end = eye + V * outerIntersections.y;\n\n\tAerialPerspective(setting, start, end, V, L, neg, transmittance, insctrMie, insctrRayleigh);\n\n\tbool intersectionTest = innerIntersections.x < 0.0 && innerIntersections.y < 0.0;\n\treturn intersectionTest ? 1.0 : 0.0;\n}\n\nvec4 ComputeSkyInscattering(ScatteringParams setting, vec3 eye, vec3 V, vec3 L)\n{\n\tvec3 insctrMie = vec3(0.0);\n\tvec3 insctrRayleigh = vec3(0.0);\n\tvec3 insctrOpticalLength = vec3(1.0);\n\tfloat intersectionTest = ComputeSkyboxChapman(setting, eye, V, L, insctrOpticalLength, insctrMie, insctrRayleigh);\n\n\tfloat phaseTheta = dot(V, L);\n\tfloat phaseMie = ComputePhaseMie(phaseTheta, setting.mieG);\n\tfloat phaseRayleigh = ComputePhaseRayleigh(phaseTheta);\n\tfloat phaseNight = 1.0 - saturate(insctrOpticalLength.x * EPSILON);\n\n\tvec3 insctrTotalMie = insctrMie * phaseMie;\n\tvec3 insctrTotalRayleigh = insctrRayleigh * phaseRayleigh;\n\n\tvec3 sky = (insctrTotalMie + insctrTotalRayleigh) * setting.sunRadiance;\n\n\tfloat angle = saturate((1.0 - phaseTheta) * setting.sunRadius);\n\tfloat cosAngle = cos(angle * PI * 0.5);\n\tfloat edge = ((angle >= 0.9) ? smoothstep(0.9, 1.0, angle) : 0.0);\n                         \n\tvec3 limbDarkening = GetTransmittance(setting, -L, V);\n\tlimbDarkening *= pow(vec3(cosAngle), vec3(0.420, 0.503, 0.652)) * mix(vec3(1.0), vec3(1.2,0.9,0.5), edge) * intersectionTest;\n\n\tsky += limbDarkening;\n\n\treturn vec4(sky, phaseNight * intersectionTest);\n}\n\nvec3 TonemapACES(vec3 x)\n{\n\tconst float A = 2.51f;\n\tconst float B = 0.03f;\n\tconst float C = 2.43f;\n\tconst float D = 0.59f;\n\tconst float E = 0.14f;\n\treturn (x * (A * x + B)) / (x * (C * x + D) + E);\n}\n\nfloat noise(vec2 uv)\n{\n\treturn fract(dot(sin(uv.xyx * uv.xyy * 1024.0), vec3(341896.483, 891618.637, 602649.7031)));\n}\n\n#endif\n\n#pragma vs vertex\n#ifdef SHADER_VS\nin vec4 aPosition;\nvoid vertex(){\n    vec4 pos = aPosition;\n    pos.xy*=2.0;\n    pos.z = 1.0;\n    gl_Position = pos;\n\n    vec4 wpos =  inverse(MATRIX_VP) * pos;\n    wpos.xyz = wpos.xyz / wpos.w - CAMERA_POS.xyz;\n    vWorldDir = wpos.xyz;\n    \n}\n#endif\n\n#pragma ps fragment\n#ifdef SHADER_PS\nout lowp vec4 fragColor;\nvoid fragment(){\n    vec3 dir = vWorldDir.xyz;\n    #ifdef ENVMAP_TYPE_CUBE\n    fragColor = texture(uSampler,dir);\n    #elif defined(ENVMAP_TYPE_TEX)\n    dir = normalize(dir);\n    float y = 1.0 - 0.5 *(1.0 + dir.y);\n    float x = atan(dir.z,dir.x) / PI_2 + 0.5;\n    fragColor = texture(uSampler,vec2(x,y));\n    #elif defined(ENVMAP_TYPE_PCG)\n\n    vec3 V = normalize(dir);\n    vec3 L = normalize(vec3(0,0.5,0.5));\n\n    ScatteringParams setting;\n\tsetting.sunRadius = 500.0;\n\tsetting.sunRadiance = 20.0;\n\tsetting.mieG = 0.76;\n\tsetting.mieHeight = 1200.0;\n\tsetting.rayleighHeight = 8000.0;\n\tsetting.earthRadius = 6360000.0;\n\tsetting.earthAtmTopRadius = 6420000.0;\n\tsetting.earthCenter = vec3(0, -setting.earthRadius, 0);\n\tsetting.waveLambdaMie = vec3(2e-7);\n    // wavelength with 680nm, 550nm, 450nm\n    setting.waveLambdaRayleigh = ComputeWaveLambdaRayleigh(vec3(680e-9, 550e-9, 450e-9));\n    \n    // see https://www.shadertoy.com/view/MllBR2\n\tsetting.waveLambdaOzone = vec3(1.36820899679147, 3.31405330400124, 0.13601728252538) * 0.6e-6 * 2.504;\n\t\n    vec3 eye = vec3(0,1000.0,0);\n   \tvec4 sky = ComputeSkyInscattering(setting, eye, V, L);\n    sky.rgb = TonemapACES(sky.rgb * 2.0);\n    sky.rgb = pow(sky.rgb, vec3(1.0 / 2.2)); // gamma\n    sky.rgb += noise(dir.xy * TIME.z) / 255.0; // dither\n\tfragColor = vec4(sky.rgb, 1.0);\n    #endif\n}\n#endif\n";
    ShaderGen.sprite = "#version 300 es\n\nprecision mediump float;\n#include SHADERFX_BASIS\n\n#queue transparent\n#zwrite off\n#blend src_alpha one_minus_src_alpha\n\ninout vec2 vUV;\n\n#pragma vs vertex\n#ifdef SHADER_VS\nin vec4 aPosition;\nin vec2 aUV;\n\nvoid vertex(){\n    gl_Position = MATRIX_MVP * aPosition;\n    vUV = aUV;\n}\n#endif\n\n#pragma ps fragment\n#ifdef SHADER_PS\nuniform sampler2D uSampler;\nuniform vec4 uColor;\nout vec4 fragColor;\nvoid fragment(){\n    fragColor = texture(uSampler,vUV) * uColor;\n}\n#endif\n";
    ShaderGen.UnlitColor = "#version 300 es\nprecision mediump float;\n#include SHADERFX_BASIS\n#queue opaque\n\n#pragma vs vertex\nin vec4 aPosition;\nvoid vertex(){\n    gl_Position = MATRIX_MVP * aPosition;\n}\n\n#pragma ps fragment\nuniform vec4 uColor;\nout vec4 fragColor;\nvoid fragment(){\n    fragColor = uColor;\n}";
    ShaderGen.UnlitTexture = "#version 300 es\n\nprecision mediump float;\n#include SHADERFX_BASIS\n\n#queue opaque\n\ninout vec2 vUV;\n\n\n#pragma vs vertex\nin vec4 aPosition;\nin vec2 aUV;\nvoid vertex(){\n    gl_Position = MATRIX_MVP * aPosition;\n    vUV = aUV;\n}\n\n#pragma ps fragment\n\nuniform sampler2D uSampler;\nout vec4 fragColor;\nvoid fragment(){\n    fragColor = texture(uSampler,vUV);\n}";
    ShaderGen.uvValue = "#version 300 es\nprecision mediump float;\n#include SHADERFX_BASIS\n\ninout vec2 vUV;\n\n#pragma vs vertex\nin vec4 aPosition;\nin vec2 aUV;\nvoid vertex(){\n    gl_Position = MATRIX_MVP * aPosition;\n    vUV = aUV;\n}\n\n#pragma ps fragment\nout vec4 fragColor;\nvoid fragment(){\n    fragColor = vec4(vUV,0,1.0);\n}";
    return ShaderGen;
}());
exports.ShaderGen = ShaderGen;
